      SUBROUTINE  ALBTRN( ALBEDO, AMB, APB, ARRAY, B, BDR, CBAND, CC,
     $     CMU, CWT, EVAL, EVECC, GL, GC, GU, IPVT,
     $     KK, LL, NLYR, NN, NSTR, NUMU, PRNT, TAUCPR,
     $     UMU, U0U, WK, YLMC, YLMU, Z, MI, MI9M2,
     $     MAXULV, MAXUMU, MXCMU, MXUMU, NNLYRI,
     $     ALBMED, TRNMED )

C        SPECIAL CASE TO GET ONLY ALBEDO AND TRANSMISSIVITY
C        OF ENTIRE MEDIUM AS A FUNCTION OF INCIDENT BEAM ANGLE
C        (MANY SIMPLIFICATIONS BECAUSE BOUNDARY CONDITION IS JUST
C        ISOTROPIC ILLUMINATION, THERE ARE NO THERMAL SOURCES, AND
C        PARTICULAR SOLUTIONS DO NOT NEED TO BE COMPUTED).  SEE
C        REF. S2 AND REFERENCES THEREIN FOR THEORY.

C        ROUTINES CALLED:  ALTRIN, LEPOLY, PRALTR, SETMTX, SOLVE1,
C                          SOLEIG, ZEROIT

      LOGICAL  PRNT( * )
      INTEGER  NLYR, NUMU, NSTR
      REAL     UMU( * ), U0U( MAXUMU,* )
      
      INTEGER IPVT( * )
      REAL    ALBMED( * ), AMB( MI,* ), APB( MI,*), ARRAY( MXCMU,* ),
     $     B( * ), BDR( MI,0:* ), CBAND( MI9M2,* ), CC( MXCMU,* ),
     $     CMU( * ), CWT( * ), EVAL( * ), EVECC( MXCMU,* ),
     $     GL( 0:MXCMU,* ), GC( MXCMU,MXCMU,* ), GU( MXUMU,MXCMU,* ),
     $     KK( MXCMU,* ), LL( MXCMU,* ), TAUCPR( 0:* ), TRNMED( * ),
     $     WK( * ), YLMC( 0:MXCMU,* ), YLMU( 0:MXCMU,* ), Z( * )
      
      LOGICAL  LAMBER, LYRCUT
        
C                    ** SET DISORT VARIABLES THAT ARE IGNORED IN THIS
C                    ** SPECIAL CASE BUT ARE NEEDED BELOW IN ARGUMENT
C                    ** LISTS OF SUBROUTINES SHARED WITH GENERAL CASE
      NCUT = NLYR
      LYRCUT = .FALSE.
      FISOT = 1.0
      LAMBER = .TRUE.
      
      MAZIM = 0
      DELM0 = 1.0
C                          ** GET LEGENDRE POLYNOMIALS FOR COMPUTATIONAL
C                          ** AND USER POLAR ANGLE COSINES

      CALL  LEPOLY( NUMU, MAZIM, MXCMU, NSTR-1, UMU, YLMU )
      CALL  LEPOLY( NN,   MAZIM, MXCMU, NSTR-1, CMU, YLMC )

C                       ** EVALUATE LEGENDRE POLYNOMIALS WITH NEGATIVE
C                       ** -CMU- FROM THOSE WITH POSITIVE -CMU-;
C                       ** DAVE/ARMSTRONG EQ. (15)
      SGN  = -1.0
      DO  5  L = MAZIM, NSTR-1
         SGN = - SGN
         DO  5  IQ = NN+1, NSTR
            YLMC( L,IQ ) = SGN * YLMC( L,IQ-NN )
 5    CONTINUE
C                                  ** ZERO BOTTOM REFLECTIVITY
C                                  ** (-ALBEDO- IS USED ONLY IN ANALYTIC
C                                  ** FORMULAE INVOLVING ALBEDO = 0
C                                  ** SOLUTIONS; EQS 16-17 OF REF S2)
      CALL  ZEROIT( BDR, MI*(MI+1) )


C ===================  BEGIN LOOP ON COMPUTATIONAL LAYERS  =============

      DO 100  LC = 1, NLYR

C                        ** SOLVE EIGENFUNCTION PROBLEM IN EQ. STWJ(8B)

         CALL  SOLEIG( AMB, APB, ARRAY, CMU, CWT, GL(0,LC), MI, MAZIM,
     $        MXCMU, NN, NSTR, WK, YLMC, CC, EVECC, EVAL,
     $        KK(1,LC), GC(1,1,LC) )
         
C                          ** INTERPOLATE EIGENVECTORS TO USER ANGLES

         CALL  TERPEV( CWT, EVECC, GL(0,LC), GU(1,1,LC), MAZIM, MXCMU,
     $        MXUMU, NN, NSTR, NUMU, WK, YLMC, YLMU )
 100  CONTINUE

C ===================  END LOOP ON COMPUTATIONAL LAYERS  ===============

C                      ** SET COEFFICIENT MATRIX OF EQUATIONS COMBINING
C                      ** BOUNDARY AND LAYER INTERFACE CONDITIONS

      CALL  SETMTX( BDR, CBAND, CMU, CWT, DELM0, GC, KK, LAMBER,
     $     LYRCUT, MI, MI9M2, MXCMU, NCOL, NCUT, NNLYRI,
     $     NN, NSTR, TAUCPR, WK )

      CALL  ZEROIT( U0U, MAXUMU*MAXULV )

      NHOM = 2
      IF( NLYR.EQ.1 )  NHOM = 1
      SPHALB = 0.0
      SPHTRN = 0.0
      DO 200  IHOM = 1, NHOM
C                             ** SOLVE FOR CONSTANTS OF INTEGRATION IN
C                             ** HOMOGENEOUS SOLUTION FOR ILLUMINATION
C                             ** FROM TOP (IHOM=1), THEN BOTTOM (IHOM=2)

         CALL  SOLVE1( B, CBAND, FISOT, IHOM, IPVT, LL, MI9M2, MXCMU,
     $        NCOL, NLYR, NN, NNLYRI, NSTR, Z )
         
C                             ** COMPUTE AZIMUTHALLY-AVERAGED INTENSITY
C                             ** AT USER ANGLES; GIVES ALBEDO IF MULTI-
C                             ** LAYER (EQ. 9 OF REF S2); GIVES BOTH
C                             ** ALBEDO AND TRANSMISSIVITY IF SINGLE
C                             ** LAYER (EQS. 3-4 OF REF S2)

         CALL  ALTRIN( GU, KK, LL, MXCMU, MXUMU, MAXUMU, NLYR,
     $                 NN, NSTR, NUMU, TAUCPR, UMU, U0U, WK )

         IF ( IHOM.EQ.1 )  THEN
C                                   ** SAVE ALBEDOS;  FLIP TRANSMISSIV.
C                                   ** END OVER END TO CORRESPOND TO
C                                   ** POSITIVE -UMU- INST. OF NEGATIVE
            DO 120  IU = 1, NUMU/2
               ALBMED(IU) = U0U( IU + NUMU/2, 1 )
               IF( NLYR.EQ.1 )  TRNMED(IU) = U0U( NUMU/2+1-IU, 2 )
     $              + EXP( - TAUCPR(NLYR) / UMU(IU+NUMU/2) )
120         CONTINUE
C                                    ** GET SPHERICAL ALBEDO AND, FOR 1
C                                    ** LAYER, SPHERICAL TRANSMISSIVITY
            IF( ALBEDO.GT.0.0 )
     $           CALL SPALTR( CMU, CWT, GC, KK, LL, MXCMU, NLYR,
     $           NN, NSTR, TAUCPR, SPHALB, SPHTRN )
            
         ELSE IF ( IHOM.EQ.2 )  THEN
C                                      ** SAVE TRANSMISSIVITIES
            DO 140  IU = 1, NUMU/2
               TRNMED(IU) = U0U( IU + NUMU/2, 1 )
     $              + EXP( - TAUCPR(NLYR) / UMU(IU+NUMU/2) )
140         CONTINUE
C                             ** GET SPHERICAL ALBEDO AND TRANSMISSIVITY
            IF( ALBEDO.GT.0.0 )
     $           CALL SPALTR( CMU, CWT, GC, KK, LL, MXCMU, NLYR,
     $           NN, NSTR, TAUCPR, SPHTRN, SPHALB )
         END IF
200   CONTINUE

      IF ( ALBEDO.GT.0.0 )  THEN
C                                ** REF. S2, EQS. 16-17 (THESE EQS. HAVE
C                                ** A SIMPLE PHYSICAL INTERPRETATION
C                                ** LIKE THAT OF THE DOUBLING EQS.)
         DO 220  IU = 1, NUMU
            ALBMED(IU) = ALBMED(IU) + ( ALBEDO / (1.-ALBEDO*SPHALB) )
     $           * SPHTRN * TRNMED(IU)
            TRNMED(IU) = TRNMED(IU) + ( ALBEDO / (1.-ALBEDO*SPHALB) )
     $           * SPHALB * TRNMED(IU)
220      CONTINUE
      END IF
C                          ** RETURN -UMU- TO ALL POSITIVE VALUES, TO
C                          ** AGREE WITH ORDERING IN -ALBMED,TRNMED-
      NUMU = NUMU / 2
      DO 230  IU = 1, NUMU
         UMU(IU) = UMU(IU+NUMU)
 230  CONTINUE
      IF ( PRNT(6) )  CALL  PRALTR( UMU, NUMU, ALBMED, TRNMED )
      
      RETURN
      END
      SUBROUTINE  ALTRIN( GU, KK, LL, MXCMU, MXUMU, MAXUMU, NLYR,
     $     NN, NSTR, NUMU, TAUCPR, UMU, U0U, WK )

C       COMPUTES AZIMUTHALLY-AVERAGED INTENSITY AT TOP AND BOTTOM
C       OF MEDIUM (RELATED TO ALBEDO AND TRANSMISSION OF MEDIUM BY
C       RECIPROCITY PRINCIPLES;  SEE REF S2).  USER POLAR ANGLES ARE
C       USED AS INCIDENT BEAM ANGLES. (THIS IS A VERY SPECIALIZED
C       VERSION OF 'USRINT')

C       ** NOTE **  USER INPUT VALUES OF -UMU- (ASSUMED POSITIVE) ARE
C                   TEMPORARILY IN UPPER LOCATIONS OF  -UMU-  AND
C                   CORRESPONDING NEGATIVES ARE IN LOWER LOCATIONS
C                   (THIS MAKES -GU- COME OUT RIGHT).  I.E. THE CONTENTS
C                   OF THE TEMPORARY -UMU- ARRAY ARE:

C                     -UMU(NUMU),..., -UMU(1), UMU(1),..., UMU(NUMU)

C   I N P U T    V A R I A B L E S:

C       GU     :  EIGENVECTORS INTERPOLATED TO USER POLAR ANGLES
C                 (i.e., g IN EQ. SC(1) )
C       KK     :  EIGENVALUES OF COEFF. MATRIX IN EQ. SS(7)
C       LL     :  CONSTANTS OF INTEGRATION IN EQ. SC(1), OBTAINED
C                 BY SOLVING SCALED VERSION OF EQ. SC(5);
C                 EXPONENTIAL TERM OF EQ. SC(12) NOT INCLUDED
C       NN     :  ORDER OF DOUBLE-GAUSS QUADRATURE (NSTR/2)
C       TAUCPR :  CUMULATIVE OPTICAL DEPTH (DELTA-M-SCALED)
C       (REMAINDER ARE 'DISORT' INPUT VARIABLES)

C   O U T P U T    V A R I A B L E:

C       U0U  :    DIFFUSE AZIMUTHALLY-AVERAGED INTENSITY AT TOP AND
C                 BOTTOM OF MEDIUM (DIRECTLY TRANSMITTED COMPONENT,
C                 CORRESPONDING TO -BNDINT- IN 'USRINT', IS OMITTED).

C   I N T E R N A L    V A R I A B L E S:

C       DTAU   :  OPTICAL DEPTH OF A COMPUTATIONAL LAYER
C       PALINT :  NON-BOUNDARY-FORCED INTENSITY COMPONENT
C       UTAUPR :  OPTICAL DEPTHS OF USER OUTPUT LEVELS (DELTA-M SCALED)
C       WK     :  SCRATCH VECTOR FOR SAVING 'EXP' EVALUATIONS
C       ALL THE EXPONENTIAL FACTORS (i.e., EXP1, EXPN,... etc.)
C       COME FROM THE SUBSTITUTION OF CONSTANTS OF INTEGRATION IN
C       EQ. SC(12) INTO EQS. S1(8-9).  ALL HAVE NEGATIVE ARGUMENTS.
C+---------------------------------------------------------------------+

      REAL     UTAUPR( 2 )
      REAL     GU( MXUMU,MXCMU,* ), KK( MXCMU,* ), LL( MXCMU,* ), MU,
     $     TAUCPR( 0:* ), UMU(*), U0U( MAXUMU,* ), WK(*)


      UTAUPR(1) = 0.0
      UTAUPR(2) = TAUCPR( NLYR )
      DO 100  LU = 1, 2
         IF ( LU.EQ.1 )  THEN
            IUMIN = NUMU / 2 + 1
            IUMAX = NUMU
            SGN = 1.0
         ELSE
            IUMIN = 1
            IUMAX = NUMU / 2
            SGN = - 1.0
         END IF
C                                   ** LOOP OVER POLAR ANGLES AT WHICH
C                                   ** ALBEDOS/TRANSMISSIVITIES DESIRED
C                                   ** ( UPWARD ANGLES AT TOP BOUNDARY,
C                                   ** DOWNWARD ANGLES AT BOTTOM )
         DO 50  IU = IUMIN, IUMAX
            MU = UMU(IU)
C                                     ** INTEGRATE FROM TOP TO BOTTOM
C                                     ** COMPUTATIONAL LAYER
            PALINT = 0.0
            DO 30  LC = 1, NLYR

               DTAU = TAUCPR(LC) - TAUCPR(LC-1)
               EXP1 =  EXP( (UTAUPR(LU) - TAUCPR(LC-1)) / MU )
               EXP2 =  EXP( (UTAUPR(LU) - TAUCPR( LC )) / MU )

C                                      ** -KK- IS NEGATIVE
               DO 20  IQ = 1, NN
                  WK(IQ) = EXP( KK(IQ,LC) * DTAU )
                  DENOM = 1.0 + MU * KK(IQ,LC)
                  IF ( ABS(DENOM).LT.0.0001 ) THEN
C                                                   ** L'HOSPITAL LIMIT
                     EXPN = DTAU / MU * EXP2
                  ELSE
                     EXPN = ( EXP1 * WK(IQ) - EXP2 ) * SGN / DENOM
                  END IF
                  PALINT = PALINT + GU(IU,IQ,LC) * LL(IQ,LC) * EXPN
20             CONTINUE
C                                      ** -KK- IS POSITIVE
               DO 21  IQ = NN+1, NSTR
                  DENOM = 1.0 + MU * KK(IQ,LC)
                  IF ( ABS(DENOM).LT.0.0001 ) THEN
                     EXPN = - DTAU / MU * EXP1
                  ELSE
                     EXPN = ( EXP1 - EXP2 * WK(NSTR+1-IQ) ) *SGN / DENOM
                  END IF
                  PALINT = PALINT + GU(IU,IQ,LC) * LL(IQ,LC) * EXPN
21             CONTINUE

 30         CONTINUE

            U0U( IU, LU ) = PALINT

 50      CONTINUE
100   CONTINUE

      RETURN
      END
      SUBROUTINE  ASYMTX( AA, EVEC, EVAL, M, IA, IEVEC, IER, WK )

C    =======  S I N G L E    P R E C I S I O N    V E R S I O N  ======

C       SOLVES EIGENFUNCTION PROBLEM FOR REAL ASYMMETRIC MATRIX
C       FOR WHICH IT IS KNOWN A PRIORI THAT THE EIGENVALUES ARE REAL.

C           ( C O M M E N T S    O M I T T E D )
C+---------------------------------------------------------------------+

      REAL              AA( IA,* ),  WK(*),  EVAL(*), EVEC( IEVEC,* )
      LOGICAL           NOCONV, NOTLAS
      DATA     C1/ 0.4375 /, C2/ 0.5 /, C3/ 0.75 /, C4/ 0.95 /,
     $     C5/ 16.0 /, C6/ 256.0 /, ZERO / 0.0 /, ONE / 1.0 /


      IER = 0
      TOL = R1MACH(3)
      IF ( M.LT.1 .OR. IA.LT.M .OR. IEVEC.LT.M )
     $     CALL ERRMSG( 'ASYMTX--BAD INPUT VARIABLE(S)', .TRUE. )

C                           ** HANDLE 1X1 AND 2X2 SPECIAL CASES
      IF ( M.EQ.1 )  THEN
         EVAL(1) = AA(1,1)
         EVEC(1,1) = ONE
         RETURN

      ELSE IF ( M.EQ.2 )  THEN
         DISCRI = ( AA(1,1) - AA(2,2) )**2 + 4. * AA(1,2) * AA(2,1)
         IF ( DISCRI.LT.ZERO )
     $        CALL ERRMSG( 'ASYMTX--COMPLEX EVALS IN 2X2 CASE', .TRUE. )
         SGN = ONE
         IF ( AA(1,1).LT.AA(2,2) )  SGN = - ONE
         EVAL(1) = 0.5 * ( AA(1,1) + AA(2,2) + SGN*SQRT(DISCRI) )
         EVAL(2) = 0.5 * ( AA(1,1) + AA(2,2) - SGN*SQRT(DISCRI) )
         EVEC(1,1) = ONE
         EVEC(2,2) = ONE
         IF ( AA(1,1).EQ.AA(2,2) .AND. 
     $        (AA(2,1).EQ.ZERO.OR.AA(1,2).EQ.ZERO) )  THEN
            RNORM =   ABS(AA(1,1)) + ABS(AA(1,2)) + ABS(AA(2,1))
     $           + ABS(AA(2,2))
            W = TOL * RNORM
            EVEC(2,1) =   AA(2,1) / W
            EVEC(1,2) = - AA(1,2) / W
         ELSE
            EVEC(2,1) = AA(2,1) / ( EVAL(1) - AA(2,2) )
            EVEC(1,2) = AA(1,2) / ( EVAL(2) - AA(1,1) )
         ENDIF
         
         RETURN

	END IF
C                                        ** INITIALIZE OUTPUT VARIABLES
	IER = 0
	DO 20 I = 1, M
	   EVAL(I) = ZERO
	   DO 10 J = 1, M
	      EVEC(I,J) = ZERO
10       CONTINUE
         EVEC(I,I) = ONE
20    CONTINUE
C                  ** BALANCE THE INPUT MATRIX AND REDUCE ITS NORM BY
C                  ** DIAGONAL SIMILARITY TRANSFORMATION STORED IN WK;
C                  ** THEN SEARCH FOR ROWS ISOLATING AN EIGENVALUE
C                  ** AND PUSH THEM DOWN
      RNORM = ZERO
      L  = 1
      K  = M

30    KKK = K
      DO 70  J = KKK, 1, -1
         ROW = ZERO
         DO 40 I = 1, K
            IF ( I.NE.J ) ROW = ROW + ABS( AA(J,I) )
 40      CONTINUE
         IF ( ROW.EQ.ZERO ) THEN
            WK(K) = J
            IF ( J.NE.K ) THEN
               DO 50 I = 1, K
                  REPL   = AA(I,J)
                  AA(I,J) = AA(I,K)
                  AA(I,K) = REPL
50             CONTINUE
               DO 60 I = L, M
                  REPL   = AA(J,I)
                  AA(J,I) = AA(K,I)
                  AA(K,I) = REPL
60             CONTINUE
            END IF
            K = K - 1
            GO TO 30
         END IF
70    CONTINUE
C                                     ** SEARCH FOR COLUMNS ISOLATING AN
C                                       ** EIGENVALUE AND PUSH THEM LEFT
 80   LLL = L
      DO 120 J = LLL, K
         COL = ZERO
         DO 90 I = L, K
            IF ( I.NE.J ) COL = COL + ABS( AA(I,J) )
90       CONTINUE
         IF ( COL.EQ.ZERO ) THEN
            WK(L) = J
            IF ( J.NE.L ) THEN
               DO 100 I = 1, K
                  REPL   = AA(I,J)
                  AA(I,J) = AA(I,L)
                  AA(I,L) = REPL
100            CONTINUE
               DO 110 I = L, M
                  REPL   = AA(J,I)
                  AA(J,I) = AA(L,I)
                  AA(L,I) = REPL
110            CONTINUE
            END IF
            L = L + 1
            GO TO 80
         END IF
 120  CONTINUE
C                           ** BALANCE THE SUBMATRIX IN ROWS L THROUGH K
      DO 130 I = L, K
         WK(I) = ONE
130   CONTINUE

140   NOCONV = .FALSE.
      DO 200 I = L, K
         COL = ZERO
         ROW = ZERO
         DO 150 J = L, K
            IF ( J.NE.I ) THEN
               COL = COL + ABS( AA(J,I) )
               ROW = ROW + ABS( AA(I,J) )
            END IF
 150     CONTINUE
         F = ONE
         G = ROW / C5
         H = COL + ROW
 160     IF ( COL.LT.G ) THEN
            F   = F * C5
            COL = COL * C6
            GO TO 160
         END IF
         G = ROW * C5
 170     IF ( COL.GE.G ) THEN
            F   = F / C5
            COL = COL / C6
            GO TO 170
         END IF
C                                                         ** NOW BALANCE
         IF ( (COL+ROW) / F .LT. C4 * H ) THEN
            WK(I)  = WK(I) * F
            NOCONV = .TRUE.
            DO 180 J = L, M
               AA(I,J) = AA(I,J) / F
 180        CONTINUE
            DO 190 J = 1, K
               AA(J,I) = AA(J,I) * F
 190        CONTINUE
         END IF
 200  CONTINUE

      IF ( NOCONV ) GO TO 140
C                                  ** IS -A- ALREADY IN HESSENBERG FORM?
      IF ( K-1.LT.L+1 ) GO TO 350
C                                   ** TRANSFER -A- TO A HESSENBERG FORM
      DO 290 N = L+1, K-1
         H       = ZERO
         WK(N+M) = ZERO
         SCALE   = ZERO
C                                                        ** SCALE COLUMN
         DO 210 I = N, K
            SCALE = SCALE + ABS(AA(I,N-1))
 210     CONTINUE
         IF ( SCALE.NE.ZERO ) THEN
            DO 220 I = K, N, -1
               WK(I+M) = AA(I,N-1) / SCALE
               H = H + WK(I+M) * WK(I+M)
 220        CONTINUE
            G = - SIGN( SQRT(H),WK(N+M) )
            H = H - WK(N+M) * G
            WK(N+M) = WK(N+M) - G
C                                                 ** FORM (I-(U*UT)/H)*A
            DO 250 J = N, M
               F = ZERO
               DO 230  I = K, N, -1
                  F = F + WK(I+M) * AA(I,J)
 230           CONTINUE
               DO 240 I = N, K
                  AA(I,J) = AA(I,J) - WK(I+M) * F / H
 240           CONTINUE
 250        CONTINUE
C                                  ** FORM (I-(U*UT)/H)*A*(I-(U*UT)/H)
            DO 280 I = 1, K
               F = ZERO
               DO 260  J = K, N, -1
                  F = F + WK(J+M) * AA(I,J)
 260           CONTINUE
               DO 270 J = N, K
                  AA(I,J) = AA(I,J) - WK(J+M) * F / H
 270           CONTINUE
 280        CONTINUE
            WK(N+M)  = SCALE * WK(N+M)
            AA(N,N-1) = SCALE * G
         END IF
 290  CONTINUE

      DO 340  N = K-2, L, -1
         N1 = N + 1
         N2 = N + 2
         F  = AA(N+1,N)
         IF ( F.NE.ZERO ) THEN
            F  = F * WK(N+1+M)
            DO 300 I = N+2, K
               WK(I+M) = AA(I,N)
 300        CONTINUE
            IF ( N+1.LE.K ) THEN
               DO 330 J = 1, M
                  G = ZERO
                  DO 310 I = N+1, K
                     G = G + WK(I+M) * EVEC(I,J)
 310              CONTINUE
                  G = G / F
                  DO 320 I = N+1, K
                     EVEC(I,J) = EVEC(I,J) + G * WK(I+M)
 320              CONTINUE
 330           CONTINUE
            END IF
         END IF
 340  CONTINUE

 350  CONTINUE
      N = 1
      DO 370 I = 1, M
         DO 360 J = N, M
            RNORM = RNORM + ABS(AA(I,J))
 360     CONTINUE
         N = I
         IF ( I.LT.L .OR. I.GT.K ) EVAL(I) = AA(I,I)
 370  CONTINUE
      N = K
      T = ZERO
C                            ** SEARCH FOR NEXT EIGENVALUES
 380  IF ( N.LT.L ) GO TO 530
      IN = 0
      N1 = N - 1
      N2 = N - 2
C                        ** LOOK FOR SINGLE SMALL SUB-DIAGONAL ELEMENT
 390  CONTINUE
      DO 400 I = L, N
         LB = N+L - I
         IF ( LB.EQ.L ) GO TO 410
         S = ABS( AA(LB-1,LB-1)) + ABS(AA(LB,LB) )
         IF ( S.EQ.ZERO ) S = RNORM
         IF ( ABS(AA(LB,LB-1)) .LE. TOL * S ) GO TO 410
 400  CONTINUE
C                                                      ** ONE EVAL FOUND
 410  X = AA(N,N)
      IF ( LB.EQ.N  ) THEN
         AA(N,N)  = X + T
         EVAL(N) = AA(N,N)
         N = N1
         GO TO 380
      END IF
C                                                     ** TWO EVALS FOUND
      Y = AA(N1,N1)
      W = AA(N,N1) * AA(N1,N)
      IF ( LB.EQ.N1 ) THEN
         P = (Y-X) * C2
         Q = P * P + W
         Z = SQRT(ABS(Q))
         AA(N,N) = X + T
         X = AA(N,N)
         AA(N1,N1) = Y + T
C                                                           ** REAL PAIR
         Z = P + SIGN(Z,P)
         EVAL(N1) = X + Z
         EVAL(N)  = EVAL(N1)
         IF ( Z.NE.ZERO ) EVAL(N) = X - W / Z
         X = AA(N,N1)
C                                  ** EMPLOY SCALE FACTOR IN CASE
C                                  ** X AND Z ARE VERY SMALL
         R = SQRT(X * X + Z * Z)
         P = X / R
         Q = Z / R
C                                                    ** ROW MODIFICATION
         DO 420 J = N1, M
            Z = AA(N1,J)
            AA(N1,J) = Q * Z + P * AA(N,J)
            AA(N,J)  = Q * AA(N,J) - P * Z
 420     CONTINUE
C                                                 ** COLUMN MODIFICATION
         DO 430 I = 1, N
            Z = AA(I,N1)
            AA(I,N1) = Q * Z + P * AA(I,N)
            AA(I,N)  = Q * AA(I,N) - P * Z
 430     CONTINUE
C                                          ** ACCUMULATE TRANSFORMATIONS
         DO 440 I = L, K
            Z = EVEC(I,N1)
            EVEC(I,N1) = Q * Z + P * EVEC(I,N)
            EVEC(I,N)  = Q * EVEC(I,N) - P * Z
 440     CONTINUE
         N = N2
         GO TO 380
      END IF
C                    ** NO CONVERGENCE AFTER 30 ITERATIONS; SET ERROR
C                    ** INDICATOR TO THE INDEX OF THE CURRENT EIGENVALUE

      IF ( IN.EQ.30 ) THEN
         IER = 128 + N
         RETURN
      END IF
C                                                          ** FORM SHIFT
      IF ( IN.EQ.10 .OR. IN.EQ.20 ) THEN
         T = T + X
         DO 450 I = L, N
            AA(I,I) = AA(I,I) - X
 450     CONTINUE
         S = ABS(AA(N,N1)) + ABS(AA(N1,N2))
         X = C3 * S
         Y = X
         W = -C1 * S * S
      END IF

      IN = IN + 1
C                ** LOOK FOR TWO CONSECUTIVE SMALL SUB-DIAGONAL ELEMENTS

      DO 460 J = LB, N2
         I = N2+LB - J
         Z = AA(I,I)
         R = X - Z
         S = Y - Z
         P = (R * S-W) / AA(I+1,I) + AA(I,I+1)
         Q = AA(I+1,I+1) - Z - R - S
         R = AA(I+2,I+1)
         S = ABS(P) + ABS(Q) + ABS(R)
         P = P / S
         Q = Q / S
         R = R / S
         IF ( I.EQ.LB ) GO TO 470
         UU = ABS( AA(I,I-1) ) * ( ABS(Q) + ABS(R) )
         VV = ABS(P) * ( ABS(AA(I-1,I-1)) + ABS(Z) + ABS(AA(I+1,I+1)) )
         IF ( UU .LE. TOL*VV ) GO TO 470
 460  CONTINUE

 470  CONTINUE
      AA(I+2,I) = ZERO
      DO 480 J = I+3, N
         AA(J,J-2) = ZERO
         AA(J,J-3) = ZERO
 480  CONTINUE

C             ** DOUBLE QR STEP INVOLVING ROWS K TO N AND COLUMNS M TO N
      
      DO 520 KA = I, N1
         NOTLAS = KA.NE.N1
         IF ( KA.EQ.I ) THEN
            S = SIGN( SQRT( P*P + Q*Q + R*R ),P )
            IF ( LB.NE.I ) AA(KA,KA-1) = - AA(KA,KA-1)
         ELSE
            P = AA(KA,KA-1)
            Q = AA(KA+1,KA-1)
            R = ZERO
            IF ( NOTLAS ) R = AA(KA+2,KA-1)
            X = ABS(P) + ABS(Q) + ABS(R)
            IF ( X.EQ.ZERO ) GO TO 520
            P = P / X
            Q = Q / X
            R = R / X
            S = SIGN( SQRT( P*P + Q*Q + R*R ),P )
            AA(KA,KA-1) = - S * X
         END IF
         P = P + S
         X = P / S
         Y = Q / S
         Z = R / S
         Q = Q / P
         R = R / P
C                                                    ** ROW MODIFICATION
         DO 490 J = KA, M
            P = AA(KA,J) + Q * AA(KA+1,J)
            IF ( NOTLAS ) THEN
               P = P + R * AA(KA+2,J)
               AA(KA+2,J) = AA(KA+2,J) - P * Z
            END IF
            AA(KA+1,J) = AA(KA+1,J) - P * Y
            AA(KA,J)   = AA(KA,J)   - P * X
 490     CONTINUE
C                                                 ** COLUMN MODIFICATION
         DO 500 II = 1, MIN0(N,KA+3)
            P = X * AA(II,KA) + Y * AA(II,KA+1)
            IF ( NOTLAS ) THEN
               P = P + Z * AA(II,KA+2)
               AA(II,KA+2) = AA(II,KA+2) - P * R
            END IF
            AA(II,KA+1) = AA(II,KA+1) - P * Q
            AA(II,KA)   = AA(II,KA) - P
 500     CONTINUE
C                                          ** ACCUMULATE TRANSFORMATIONS
         DO 510 II = L, K
            P = X * EVEC(II,KA) + Y * EVEC(II,KA+1)
            IF ( NOTLAS ) THEN
               P = P + Z * EVEC(II,KA+2)
               EVEC(II,KA+2) = EVEC(II,KA+2) - P * R
            END IF
            EVEC(II,KA+1) = EVEC(II,KA+1) - P * Q
            EVEC(II,KA)   = EVEC(II,KA) - P
 510     CONTINUE
 520  CONTINUE
      GO TO 390
C                     ** ALL EVALS FOUND, NOW BACKSUBSTITUTE REAL VECTOR
 530  CONTINUE
      IF ( RNORM.NE.ZERO ) THEN
         DO 560  N = M, 1, -1
            N2 = N
            AA(N,N) = ONE
            DO 550  I = N-1, 1, -1
               W = AA(I,I) - EVAL(N)
               IF ( W.EQ.ZERO ) W = TOL * RNORM
               R = AA(I,N)
               DO 540 J = N2, N-1
                  R = R + AA(I,J) * AA(J,N)
 540           CONTINUE
               AA(I,N) = -R / W
               N2 = I
 550        CONTINUE
 560     CONTINUE
C                      ** END BACKSUBSTITUTION VECTORS OF ISOLATED EVALS

         DO 580 I = 1, M
            IF ( I.LT.L .OR. I.GT.K ) THEN
               DO 570 J = I, M
                  EVEC(I,J) = AA(I,J)
 570           CONTINUE
            END IF
 580     CONTINUE
C                                   ** MULTIPLY BY TRANSFORMATION MATRIX
         IF ( K.NE.0 ) THEN
            DO 610  J = M, L, -1
               DO 600 I = L, K
                  Z = ZERO
                  DO 590 N = L, MIN0(J,K)
                     Z = Z + EVEC(I,N) * AA(N,J)
 590              CONTINUE
                  EVEC(I,J) = Z
 600           CONTINUE
 610        CONTINUE
         END IF
         
      END IF

      DO 620 I = L, K
         DO 620 J = 1, M
            EVEC(I,J) = EVEC(I,J) * WK(I)
 620  CONTINUE
C                           ** INTERCHANGE ROWS IF PERMUTATIONS OCCURRED
      DO 640  I = L-1, 1, -1
         J = WK(I)
         IF ( I.NE.J ) THEN
            DO 630 N = 1, M
               REPL      = EVEC(I,N)
               EVEC(I,N) = EVEC(J,N)
               EVEC(J,N) = REPL
 630        CONTINUE
         END IF
 640  CONTINUE
      
      DO 660 I = K+1, M
         J = WK(I)
         IF ( I.NE.J ) THEN
            DO 650 N = 1, M
               REPL      = EVEC(I,N)
               EVEC(I,N) = EVEC(J,N)
               EVEC(J,N) = REPL
 650        CONTINUE
         END IF
 660  CONTINUE
      
      RETURN
      END
      SUBROUTINE  CHEKIN( ACCUR, ALBEDO, BTEMP, DTAUC, FBEAM, FISOT, HL,
     $                    IBCND, LAMBER, MAXCLY, MAXCMU, MAXCOE, MAXPHI,
     $                    MAXULV, MAXUMU, MXCLY, MXCMU, MXPHI, MXULV,
     $                    MXUMU, NLYR, NPHI, NSTR, NTAU, NUMU, ONLYFL,
     $                    ierror, quiet, PHI, PHI0, PLANCK, PMOM, SSALB, 
     $                    TAUC, TEMIS, TEMPER, TTEMP, UMU, UMU0, USRANG,
     $                    USRTAU, UTAU, WVNMHI, WVNMLO )

C                              Checks the input dimensions and variables

	LOGICAL  WRTBAD, WRTDIM
	LOGICAL  LAMBER, PLANCK, ONLYFL, USRANG, USRTAU, InpErr, quiet
	INTEGER  IBCND, MAXCLY, MAXUMU, MAXULV, MAXCMU, MAXPHI, NLYR,
     $         NUMU, NSTR, NPHI, NTAU, MXCMU, MXUMU, MXPHI, MXCLY,
     $         MXULV, ierror(*)
	REAL     ACCUR, ALBEDO, BTEMP, DTAUC( * ), FBEAM, FISOT,
     $         HL( 0:* ), PHI( * ), PMOM( 0:MAXCMU,* ), PHI0,
     $         SSALB( * ), TAUC( 0:* ), TEMPER( 0:* ), TEMIS,
     $         TTEMP, WVNMLO, WVNMHI, UMU( * ), UMU0, UTAU( * )

	InpErr = .FALSE.
        IF ( nlyr.LT.1 ) THEN
           InpErr = wrtbad(quiet, 'nlyr' )
           ierror(1) = 1
        ENDIF
        IF ( nlyr.GT.maxcly ) THEN
           InpErr = wrtbad(quiet, 'maxcly' )
           ierror(2) = 1
        ENDIF

	DO 10 LC = 1, NLYR
           IF ( dtauc(lc).LT.0.0 )  THEN
              InpErr = wrtbad( quiet, 'dtauc' )
              ierror(3) = ierror(3) + 1
           ENDIF
           IF ( ssalb(lc).LT.0.0 .OR. ssalb(lc).GT.1.0 ) THEN
              InpErr = wrtbad( quiet, 'ssalb' )
              ierror(4) = ierror(4) + 1
           ENDIF
	   IF ( PLANCK .AND. IBCND.NE.1 )  THEN
              IF( lc.EQ.1 .AND. temper(0).LT.0.0 ) THEN
                 InpErr = wrtbad( quiet, 'temper' )
                 ierror(5) = ierror(5) + 1
              ENDIF
              IF( temper(lc).LT.0.0 ) THEN 
                 InpErr = wrtbad( quiet, 'temper' )
                 ierror(5) = ierror(5) + 1
              ENDIF
           ENDIF
           DO k = 0, nstr
              IF( pmom(k,lc).LT.-1.0 .OR. pmom(k,lc).GT.1.0 ) THEN
                 InpErr = wrtbad( quiet, 'pmom' )
                 ierror(6) = ierror(6) + 1
              ENDIF
           ENDDO
 10     CONTINUE

	IF ( IBCND.EQ.1 )  THEN
	   IF ( MAXULV.LT.2 ) THEN
              InpErr = WRTBAD( quiet, 'MAXULV' )
              ierror(34) = 1
           ENDIF
	ELSE IF ( USRTAU )  THEN
           IF ( ntau.LT.1 ) THEN
              inperr = wrtbad( quiet, 'ntau' )
              ierror(8) = 1
           ENDIF
           IF ( maxulv.LT.ntau ) THEN
              inperr = wrtbad( quiet, 'maxulv' )
              ierror(9) = 1
           ENDIF
	   DO 20  LU = 1, NTAU
	      IF( ABS(UTAU(LU)-TAUC(NLYR)).LE.1.E-4)
     $             UTAU(LU) = TAUC(NLYR)
              IF( utau(lu).LT.0.0 .OR. utau(lu).GT.tauc(nlyr) ) THEN
                 inperr = wrtbad( quiet, 'utau' )
                 ierror(10) = ierror(10) + 1
              ENDIF
20       CONTINUE
	ELSE
           IF ( maxulv.LT.nlyr+1 ) THEN
              inperr = wrtbad( quiet, 'maxulv' )
              ierror(11) = 1
           ENDIF
	END IF

        IF ( NSTR.LT.2 .OR. MOD(NSTR,2).NE.0 ) THEN
           inperr = wrtbad( quiet, 'nstr' )
           ierror(23) = 1
        ENDIF
        IF ( NSTR.GT.MAXCMU ) THEN
           inperr = wrtbad( quiet, 'maxcmu' )
           ierror(24) = 1
        ENDIF

	IF ( USRANG )  THEN
           IF ( numu .LT. 0 ) THEN
              inperr = wrtbad( quiet, 'numu' )
              ierror(25) = 1
           ENDIF
           IF ( .NOT. onlyfl .AND. numu .EQ. 0 ) THEN
              inperr = wrtbad( quiet, 'numu' )
              ierror(26) = 1
           ENDIF
           IF ( numu .GT. maxumu ) THEN
              inperr = wrtbad( quiet, 'maxumu' )
              ierror(27) = 1
           ENDIF
	   IF ( IBCND.EQ.1 .AND. 2*NUMU.GT.MAXUMU ) THEN
              inperr = wrtbad( quiet, 'maxumu' )
              ierror(35) = 1
           ENDIF
	   DO 30 IU = 1, NUMU
c              write(*,*) "umu", iu, umu(iu)
              IF( umu(iu).LT.-1.0 .OR. umu(iu).GT.1.0 .OR.
     $             umu(iu).EQ. 0.0) THEN
                 inperr = wrtbad( quiet, 'umu' )
                 ierror(28) = ierror(28) + 1
              ENDIF
              IF( iu.GT.1 .AND. umu(iu).LT.umu(iu-1) ) THEN
                 inperr = wrtbad( quiet, 'umu' )
                 ierror(29) = ierror(29) + 1
              ENDIF
	      IF( IBCND.EQ.1 .AND. UMU(IU).LT.0.0 ) THEN
                 inperr = wrtbad( quiet, 'umu' )
                 ierror(36) = 1
              ENDIF
30       CONTINUE
	ELSE
           IF( maxumu .LT. nstr ) THEN
              inperr = wrtbad( quiet, 'maxumu' )
              ierror(30) = 1
           ENDIF
	END IF

	IF ( .NOT.ONLYFL .AND. IBCND.NE.1 )  THEN
	   IF ( NPHI.LE.0 ) THEN
              InpErr = WRTBAD( quiet, 'NPHI' )
              ierror(37) = 1
           ENDIF
	   IF ( NPHI.GT.MAXPHI ) THEN
              InpErr = WRTBAD( quiet, 'MAXPHI' )
              ierror(38) = 1
           ENDIF
	   DO 40 J = 1, NPHI
	      IF ( PHI(J).LT.0.0 .OR. PHI(J).GT.360.0 ) THEN
                 InpErr = WRTBAD( quiet, 'PHI' )
                 ierror(39) = ierror(39) + 1
              ENDIF
40       CONTINUE
	END IF

	IF ( IBCND.LT.0 .OR. IBCND.GT.1 ) THEN
           InpErr = WRTBAD( quiet, 'IBCND' )
           ierror(40) = 1
        ENDIF
	IF ( IBCND.EQ.0 )  THEN
           IF ( fbeam.LT.0.0 ) THEN
              inperr = wrtbad( quiet, 'fbeam' )
              ierror(12) = 1
           ENDIF
           IF ( fbeam.GT.0.0 .AND. ( umu0.LE.0.0 .OR. umu0.GT.1.0 ) )
     $          THEN
              inperr = wrtbad( quiet, 'umu0' )
              ierror(13) = 1
           ENDIF
	   IF ( FBEAM.GT.0.0 .AND. ( PHI0.LT.0.0 .OR. PHI0.GT.360.0 ) )
     $        THEN
              InpErr = WRTBAD( quiet, 'PHI0' )
              ierror(41) = 1
           ENDIF
           IF ( fisot.LT.0.0 ) THEN
              inperr = wrtbad( quiet, 'fisot' )
              ierror(14) = 1
           ENDIF
	   IF ( LAMBER )  THEN
              IF ( albedo.LT.0.0 .OR. albedo.GT.1.0 ) THEN
                 inperr = wrtbad( quiet, 'albedo' )
                 ierror(15) = 1
              ENDIF
	   ELSE
C                        Make sure flux albedo at dense mesh of incident
C                               angles does not assume unphysical values

	      DO 50 RMU = 0.0, 1.0, 0.01
	         FLXALB = DREF( RMU, HL, NSTR )
	         IF ( FLXALB.LT.0.0 .OR. FLXALB.GT.1.0 ) THEN
                    InpErr = WRTBAD( quiet, 'HL' )
                    ierror(42) = 1
                 ENDIF
50          CONTINUE
	   ENDIF

	ELSE IF ( IBCND.EQ.1 )  THEN
           IF ( albedo.LT.0.0 .OR. albedo.GT.1.0 ) THEN
              inperr = wrtbad( quiet, 'albedo' )
              ierror(15) = 1
           ENDIF
	END IF

	IF ( PLANCK .AND. IBCND.NE.1 )  THEN
           IF ( wvnmlo.LT.0.0 .OR. wvnmhi.LT.wvnmlo ) THEN
              inperr = wrtbad( quiet, 'wvnmlo,hi' )
              ierror(16) = 1
           ENDIF
           IF ( temis.LT.0.0 .OR. temis.GT.1.0 ) THEN
              inperr = wrtbad( quiet, 'temis' )
              ierror(17) = 1
           ENDIF
           IF ( btemp.LT.0.0 ) THEN
              inperr = wrtbad( quiet, 'btemp' )
              ierror(18) = 1
           ENDIF
           IF ( ttemp.LT.0.0 ) THEN
              inperr = wrtbad( quiet, 'ttemp' )
              ierror(19) = 1
           ENDIF
	END IF

	IF ( ACCUR.LT.0.0 .OR. ACCUR.GT.1.E-2 ) THEN
           InpErr = WRTBAD( quiet, 'ACCUR' )
           ierror(43) = 1
        ENDIF

        IF ( mxcly.LT.nlyr ) THEN
           inperr = wrtdim( quiet, 'mxcly', nlyr )
           ierror(20) = 1
        ENDIF
	IF ( IBCND.NE.1 )  THEN
           IF ( usrtau .AND. mxulv.LT.ntau ) THEN
              inperr = wrtdim( quiet, 'mxulv', ntau )
              ierror(21) = 1
           ENDIF
           IF ( .NOT.usrtau .AND. mxulv.LT.nlyr+1 ) THEN
              inperr = wrtdim( quiet, 'mxulv', nlyr+1 )
              ierror(22) = 1
           ENDIF
	ELSE
	   IF ( MXULV.LT.2 ) THEN
              InpErr = WRTDIM( quiet, 'MXULV', 2 )
              ierror(44) = 1
           ENDIF
	END IF

        IF ( MXCMU.LT.NSTR ) THEN
           INPERR = WRTDIM( quiet, 'MXCMU', NSTR )
           ierror(31) = 1
        ENDIF
        IF ( usrang .AND. mxumu.LT.numu ) THEN
           inperr = wrtdim( quiet, 'mxumu', numu )
           ierror(32) = 1
        ENDIF
	IF ( USRANG .AND. IBCND.EQ.1 .AND. MXUMU.LT.2*NUMU ) THEN
           InpErr = WRTDIM( quiet, 'MXUMU', 2*NUMU )
           ierror(45) = 1
        ENDIF
        IF ( .NOT. usrang .AND. mxumu.LT.nstr ) THEN
           inperr = wrtdim( quiet, 'mxumu', nstr )
           ierror(33) = 1
        ENDIF
	IF ( .NOT.ONLYFL .AND. IBCND.NE.1 .AND. MXPHI.LT.NPHI ) THEN
           InpErr = WRTDIM( quiet, 'MXPHI', NPHI )
           ierror(46) = 1
        ENDIF

	IF ( InpErr .AND. .NOT. quiet )
     $   CALL ERRMSG( 'DISORT--input and/or dimension errors', .True. )

      IF ( PLANCK )  THEN
	   DO 60 LC = 1, NLYR
	   IF ( ABS(TEMPER(LC)-TEMPER(LC-1)) .GT. 20.0 
     $             .AND. .NOT. quiet)
     $             CALL ERRMSG( 'CHEKIN--vertical temperature step may'
     $             // ' be too large for good accuracy', .False. )
60       CONTINUE
	END IF

	RETURN
	END
      SUBROUTINE  CMPINT( FBEAM, GC, KK, LAYRU, LL, LYRCUT, MAZIM,
     $     MXCMU, MXUMU, NCUT, NN, NSTR, PLANCK, NTAU,
     $     TAUCPR, UMU0, UTAUPR, XRA, ZZ, ZPLK0, ZPLK1,
     $     UUM )

C          Calculates the Fourier intensity components at the quadrature
C              angles for azimuthal expansion terms (MAZIM) in EQ. SD(2)

C                  I N P U T    V A R I A B L E S:

C       KK      :  eigenvalues of coeff. matrix in EQ. SS(7)
C       GC      :  eigenvectors at polar quadrature angles in EQ. SC(1)
C       LL      :  constants of integration in EQ. SC(1), obtained
C                  by solving scaled version of EQ. SC(5);
C                  exponential term of EQ. SC(12) not included
C       LYRCUT  :  logical flag for truncation of computational layer
C       MAZIM   :  order of azimuthal component
C       NCUT    :  number of computational layer where absorption
C                  optical depth exceeds -ABSCUT-
C       NN      :  order of double-Gauss quadrature (NSTR/2)
C       TAUCPR  :  cumulative optical depth (delta-M-scaled)
C       UTAUPR  :  optical depths of user output levels in delta-M
C                  coordinates;  equal to -UTAU- if no delta-M
C       XRA     :  thermal source expansion, alpha in Eq. KS(15)
C       ZZ      :  beam source vectors in EQ. SS(19)
C       ZPLK0   :  thermal source vectors -Z0-, by solving EQ. SS(16)
C       ZPLK1   :  thermal source vectors -Z1-, by solving EQ. SS(16)
C       (Remainder are 'DisORT' input variables)

C                  O U T P U T   V A R I A B L E S:

C       UUM     :  Fourier components of the intensity in EQ. SD(12)
C                  (at polar quadrature angles)

C                  I N T E R N A L   V A R I A B L E S:

C       FACT    :  EXP( - UTAUPR / UMU0 )
C       ZINT    :  intensity of M=0 case, in EQ. SC(1)
C+----------------------------------------------------------------------

      LOGICAL  LYRCUT, PLANCK
      INTEGER  LAYRU( * )
      REAL     UUM( MXUMU,* )
      REAL     GC( MXCMU,MXCMU,* ), KK( MXCMU,* ), LL( MXCMU,* ),
     $     TAUCPR( 0:* ), UTAUPR( * ), XRA(*), ZZ( MXCMU,* ),
     $     ZPLK0( MXCMU,* ), ZPLK1( MXCMU,* )
      
C                                                  Loop over user levels
      DO 40 LU = 1, NTAU
         
         LYU = LAYRU(LU)
         IF ( LYRCUT .AND. LYU.GT.NCUT )  GO TO 40
         
         DO 30 IQ = 1, NSTR
            ZINT = 0.0
            DO 10 JQ = 1, NN
               ZINT = ZINT + GC(IQ,JQ,LYU) * LL(JQ,LYU) *
     $              EXP( - KK(JQ,LYU)*(UTAUPR(LU) - TAUCPR(LYU)) )
 10         CONTINUE
            DO 20 JQ = NN+1, NSTR
               ZINT = ZINT + GC(IQ,JQ,LYU) * LL(JQ,LYU) *
     $              EXP( - KK(JQ,LYU)*(UTAUPR(LU) - TAUCPR(LYU-1)) )
 20         CONTINUE

            UUM(IQ,LU) = ZINT
            IF ( FBEAM.GT.0.0 )
     $           UUM(IQ,LU) = ZINT + 
     $           ZZ(IQ,LYU) * EXP( - UTAUPR(LU) / UMU0 )
            IF ( PLANCK .AND. MAZIM.EQ.0 )
     $           UUM(IQ,LU) = UUM(IQ,LU) + 
     $           EXP( -XRA(LYU) * UTAUPR(LU) ) * 
     $           ( ZPLK0(IQ,LYU) + ZPLK1(IQ,LYU) * UTAUPR(LU) )
 30      CONTINUE

 40   CONTINUE

      RETURN
      END
      SUBROUTINE  DISORT( NLYR, DTAUC, SSALB, PMOM, NCOEF, TEMPER, 
     $     WVNMLO, WVNMHI, USRTAU, NTAU, UTAU, NSTR, 
     $     USRANG, NUMU, UMU, NPHI, PHI, IBCND, FBEAM, 
     $     UMU0, PHI0, FISOT, LAMBER, ALBEDO, HL, BTEMP, 
     $     TTEMP, TEMIS, DELTAM, IMTHD, PLANCK, ONLYFL, 
     $     ACCUR, quiet, ierror, PRNT, HEADER, MAXCLY, 
     $     MAXULV, MAXUMU, MAXCMU, MAXCOE, MAXPHI, 
     $     RFLDIR, RFLDN, FLUP, DFDT, UAVG, UU, U0U,
     $     ALBMED, TRNMED, uavgdn, uavgso, uavgup )

C **********************************************************************
C       Plane-parallel discrete ordinates radiative transfer program
C             ( see DISORT.DOC for complete documentation )
C **********************************************************************

Changes by arve kylling, june 1995

C  i)   Added quiet and ierror to i/o, see disort.doc for meaning.
C  ii)  Modified plkavg to return the Planck function when wvnmlo .EQ.
C       wvnmhi.

Changes by arve kylling, february 1994, june 1995

C  i)   TOL in qgausn have been increased drastically to make qgausn
C       converge on single precision machines. The old qgausn was in
C       double precision and allowed a smaller TOL
C  ii)  All calls to r1mach(4) have been changed to r1mach(3) to make
C       dis2 run with my machine.f, now changed back again as I no longer
C       use machine.f.
C  iii) Changed the following wrtdims in chekin
C	IF ( USRANG .AND. IBCND.EQ.1 .AND. MXUMU.LT.2*NUMU )
C     $     InpErr = WRTDIM( 'MXUMU', NUMU )
C       now  reads
C	IF ( USRANG .AND. IBCND.EQ.1 .AND. MXUMU.LT.2*NUMU )
C     $     InpErr = WRTDIM( 'MXUMU', 2*NUMU )
C  iv)  "correct" answers in disotest changed for thermal cases, 
C       should be checked on a 64 bit machine, I don't have access to one
C  v)   "correct" answers in slftst changed, should be checked on a 64
C       bit machine, I don't have access to one
C  vi)  Changed PLANK to PLANCK in all routines where used

C+---------------------------------------------------------------------+
C------------------    I/O VARIABLE SPECIFICATIONS     -----------------
C+---------------------------------------------------------------------+

      CHARACTER  HEADER*127
      LOGICAL  DELTAM, LAMBER, quiet, PLANCK, ONLYFL, PRNT(7), 
     $     USRANG, USRTAU
      INTEGER  IBCND, ierror(47), IMTHD, MAXCLY, MAXUMU, MAXULV, 
     $     MAXCMU, MAXPHI, NCOEF, NLYR, NUMU, NSTR, NPHI, NTAU
      REAL     ACCUR, ALBEDO, BTEMP, DTAUC( MAXCLY ), FBEAM, FISOT,
     $     HL( 0:MAXCMU ), PHI( MAXPHI ), PMOM( 0:MAXCOE, MAXCLY ),
     $     PHI0, SSALB( MAXCLY ), TEMPER( 0:MAXCLY ), TEMIS, TTEMP,
     $     WVNMLO, WVNMHI, UMU( MAXUMU ), UMU0, UTAU( MAXULV )
      
      REAL     RFLDIR( MAXULV ), RFLDN( MAXULV ), FLUP( MAXULV ),
     $     UAVG( MAXULV ), DFDT( MAXULV ), U0U( MAXUMU, MAXULV ),
     $     UU( MAXUMU, MAXULV, MAXPHI ), ALBMED( MAXUMU ),
     $     TRNMED( MAXUMU ), 
     $     UAVGDN( MAXULV ), UAVGUP( MAXULV ), UAVGSO( MAXULV )

C+---------------------------------------------------------------------+
C      Routines called (IN ORDER):  SLFTST, ZEROAL, CHEKIN, SETDIS,
C                                   PRTINP, ALBTRN, LEPOLY, SURFAC,
C                                   SOLEIG, UPBEAM, SETCOE, UPISOT, 
C                                   TERPEV, TERPSO, SETMTX, SOLVE0, 
C                                   FLUXES, USRINT, PRAVIN, PRTINT
C+---------------------------------------------------------------------+

C  Index conventions (for all DO-loops and all variable descriptions):

C     IU     :  for user polar angles

C  IQ,JQ,KQ  :  for computational polar angles ('quadrature angles')

C   IQ/2     :  for half the computational polar angles (just the ones
C               in either 0-90 degrees, or 90-180 degrees)

C     J      :  for user azimuthal angles

C     K,L    :  for Legendre expansion coefficients or, alternatively,
C               subscripts of associated Legendre polynomials

C     LU     :  for user levels

C     LC     :  for computational layers (each having a different
C               single-scatter albedo and/or phase function)

C    LEV     :  for computational levels

C    MAZIM   :  for azimuthal components in Fourier cosine expansion
C               of intensity and phase function

C+---------------------------------------------------------------------+
C               I N T E R N A L    V A R I A B L E S

C   AMB(IQ/2,IQ/2)    First matrix factor in reduced eigenvalue problem
C                     of Eqs. SS(12), STWJ(8E)  (used only in SOLEIG)

C   APB(IQ/2,IQ/2)    Second matrix factor in reduced eigenvalue problem
C                     of Eqs. SS(12), STWJ(8E)  (used only in SOLEIG)

C   ARRAY(IQ,IQ)      Scratch matrix for SOLEIG, UPBEAM and UPISOT
C                     (see each subroutine for definition)

C   B()               Right-hand side vector of Eq. SC(5) going into
C                     SOLVE0,1;  returns as solution vector
C                     vector  L, the constants of integration

C   BDR(IQ/2,0:IQ/2)  Bottom-boundary bidirectional reflectivity for a
C                     given azimuthal component.  First index always
C                     refers to a computational angle.  Second index:
C                     if zero, refers to incident beam angle UMU0;
C                     if non-zero, refers to a computational angle.

C   BEM(IQ/2)         Bottom-boundary directional emissivity at compu-
C                     tational angles.

C   BPLANCK            Intensity emitted from bottom boundary

C   CBAND()           Matrix of left-hand side of the linear system
C                     Eq. SC(5), scaled by Eq. SC(12);  in banded
C                     form required by LINPACK solution routines

C   CC(IQ,IQ)         C-sub-IJ in Eq. SS(5)

C   CMU(IQ)           Computational polar angles (Gaussian)

C   CWT(IQ)           Quadrature weights corresponding to CMU

C   DELM0             Kronecker delta, delta-sub-M0, where M = MAZIM
C                     is the number of the Fourier component in the
C                     azimuth cosine expansion

C   EMU(IU)           Bottom-boundary directional emissivity at user
c                     angles.

C   EVAL(IQ)          Temporary storage for eigenvalues of Eq. SS(12)

C   EVECC(IQ,IQ)      Complete eigenvectors of SS(7) on return from
C                     SOLEIG; stored permanently in  GC

C   EXPBEA(LC)        Transmission of direct beam in delta-M optical
C                     depth coordinates

C   FLYR(LC)          Truncated fraction in delta-M method

C   GL(K,LC)          Phase function Legendre polynomial expansion
C                     coefficients, calculated from PMOM by
C                     including single-scattering albedo, factor
C                     2K+1, and (if DELTAM=TRUE) the delta-M
C                     scaling

C   GC(IQ,IQ,LC)      Eigenvectors at polar quadrature angles,
C                     g  in Eq. SC(1)

C   GU(IU,IQ,LC)      Eigenvectors interpolated to user polar angles
C                     ( g  in Eqs. SC(3) and S1(8-9), i.e.
C                       G without the L factor )

C   HLPR()            Legendre coefficients of bottom bidirectional
C                     reflectivity (after inclusion of 2K+1 factor)

C   IPVT(LC*IQ)       Integer vector of pivot indices for LINPACK
C                     routines

C   KK(IQ,LC)         Eigenvalues of coeff. matrix in Eq. SS(7)

C   KCONV             Counter in azimuth convergence test

C   LAYRU(LU)         Computational layer in which user output level
C                     UTAU(LU) is located

C   LL(IQ,LC)         Constants of integration L in Eq. SC(1),
C                     obtained by solving scaled version of Eq. SC(5)

C   LYRCUT            TRUE, radiation is assumed zero below layer
C                     NCUT because of almost complete absorption

C   NAZ               Number of azimuthal components considered

C   NCUT              Computational layer number in which absorption
c                     optical depth first exceeds ABSCUT

C   OPRIM(LC)         Single scattering albedo after delta-M scaling

C   PASS1             TRUE on first entry, FALSE thereafter

C   PKAG(0:LC)        Integrated Planck function for internal emission

C   PKAGC(LC)        Integrated Planck function for internal emission
C                    at the center of each layer.

C   PSI(IQ)           Sum just after square bracket in  Eq. SD(9)

C   RMU(IU,0:IQ)      Bottom-boundary bidirectional reflectivity for a
c                     given azimuthal component.  First index always
c                     refers to a user angle.  Second index:
C                     if zero, refers to incident beam angle UMU0;
C                     if non-zero, refers to a computational angle.

C   TAUC(0:LC)        Cumulative optical depth (un-delta-M-scaled)

C   TAUCPR(0:LC)      Cumulative optical depth (delta-M-scaled if
C                     DELTAM = TRUE, otherwise equal to TAUC)

C   TPLANCK            Intensity emitted from top boundary

C   UUM(IU,LU)        Components of the intensity (u-super-M) when
C                     expanded in Fourier cosine series in azimuth angle

C   U0C(IQ,LU)        Azimuthally-averaged intensity

C   UTAUPR(LU)        Optical depths of user output levels in delta-M
C                     coordinates;  equal to  UTAU(LU) if no delta-M

C   WK()              scratch array

C   XR0(LC)           b-sub-zero in expansion of thermal source func-
C                     tion preceding Eq. KS(15) (has no mu-dependence)

C   XR1(LC)           b-sub-one in expansion of thermal source func-
C                     tion;  see  Eq. KS(15)

C   XRA(LC)           alpha in expansion of thermal source func-
C                     tion;  see  Eq. KS(15)

C   YLM0(L)           Normalized associated Legendre polynomial
C                     of subscript L at the beam angle (not saved
C                     as function of superscipt M)

C   YLMC(L,IQ)        Normalized associated Legendre polynomial
C                     of subscript L at the computational angles
C                     (not saved as function of superscipt M)

C   YLMU(L,IU)        Normalized associated Legendre polynomial
C                     of subscript L at the user angles
C                     (not saved as function of superscipt M)

C   Z()               scratch array used in  SOLVE0,1  to solve a
C                     linear system for the constants of integration

C   Z0(IQ)            Solution vectors Y-sub-zero of Eq. KS(16)

C   Z0U(IU,LC)        Y-sub-zero in Eq. KS(16) interpolated to user
C                     angles from an equation derived from KS(16)

C   Z1(IQ)            Solution vectors Y-sub-one  of Eq. KS(16)

C   Z1U(IU,LC)        Y-sub-one in Eq. KS(16) interpolated to user
C                     angles from an equation derived from KS(16)

C   ZBEAM(IU,LC)      Particular solution for beam source

C   ZJ(IQ)            Right-hand side vector  X-sub-zero in
C                     Eq. SS(19), also the solution vector
C                     Z-sub-zero after solving that system

C   ZZ(IQ,LC)         Permanent storage for the beam source vectors ZJ

C   ZPLK0(IQ,LC)      Permanent storage for the thermal source
C                     vectors  Z0  obtained by solving  Eq. SS(16)

C   ZPLK1(IQ,LC)      Permanent storage for the thermal source
C                     vectors  Z1  obtained by solving  Eq. SS(16)

C+---------------------------------------------------------------------+
C  LOCAL SYMBOLIC DIMENSIONS (have big effect on storage requirements):

C       MXCLY  = Max no. of computational layers
C       MXULV  = Max no. of output levels
C       MXCMU  = Max no. of computation polar angles
C       MXUMU  = Max no. of output polar angles
C       MXPHI  = Max no. of output azimuthal angles

      PARAMETER ( MXCLY = 105, MXULV = 106, MXCMU = 32, MXUMU = 200,
     $     MXPHI = 10, MI = MXCMU/2, MI9M2 = 9*MI-2,
     $     NNLYRI = MXCMU*MXCLY )
C+---------------------------------------------------------------------+

      LOGICAL LYRCUT, PASS1
      INTEGER IPVT( NNLYRI ), LAYRU( MXULV )
      REAL    AMB( MI,MI ), APB( MI,MI ),
     $     ARRAY( MXCMU,MXCMU ), B( NNLYRI ), BDR( MI,0:MI ),
     $     BEM( MI ), CBAND( MI9M2,NNLYRI ), CC( MXCMU,MXCMU ),
     $     CMU( MXCMU ), CWT( MXCMU ), EMU( MXUMU ), EVAL( MI ),
     $     EVECC( MXCMU,MXCMU ), EXPBEA( 0:MXCLY ), FLYR( MXCLY ),
     $     FLDN( MXULV ), FLDIR( MXULV ), GL( 0:MXCMU,MXCLY ),
     $     GC( MXCMU,MXCMU,MXCLY ), GU( MXUMU,MXCMU,MXCLY ),
     $     HLPR( 0:MXCMU ), KK( MXCMU,MXCLY ), LL( MXCMU,MXCLY ),
     $     OPRIM( MXCLY ), PHASA( MXCLY ), PHASE( MXCLY ),
     $     PHASM( MXCLY ), PHIRAD( MXPHI ), PKAG( 0:MXCLY ),
     $     PKAGC(MXCLY ), PSI0( MXCMU ), PSI1( MXCMU ), 
     $     RMU( MXUMU,0:MI ), TAUC( 0:MXCLY ),
     $     TAUCPR( 0:MXCLY ), U0C( MXCMU,MXULV ), UTAUPR( MXULV ),
     $     UUM( MXUMU,MXULV ), WK( MXCMU ), XR0( MXCLY ),
     $     XR1( MXCLY ), XRA( MXCLY ), YLM0( 0:MXCMU ),
     $     YLMC( 0:MXCMU,MXCMU ),
     $     YLMU( 0:MXCMU,MXUMU ), Z( NNLYRI ), Z0( MXCMU ),
     $     Z0U( MXUMU,MXCLY ), Z1( MXCMU ), Z1U( MXUMU,MXCLY ),
     $     ZBEAM( MXUMU,MXCLY ), ZJ( MXCMU ), ZZ( MXCMU,MXCLY ), 
     $     ZPLK0( MXCMU,MXCLY ), ZPLK1( MXCMU,MXCLY )
      REAL  R1MACH
      SAVE  PASS1, PI, EPSIL, RPD
      DATA  PASS1 / .TRUE. /, nerr / 47 /
      
      IF ( PASS1 )  THEN
         PI = 2. * ASIN(1.0)
         EPSIL = 10.*R1MACH(4)
         RPD = PI / 180.0
C                                      Insert input values for self-test
C                                   NOTE: self-test must not use IBCND=1

         CALL  SLFTST( ACCUR, ALBEDO, BTEMP, DELTAM, DTAUC(1), FBEAM,
     $        FISOT, IBCND, IMTHD, LAMBER, NLYR, PLANCK, NPHI,
     $        NUMU, NSTR, NTAU, ONLYFL, PHI( 1 ), PHI0, PMOM,
     $        PRNT, SSALB( 1 ), TEMIS, TEMPER, TTEMP, UMU( 1 ),
     $        USRANG, USRTAU, UTAU( 1 ), UMU0, WVNMHI, WVNMLO,
     $        .FALSE., DUM, DUM, DUM, DUM )
      END IF

 10   CONTINUE

C                          Calculate cumulative optical depth and dither
C                             single-scatter albedo to improve numerical
C                              behavior of eigenvalue/vector computation

      CALL  ZEROIT( TAUC, MXCLY+1 )
      DO 20 LC = 1, NLYR
cakym         IF( SSALB(LC).EQ.1.0 )  SSALB(LC) = 1.0 - EPSIL
         IF( SSALB(LC) .GT. 1.0-epsil )  SSALB(LC) = 1.0 - epsil
         TAUC(LC) = TAUC(LC-1) + DTAUC(LC)
 20   CONTINUE
C                                   Check input dimensions and variables

      DO i = 1, nerr
         ierror(i) = 0
      ENDDO
      CALL  CHEKIN( ACCUR, ALBEDO, BTEMP, DTAUC, FBEAM, FISOT, HL,
     $     IBCND, LAMBER, MAXCLY, MAXCMU, MAXCOE, MAXPHI,
     $     MAXULV, MAXUMU, MXCLY, MXCMU, MXPHI, MXULV, MXUMU,
     $     NLYR, NPHI, NSTR, NTAU, NUMU, ONLYFL, ierror, 
     $     quiet, PHI, PHI0, PLANCK, PMOM, SSALB, TAUC, 
     $     TEMIS, TEMPER, TTEMP, UMU, UMU0, USRANG, USRTAU, 
     $     UTAU, WVNMHI, WVNMLO )
      
      iret = 0
      DO ierr = 1, nerr
         IF ( ierror(ierr) .NE. 0 ) THEN
            iret = 1
            IF ( .NOT. quiet )  THEN
               WRITE(*,'(/,A,I4,/)')  "DISORT REPORTS FATAL ERROR: ",
     $              ierr
            ENDIF
         ENDIF
      ENDDO
      IF ( iret .EQ. 1 ) RETURN
      
C     Zero some arrays (not strictly necessary, but
C     otherwise unused parts of arrays collect garbage)
      
      CALL  ZEROAL( MXCLY, EXPBEA(1), FLYR, OPRIM, PHASA, PHASE, PHASM,
     $     TAUCPR(1), XR0, XR1, XRA,
     $     MXCMU, CMU, CWT, PSI0, PSI1, WK, Z0, Z1, ZJ,
     $     MXCMU+1, HLPR, YLM0,
     $     MXCMU**2, ARRAY, CC, EVECC,
     $     (MXCMU+1)*MXCLY, GL,
     $     (MXCMU+1)*MXCMU, YLMC,
     $     (MXCMU+1)*MXUMU, YLMU,
     $     MXCMU*MXCLY, KK, LL, ZZ, ZPLK0, ZPLK1,
     $     MXCMU**2*MXCLY, GC,
     $     MXULV, LAYRU, UTAUPR,
     $     MXUMU*MXCMU*MXCLY, GU,
     $     MXUMU*MXCLY, Z0U, Z1U, ZBEAM,
     $     MI, EVAL,
     $     MI**2, AMB, APB,
     $     NNLYRI, IPVT, Z )

C                                       Perform various setup operations

      CALL  SETDIS( CMU, CWT, DELTAM, DTAUC, EXPBEA, FBEAM, FLYR,
     $     GL, HL, HLPR, IBCND, ierror, LAMBER, LAYRU, LYRCUT, 
     $     MAXCOE, MAXUMU, MXCMU, NCUT, NLYR, NTAU, NN,
     $     NSTR, PLANCK, NUMU, ONLYFL, OPRIM, PMOM, quiet, SSALB,
     $     TAUC, TAUCPR, UTAU, UTAUPR, UMU, UMU0, USRTAU,
     $     USRANG )
*
*     Check ierror once more because evil may be found inside setdis
*     as well.
*
      iret = 0
      DO ierr = 1, nerr
         IF ( ierror(ierr) .NE. 0 ) THEN
            iret = 1
            IF ( .NOT. quiet )  THEN
               WRITE(*,'(/,A,I4,/)')  "DISORT REPORTS FATAL ERROR: ",
     $              ierr
            ENDIF
         ENDIF
      ENDDO
      IF ( iret .EQ. 1 ) RETURN

C                                                Print input information
      IF ( PRNT(1) )
     $     CALL PRTINP( HEADER, NLYR, DTAUC, SSALB, PMOM, TEMPER,
     $     WVNMLO, WVNMHI, NTAU, UTAU, NSTR, NUMU, UMU,
     $     NPHI, PHI, IBCND, FBEAM, UMU0, PHI0, FISOT,
     $     LAMBER, ALBEDO, HL, BTEMP, TTEMP, TEMIS,
     $     DELTAM, PLANCK, ONLYFL, ACCUR, FLYR, LYRCUT,
     $     OPRIM, TAUC, TAUCPR, MAXCOE, PRNT(7) )

C                             Handle special case for getting albedo and
C                  transmissivity of medium for many beam angles at once

      IF ( IBCND.EQ.1 )  THEN
         CALL  ALBTRN( ALBEDO, AMB, APB, ARRAY, B, BDR, CBAND, CC,
     $        CMU, CWT, EVAL, EVECC, GL, GC, GU, IPVT, KK,
     $        LL, NLYR, NN, NSTR, NUMU, PRNT, TAUCPR, UMU,
     $        U0U, WK, YLMC, YLMU, Z, MI, MI9M2, MAXULV,
     $        MAXUMU, MXCMU, MXUMU, NNLYRI, ALBMED, TRNMED )
         RETURN
      END IF
C                                             Calculate Planck functions
      IF ( .NOT.PLANCK )  THEN
         BPLANCK = 0.0
         TPLANCK = 0.0
         CALL  ZEROIT( PKAG, MXCLY+1 )
         CALL  ZEROIT( PKAGC, MXCLY  )
      ELSE
         TPLANCK = TEMIS * PLKAVG( WVNMLO, WVNMHI, TTEMP )
         BPLANCK =         PLKAVG( WVNMLO, WVNMHI, BTEMP )
         LEV = 0
         PKAG( LEV ) = PLKAVG( WVNMLO, WVNMHI, TEMPER(LEV) )
         DO 30 LEV = 1, NLYR
            PKAG( LEV ) = PLKAVG( WVNMLO, WVNMHI, TEMPER(LEV) )
            TEMPC = ( TEMPER(LEV-1) + TEMPER(LEV) ) / 2.
            PKAGC( LEV ) = PLKAVG( WVNMLO, WVNMHI, TEMPC )
 30      CONTINUE
      END IF


C ========  BEGIN LOOP TO SUM AZIMUTHAL COMPONENTS OF INTENSITY  =======
C           (EQ STWJ 5)

      KCONV = 0
      NAZ = NSTR-1
C     azimuth-independent case
      
      IF ( FBEAM.EQ.0.0 .OR. (1.-UMU0).LT.1.E-5 .OR. ONLYFL .OR.
     $     (NUMU.EQ.1.AND.(1.-UMU(1)).LT.1.E-5 ) )
     $     NAZ = 0
      
      CALL  ZEROIT( UU, MAXUMU*MAXULV*MAXPHI )
      DO 200 MAZIM = 0, NAZ
         
         IF ( MAZIM.EQ.0 )  DELM0 = 1.0
         IF ( MAZIM.GT.0 )  DELM0 = 0.0

C                                     Get normalized associated Legendre
C                             polynomials for incident beam angle cosine
         IF ( FBEAM.GT.0.0 )
     $        CALL  LEPOLY( 1, MAZIM, MXCMU, NSTR-1, -UMU0, YLM0 )

C                         Get normalized associated Legendre polynomials
C                         for computational and user polar angle cosines

         IF ( .NOT.ONLYFL .AND. USRANG )
     $        CALL  LEPOLY( NUMU, MAZIM, MXCMU, NSTR-1, UMU, YLMU )
	 CALL  LEPOLY( NN,   MAZIM, MXCMU, NSTR-1, CMU, YLMC )

C                                Evaluate normalized associated Legendre
C                               polynomials with negative CMU from those
C                              with positive CMU; Dave/Armstrong Eq.(15)
         SGN  = - 1.0
         DO 40 L = MAZIM, NSTR-1
            SGN = - SGN
            DO 40 IQ = NN+1, NSTR
               YLMC( L,IQ ) = SGN * YLMC( L,IQ-NN )
 40      CONTINUE

C            Specify users bottom reflectivity and emissivity properties

         IF ( .NOT.LYRCUT )
     $        CALL  SURFAC( ALBEDO, DELM0, FBEAM, HLPR, LAMBER,
     $        MI, MAZIM, MXCMU, MXUMU, NN, NUMU, NSTR, ONLYFL,
     $        UMU, USRANG, YLM0, YLMC, YLMU, BDR, EMU, BEM,
     $        RMU )
         
C ===================  BEGIN LOOP ON COMPUTATIONAL LAYERS  =============

         DO 100 LC = 1, NCUT

C                           Solve eigenfunction problem in Eq. STWJ(8B);
C                                    return eigenvalues and eigenvectors

            CALL  SOLEIG( AMB, APB, ARRAY, CMU, CWT, GL(0,LC), MI, 
     $           MAZIM, MXCMU, NN, NSTR, WK, YLMC, CC, EVECC, EVAL,
     $           KK(1,LC), GC(1,1,LC) )
            
C                                      Calculate particular solutions of
C                                     Eq.SS(18) for incident beam source
            IF ( FBEAM.GT.0.0 )
     $           CALL  UPBEAM( ARRAY, CC, CMU, DELM0, FBEAM, GL(0,LC),
     $           IPVT, MAZIM, MXCMU, NN, NSTR, PI, UMU0, WK,
     $           YLM0, YLMC, ZJ, ZZ(1,LC) )

C                                      Calculate particular solutions of
C                                  Eq.SS(15) for thermal emission source

            IF ( PLANCK .AND. MAZIM.EQ.0 )  THEN

               CALL SETCOE( KK(1, LC), LC, NCUT, NN, PKAG(LC-1), 
     $              PKAGC(LC), PKAG(LC), TAUCPR, XR0(LC), XR1(LC), 
     $              XRA(LC) )
               
               CALL UPISOT( ARRAY, CC, CMU, IPVT, MXCMU, NN, NSTR,
     $              OPRIM(LC), WK, XR0(LC), XR1(LC), XRA(LC), 
     $              Z0, Z1, ZPLK0(1,LC), ZPLK1(1,LC) )

            END IF

C                                Interpolate eigenvectors to user angles

            IF ( .NOT.ONLYFL .AND. USRANG )  THEN
               CALL  TERPEV( CWT, EVECC, GL(0,LC), GU(1,1,LC), MAZIM, 
     $              MXCMU, MXUMU, NN, NSTR, NUMU, WK, YLMC, YLMU)

C                                Interpolate source terms to user angles
               
               CALL  TERPSO( CWT, DELM0, FBEAM, GL(0,LC), MAZIM, MXCMU, 
     $              PLANCK, NUMU, NSTR, OPRIM(LC), PI, YLM0, 
     $              YLMC, YLMU, PSI0, PSI1, XR0(LC), XR1(LC),
     $              Z0, Z1, ZJ, ZBEAM(1,LC), Z0U(1,LC),
     $              Z1U(1,LC))  
            END IF

 100     CONTINUE

C ===================  END LOOP ON COMPUTATIONAL LAYERS  ===============

C                          Set coefficient matrix of equations combining
C                                boundary and layer interface conditions

         CALL  SETMTX( BDR, CBAND, CMU, CWT, DELM0, GC, KK, LAMBER,
     $        LYRCUT, MI, MI9M2, MXCMU, NCOL, NCUT, NNLYRI,
     $        NN, NSTR, TAUCPR, WK )

C                      Solve for constants of integration in homogeneous
C                                 solution (general boundary conditions)

         CALL  SOLVE0( B, BDR, BEM, BPLANCK, CBAND, CMU, CWT, EXPBEA,
     $        FBEAM, FISOT, IPVT, LAMBER, LL, LYRCUT, MAZIM, MI,
     $        MI9M2, MXCMU, NCOL, NCUT, NN, NSTR, NNLYRI, PI, 
     $        TPLANCK, TAUCPR, UMU0, XRA, Z, ZZ, ZPLK0, ZPLK1 )

C                                     Compute upward and downward fluxes
         IF ( MAZIM.EQ.0 )
     $        CALL FLUXES( CMU, CWT, FBEAM, GC, KK, LAYRU, LL, LYRCUT,
     $        MAXULV, MXCMU, MXULV, NCUT, NN, NSTR, NTAU, 
     $        PI, PRNT, SSALB, TAUCPR, UMU0, UTAU, UTAUPR,
     $        XR0, XR1, XRA, ZZ, ZPLK0, ZPLK1, DFDT, FLUP,
     $        FLDN, FLDIR, RFLDIR, RFLDN, UAVG, U0C,
     $        uavgdn, uavgso, uavgup )

         IF ( ONLYFL )  THEN
            IF ( MAXUMU.GE.NSTR )  THEN

C             Save azimuthally averaged intensities at quadrature angles

               DO 110 LU = 1, NTAU
                  DO 110 IQ = 1, NSTR
                     U0U( IQ,LU ) = U0C( IQ,LU )
 110           CONTINUE
            ELSE
               CALL  ZEROIT( U0U, MAXUMU*MAXULV )
            END IF
            GO TO 300
         END IF

         CALL  ZEROIT( UUM, MXUMU*MXULV )
         IF ( USRANG )  THEN

C                  Compute azimuthal intensity components at user angles

            CALL  USRINT( BPLANCK, CMU, CWT, DELM0, EMU, EXPBEA, FBEAM,
     $           FISOT, GC, GU, KK, LAMBER, LAYRU, LL, LYRCUT,
     $           MAZIM, MXCMU, MXUMU, NCUT, NLYR, NN, NSTR,
     $           PLANCK, NUMU, NTAU, PI, RMU, TAUCPR, TPLANCK,
     $           UMU, UMU0, UTAUPR, WK, XRA, ZBEAM, Z0U, Z1U, ZZ, 
     $           ZPLK0, ZPLK1, UUM )
         ELSE
C            Compute azimuthal intensity components at quadrature angles

            CALL  CMPINT( FBEAM, GC, KK, LAYRU, LL, LYRCUT, MAZIM, 
     $           MXCMU, MXUMU, NCUT, NN, NSTR, PLANCK, NTAU, TAUCPR,
     $           UMU0, UTAUPR, XRA, ZZ, ZPLK0, ZPLK1, UUM )
         END IF

         IF ( MAZIM.EQ.0 )  THEN
C                                  Save azimuthally averaged intensities
            DO 120 LU = 1, NTAU
               DO 120  IU = 1, NUMU
                  U0U( IU,LU ) = UUM( IU,LU )
                  DO 120 J = 1, NPHI
                     UU( IU,LU,J ) = UUM( IU,LU )
120         CONTINUE
C                  Print azimuthally averaged intensities at user angles

            IF ( PRNT(4) )
     $           CALL  PRAVIN( UMU, NUMU, MAXUMU, UTAU, NTAU, U0U )
            IF ( NAZ.GT.0 )  THEN
               CALL  ZEROIT( PHIRAD, MXPHI )
               DO 130 J = 1, NPHI
 130              PHIRAD( J ) = RPD * ( PHI(J) - PHI0 )
            END IF

         ELSE
C                               Increment intensity by current azimuthal
C                            component (Fourier cosine series); Eq.SD(2)
            AZERR = 0.0
            DO 140 J = 1, NPHI
               COSPHI = COS( MAZIM * PHIRAD(J) )
               DO 140 LU = 1, NTAU
                  DO 140 IU = 1, NUMU
                     AZTERM = UUM( IU,LU ) * COSPHI
                     UU( IU,LU,J ) = UU( IU,LU,J ) + AZTERM
                     AZERR = AMAX1( RATIO( ABS(AZTERM), 
     $                    ABS(UU(IU,LU,J)) ), AZERR )
140        CONTINUE
	   IF ( AZERR.LE.ACCUR )  KCONV = KCONV + 1
	   IF ( KCONV.GE.2 )      GO TO 300
	END IF

 200  CONTINUE

C ===================  END LOOP ON AZIMUTHAL COMPONENTS  ===============

 300  CONTINUE
C                                  Nakajima/Tanaka intensity corrections

      IF ( IMTHD.GT.0 .AND. .NOT.ONLYFL .AND. FBEAM.GT.0. )
     $     CALL  INTCOR( EPSIL, FBEAM, FLYR, IMTHD, LAYRU, LYRCUT, 
     $     MAXCOE, MAXULV, MAXUMU, NCOEF, NCUT, NPHI,
     $     NSTR, NTAU, NUMU, OPRIM, PHASA, PHASE, PHASM,
     $     PHIRAD, PI, PMOM, SSALB, TAUC, TAUCPR,
     $     UMU, UMU0, UTAU, UTAUPR, UU )
      
C                                                      Print intensities
      IF ( PRNT(5) .AND. .NOT.ONLYFL )
     $     CALL  PRTINT( UU, UTAU, NTAU, UMU, NUMU, PHI, NPHI,
     $     MAXULV, MAXUMU )

C        Compare test case results with correct answers and abort if bad

      IF ( PASS1 )  THEN
         CALL SLFTST( ACCUR, ALBEDO, BTEMP, DELTAM, DTAUC( 1 ), FBEAM,
     $        FISOT, IBCND, IMTHD, LAMBER, NLYR, PLANCK, NPHI,
     $        NUMU, NSTR, NTAU, ONLYFL, PHI( 1 ), PHI0, PMOM,
     $        PRNT, SSALB( 1 ), TEMIS, TEMPER, TTEMP, UMU( 1 ),
     $        USRANG, USRTAU, UTAU( 1 ), UMU0, WVNMHI, WVNMLO,
     $        .TRUE., FLUP( 1 ), RFLDIR( 1 ), RFLDN( 1 ),
     $        UU( 1,1,1 ) )
         
         PASS1 = .FALSE.
         GO TO 10
      END IF

      RETURN
      END
      REAL FUNCTION  DREF( MU, HL, NSTR )

C        EXACT FLUX ALBEDO FOR GIVEN ANGLE OF INCIDENCE, GIVEN
C        A BIDIRECTIONAL REFLECTIVITY CHARACTERIZED BY ITS
C        LEGENDRE COEFFICIENTS ( NOTE** THESE WILL ONLY AGREE
C        WITH BOTTOM-BOUNDARY ALBEDOS CALCULATED BY 'DISORT' IN
C        THE LIMIT AS NUMBER OF STREAMS GO TO INFINITY, BECAUSE
C        'DISORT' EVALUATES THE INTEGRAL 'CL' ONLY APPROXIMATELY,
C        BY QUADRATURE, WHILE THIS ROUTINE CALCULATES IT EXACTLY. )

C      INPUT :   MU     COSINE OF INCIDENCE ANGLE
C                HL     LEGENDRE COEFFICIENTS OF BIDIRECTIONAL REF'Y
C              NSTR     NUMBER OF ELEMENTS OF 'HL' TO CONSIDER

C      INTERNAL VARIABLES (P-SUB-L IS THE L-TH LEGENDRE POLYNOMIAL) :

C              CL    INTEGRAL FROM 0 TO 1 OF  MU * P-SUB-L(MU)
C                       (VANISHES FOR  L = 3, 5, 7, ... )
C              PL    P-SUB-L
C            PLM1    P-SUB-(L-1)
C            PLM2    P-SUB-(L-2)

      PARAMETER  ( MAXTRM = 100 )
      LOGICAL      PASS1
      REAL         MU, HL( 0:* )
      REAL         C( MAXTRM )
      SAVE  PASS1, C
      DATA  PASS1 / .TRUE. /

      IF ( PASS1 )  THEN
         PASS1 = .FALSE.
         CL = 0.125
         C(2) = 10. * CL
         DO 1  L = 4, MAXTRM, 2
            CL = - CL * (L-3) / (L+2)
            C(L) = 2. * (2*L+1) * CL
    1    CONTINUE
      END IF

      IF ( NSTR.GT.MAXTRM )  CALL
     $     ERRMSG( 'DREF--PARAMETER MAXTRM TOO SMALL', .TRUE. )

      DREF = HL(0) - 2.*HL(1) * MU
      PLM2 = 1.0
      PLM1 = - MU
      DO 10  L = 2, NSTR-1
C                                ** LEGENDRE POLYNOMIAL RECURRENCE

         PL = ( (2*L-1) * (-MU) * PLM1 - (L-1) * PLM2 ) / L
         IF( MOD(L,2).EQ.0 )  DREF = DREF + C(L) * HL(L) * PL
         PLM2 = PLM1
         PLM1 = PL
 10   CONTINUE

      RETURN
      END
      SUBROUTINE  FLUXES( CMU, CWT, FBEAM, GC, KK, LAYRU, LL, LYRCUT,
     $     MAXULV, MXCMU, MXULV, NCUT, NN, NSTR, NTAU,
     $     PI, PRNT, SSALB, TAUCPR, UMU0, UTAU, UTAUPR,
     $     XR0, XR1, XRA, ZZ, ZPLK0, ZPLK1, DFDT, FLUP,
     $     FLDN, FLDIR, RFLDIR, RFLDN, UAVG, U0C,
     $     uavgdn, uavgso, uavgup )

C       CALCULATES THE RADIATIVE FLUXES, MEAN INTENSITY, AND FLUX
C       DERIVATIVE WITH RESPECT TO OPTICAL DEPTH FROM THE M=0 INTENSITY
C       COMPONENTS (THE AZIMUTHALLY-AVERAGED INTENSITY)

C                   I N P U T     V A R I A B L E S:

C       CMU      :  ABSCISSAE FOR GAUSS QUADRATURE OVER ANGLE COSINE
C       CWT      :  WEIGHTS FOR GAUSS QUADRATURE OVER ANGLE COSINE
C       GC       :  EIGENVECTORS AT POLAR QUADRATURE ANGLES, SC(1)
C       KK       :  EIGENVALUES OF COEFF. MATRIX IN EQ. SS(7)
C       LAYRU    :  LAYER NUMBER OF USER LEVEL -UTAU-
C       LL       :  CONSTANTS OF INTEGRATION IN EQ. SC(1), OBTAINED
C                   BY SOLVING SCALED VERSION OF EQ. SC(5);
C                   EXPONENTIAL TERM OF EQ. SC(12) NOT INCLUDED
C       LYRCUT   :  LOGICAL FLAG FOR TRUNCATION OF COMPUT. LAYER
C       NN       :  ORDER OF DOUBLE-GAUSS QUADRATURE (NSTR/2)
C       NCUT     :  NUMBER OF COMPUTATIONAL LAYER WHERE ABSORPTION
C                     OPTICAL DEPTH EXCEEDS -ABSCUT-
C       TAUCPR   :  CUMULATIVE OPTICAL DEPTH (DELTA-M-SCALED)
C       UTAUPR   :  OPTICAL DEPTHS OF USER OUTPUT LEVELS IN DELTA-M
C                     COORDINATES;  EQUAL TO  -UTAU- IF NO DELTA-M
C       XR0      :  EXPANSION OF THERMAL SOURCE FUNCTION IN EQ. KS(15)
C       XR1      :  EXPANSION OF THERMAL SOURCE FUNCTION EQS. KS(15)
C       XRA      :  EXPANSION OF THERMAL SOURCE FUNCTION EQS. KS(15)
C       ZZ       :  BEAM SOURCE VECTORS IN EQ. SS(19)
C       ZPLK0    :  THERMAL SOURCE VECTORS -Z0-, BY SOLVING EQ. SS(16)
C       ZPLK1    :  THERMAL SOURCE VECTORS -Z1-, BY SOLVING EQ. SS(16)
C       (REMAINDER ARE 'DISORT' INPUT VARIABLES)

C                   O U T P U T     V A R I A B L E S:

C       U0C      :  AZIMUTHALLY AVERAGED INTENSITIES
C                   ( AT POLAR QUADRATURE ANGLES )
C       (RFLDIR, RFLDN, FLUP, DFDT, UAVG ARE 'DISORT' OUTPUT VARIABLES)

C                   I N T E R N A L       V A R I A B L E S:

C       DIRINT   :  DIRECT INTENSITY ATTENUATED
C       FDNTOT   :  TOTAL DOWNWARD FLUX (DIRECT + DIFFUSE)
C       FLDIR    :  DIRECT-BEAM FLUX (DELTA-M SCALED)
C       FLDN     :  DIFFUSE DOWN-FLUX (DELTA-M SCALED)
C       FNET     :  NET FLUX (TOTAL-DOWN - DIFFUSE-UP)
C       FACT     :  EXP( - UTAUPR / UMU0 )
C       PLSORC   :  PLANCK SOURCE FUNCTION (THERMAL)
C       ZINT     :  INTENSITY OF m = 0 CASE, IN EQ. SC(1)
C+---------------------------------------------------------------------+

      LOGICAL LYRCUT, PRNT(*)
      REAL    DFDT(*), FLUP(*), FLDIR(*), FLDN(*), RFLDIR(*), RFLDN(* ),
     $     U0C( MXCMU,MXULV ), UAVG(*), uavgdn(*), uavgso(*),
     $     uavgup(*)
      INTEGER LAYRU(*)
      REAL    CMU(*), CWT(*), GC( MXCMU,MXCMU,* ), KK( MXCMU,* ),
     $     LL( MXCMU,* ), SSALB(*), TAUCPR( 0:* ),
     $     UTAU(*), UTAUPR(*), XR0(*), XR1(*), XRA(*), 
     $     ZZ( MXCMU,* ), ZPLK0( MXCMU,* ), ZPLK1( MXCMU,* )


      IF ( PRNT(2) )  WRITE( *,1010 )
C                                          ** ZERO DISORT OUTPUT ARRAYS
      CALL  ZEROIT( U0C, MXULV*MXCMU )
      CALL  ZEROIT( RFLDIR, MAXULV )
      CALL  ZEROIT( FLDIR,  MXULV )
      CALL  ZEROIT( RFLDN,  MAXULV )
      CALL  ZEROIT( FLDN,   MXULV )
      CALL  ZEROIT( FLUP,   MAXULV )
      CALL  ZEROIT( UAVG,   MAXULV )
      CALL  ZEROIT( UAVGdn, MAXULV )
      CALL  ZEROIT( UAVGso, MAXULV )
      CALL  ZEROIT( UAVGup, MAXULV )
      CALL  ZEROIT( DFDT,   MAXULV )
C                                        ** LOOP OVER USER LEVELS
      DO 100  LU = 1, NTAU

         LYU = LAYRU(LU)

         IF ( LYRCUT .AND. LYU.GT.NCUT ) THEN
C                                                ** NO RADIATION REACHES
C                                                ** THIS LEVEL
            FDNTOT = 0.0
            FNET   = 0.0
            PLSORC = 0.0
            GO TO 90
         END IF
         
         IF ( FBEAM.GT.0.0 )  THEN
            FACT  = EXP( - UTAUPR(LU) / UMU0 )
            DIRINT = FBEAM * FACT
            FLDIR(  LU ) = UMU0 * ( FBEAM * FACT )
            RFLDIR( LU ) = UMU0 * FBEAM * EXP( - UTAU( LU ) / UMU0 )
         ELSE
            DIRINT = 0.0
            FLDIR(  LU ) = 0.0
            RFLDIR( LU ) = 0.0
         END IF

         DO 20  IQ = 1, NN
            
            ZINT = 0.0
            DO 10  JQ = 1, NN
               ZINT = ZINT + GC(IQ,JQ,LYU) * LL(JQ,LYU) *
     $              EXP( - KK(JQ,LYU) * (UTAUPR(LU) - TAUCPR(LYU)) )
 10         CONTINUE
            DO 11  JQ = NN+1, NSTR
               ZINT = ZINT + GC(IQ,JQ,LYU) * LL(JQ,LYU) *
     $              EXP( - KK(JQ,LYU) * (UTAUPR(LU) - TAUCPR(LYU-1)) )
 11         CONTINUE

            U0C( IQ,LU ) = ZINT
            IF ( FBEAM.GT.0.0 )  U0C( IQ,LU ) = ZINT + ZZ(IQ,LYU) * FACT
            U0C( IQ,LU ) = U0C( IQ,LU ) +
     $           EXP( -XRA(LYU)*UTAUPR(LU) ) *
     $           ( ZPLK0(IQ,LYU) + ZPLK1(IQ,LYU) * UTAUPR(LU) )
            UAVG(LU) = UAVG(LU) + CWT(NN+1-IQ) * U0C( IQ,LU )
cakyp
*            write(12,*) CMU(NN+1-IQ) ,CWT(NN+1-IQ), U0C( IQ,LU )
cakyp
            UAVGdn(LU) = UAVGdn(LU) + CWT(NN+1-IQ) * U0C( IQ,LU )
            FLDN(LU) = FLDN(LU) + CWT(NN+1-IQ)*CMU(NN+1-IQ) * U0C(IQ,LU)
 20      CONTINUE

         DO 40  IQ = NN+1, NSTR

            ZINT = 0.0
            DO 30  JQ = 1, NN
               ZINT = ZINT + GC(IQ,JQ,LYU) * LL(JQ,LYU) *
     $              EXP( - KK(JQ,LYU) * (UTAUPR(LU) - TAUCPR(LYU)) )
 30         CONTINUE
            DO 31  JQ = NN+1, NSTR
               ZINT = ZINT + GC(IQ,JQ,LYU) * LL(JQ,LYU) *
     $              EXP( - KK(JQ,LYU) * (UTAUPR(LU) - TAUCPR(LYU-1)) )
 31         CONTINUE

            U0C( IQ,LU ) = ZINT
            IF ( FBEAM.GT.0.0 )  U0C( IQ,LU ) = ZINT + ZZ(IQ,LYU) * FACT
            U0C( IQ,LU ) = U0C( IQ,LU ) + 
     $           EXP( -XRA(LYU)*UTAUPR(LU) ) * 
     $           ( ZPLK0(IQ,LYU) + ZPLK1(IQ,LYU) * UTAUPR(LU) )
            UAVG(LU) = UAVG(LU) + CWT(IQ-NN) * U0C( IQ,LU )
cakyp
*            write(12,*)  CMU(IQ-NN), CWT(IQ-NN), U0C( IQ,LU )
cakyp
            UAVGup(LU) = UAVGup(LU) + CWT(IQ-NN) * U0C( IQ,LU )
            FLUP(LU) = FLUP(LU) + CWT(IQ-NN) * CMU(IQ-NN) * U0C( IQ,LU )
 40      CONTINUE

         FLUP( LU )  = 2.0 * PI * FLUP( LU )
         FLDN( LU )  = 2.0 * PI * FLDN( LU )
         FDNTOT = FLDN( LU ) + FLDIR( LU )
         FNET   = FDNTOT - FLUP( LU )
         RFLDN( LU ) = FDNTOT - RFLDIR( LU )
         UAVG( LU ) = ( 2.0 * PI * UAVG(LU) + DIRINT ) / ( 4.*PI )
         UAVGdn( LU ) = ( 2.0 * PI * UAVGdn(LU) ) / ( 4.*PI )
         UAVGso( LU ) =  DIRINT  / ( 4.*PI )
cakyp
*         write(*,*)"Debug disort", lu, dirint, rfldir(lu), uavgso(lu),
*     $        utaupr(lu), umu0
cakyp         
         UAVGup( LU ) = ( 2.0 * PI * UAVGup(LU) ) / ( 4.*PI )
         PLSORC = EXP( -XRA(LYU)*UTAUPR(LU) ) * 
     $        (XR0(LYU) + XR1(LYU) * UTAUPR(LU)) 
         DFDT( LU ) = ( 1.0-SSALB(LYU) ) * 4.*PI* ( UAVG(LU) - PLSORC )
 90      IF( PRNT(2) )  WRITE( *,1020 ) UTAU(LU), LYU, RFLDIR(LU),
     $        RFLDN(LU), FDNTOT, FLUP(LU),
     $        FNET, UAVG(LU), PLSORC, DFDT(LU)
 100  CONTINUE

      IF ( PRNT(3) )  THEN
         WRITE ( *,1100 )
         DO 200  LU = 1, NTAU
            WRITE( *,1110 )  UTAU( LU )
            DO  200  IQ = 1, NN
               ANG1 = 180./PI * ACOS( CMU(2*NN-IQ+1) )
               ANG2 = 180./PI * ACOS( CMU(IQ) )
               WRITE( *,1120 ) ANG1, CMU(2*NN-IQ+1), U0C(IQ,LU),
     $              ANG2, CMU(IQ),        U0C(IQ+NN,LU)
 200     CONTINUE
      END IF

 1010 FORMAT( //, 21X,
     $ '<----------------------- FLUXES ----------------------->', /,
     $ '   OPTICAL  COMPU    DOWNWARD    DOWNWARD    DOWNWARD     ',
     $ ' UPWARD                    MEAN      PLANCK   D(NET FLUX)', /,
     $ '     DEPTH  LAYER      DIRECT     DIFFUSE       TOTAL     ',
     $ 'DIFFUSE         NET   INTENSITY      SOURCE   / D(OP DEP)', / )
 1020 FORMAT( F10.4, I7, 1P,7E12.3, E14.3 )
 1100 FORMAT( //, ' ******** AZIMUTHALLY AVERAGED INTENSITIES',
     $      ' ( AT POLAR QUADRATURE ANGLES ) *******' )
 1110 FORMAT( /, ' OPTICAL DEPTH =', F10.4, //,
     $  '     ANGLE (DEG)   COS(ANGLE)     INTENSITY',
     $  '     ANGLE (DEG)   COS(ANGLE)     INTENSITY' )
 1120 FORMAT( 2( 0P,F16.4, F13.5, 1P,E14.3 ) )

      RETURN
      END
      SUBROUTINE  GETMOM( IPHAS, GG, NMOM, PMOM )
C
C        CALCULATE PHASE FUNCTION LEGENDRE EXPANSION COEFFICIENTS
C        IN VARIOUS SPECIAL CASES
C
      INTEGER  IPHAS, NMOM
      REAL     GG, PMOM( 0:* )
C
C       INPUT: IPHAS   PHASE FUNCTION OPTIONS
C                      1 : ISOTROPIC
C                      2 : RAYLEIGH
C                      3 : HENYEY-GREENSTEIN WITH ASYMMETRY FACTOR -GG-
C                      4 : HAZE L AS SPECIFIED BY GARCIA/SIEWERT
C                      5 : CLOUD C.1 AS SPECIFIED BY GARCIA/SIEWERT
C
C              GG      ASYMMETRY FACTOR FOR HENYEY-GREENSTEIN CASE
C
C              NMOM    INDEX OF HIGHEST LEGENDRE COEFFICIENT NEEDED
C                        ( = NUMBER OF STREAMS 'NSTR'  CHOSEN
C                         FOR THE DISCRETE ORDINATE METHOD)
C
C      OUTPUT: PMOM(K)  LEGENDRE EXPANSION COEFFICIENTS (K=0 TO NMOM)
C                         (BE SURE TO DIMENSION '0:maxval' IN CALLING
C                          PROGRAM)
C
C      REFERENCE:  GARCIA, R. AND C. SIEWERT, 1985: BENCHMARK RESULTS
C                     IN RADIATIVE TRANSFER, TRANSP. THEORY AND STAT.
C                     PHYSICS 14, 437-484, TABLES 10 AND 17
C
      REAL  HAZELM( 82 ), CLDMOM( 299 )
C
      DATA HAZELM /  2.41260, 3.23047, 3.37296, 3.23150, 2.89350,
     A               2.49594, 2.11361, 1.74812, 1.44692, 1.17714,
     B               0.96643, 0.78237, 0.64114, 0.51966, 0.42563,
     C               0.34688, 0.28351, 0.23317, 0.18963, 0.15788,
     D               0.12739, 0.10762, 0.08597, 0.07381, 0.05828,
     E               0.05089, 0.03971, 0.03524, 0.02720, 0.02451,
     F               0.01874, 0.01711, 0.01298, 0.01198, 0.00904,
     G               0.00841, 0.00634, 0.00592, 0.00446, 0.00418,
     H               0.00316, 0.00296, 0.00225, 0.00210, 0.00160,
     I               0.00150, 0.00115, 0.00107, 0.00082, 0.00077,
     J               0.00059, 0.00055, 0.00043, 0.00040, 0.00031,
     K               0.00029, 0.00023, 0.00021, 0.00017, 0.00015,
     L               0.00012, 0.00011, 0.00009, 0.00008, 0.00006,
     M               0.00006, 0.00005, 0.00004, 0.00004, 0.00003,
     N               0.00003, 3*0.00002, 8*0.00001 /
C
      DATA  ( CLDMOM(K), K = 1, 159 ) /
     A  2.544,  3.883,  4.568,  5.235,  5.887,  6.457,  7.177,  7.859,
     B  8.494,  9.286,  9.856, 10.615, 11.229, 11.851, 12.503, 13.058,
     C 13.626, 14.209, 14.660, 15.231, 15.641, 16.126, 16.539, 16.934,
     D 17.325, 17.673, 17.999, 18.329, 18.588, 18.885, 19.103, 19.345,
     E 19.537, 19.721, 19.884, 20.024, 20.145, 20.251, 20.330, 20.401,
     F 20.444, 20.477, 20.489, 20.483, 20.467, 20.427, 20.382, 20.310,
     G 20.236, 20.136, 20.036, 19.909, 19.785, 19.632, 19.486, 19.311,
     H 19.145, 18.949, 18.764, 18.551, 18.348, 18.119, 17.901, 17.659,
     I 17.428, 17.174, 16.931, 16.668, 16.415, 16.144, 15.883, 15.606,
     J 15.338, 15.058, 14.784, 14.501, 14.225, 13.941, 13.662, 13.378,
     K 13.098, 12.816, 12.536, 12.257, 11.978, 11.703, 11.427, 11.156,
     L 10.884, 10.618, 10.350, 10.090,  9.827,  9.574,  9.318,  9.072,
     M  8.822, 8.584, 8.340, 8.110, 7.874, 7.652, 7.424, 7.211, 6.990,
     N  6.785, 6.573, 6.377, 6.173, 5.986, 5.790, 5.612, 5.424, 5.255,
     O  5.075, 4.915, 4.744, 4.592, 4.429, 4.285, 4.130, 3.994, 3.847,
     P  3.719, 3.580, 3.459, 3.327, 3.214, 3.090, 2.983, 2.866, 2.766,
     Q  2.656, 2.562, 2.459, 2.372, 2.274, 2.193, 2.102, 2.025, 1.940,
     R  1.869, 1.790, 1.723, 1.649, 1.588, 1.518, 1.461, 1.397, 1.344,
     S  1.284, 1.235, 1.179, 1.134, 1.082, 1.040, 0.992, 0.954, 0.909 /
      DATA  ( CLDMOM(K), K = 160, 299 ) /
     T  0.873, 0.832, 0.799, 0.762, 0.731, 0.696, 0.668, 0.636, 0.610,
     U  0.581, 0.557, 0.530, 0.508, 0.483, 0.463, 0.440, 0.422, 0.401,
     V  0.384, 0.364, 0.349, 0.331, 0.317, 0.301, 0.288, 0.273, 0.262,
     W  0.248, 0.238, 0.225, 0.215, 0.204, 0.195, 0.185, 0.177, 0.167,
     X  0.160, 0.151, 0.145, 0.137, 0.131, 0.124, 0.118, 0.112, 0.107,
     Y  0.101, 0.097, 0.091, 0.087, 0.082, 0.079, 0.074, 0.071, 0.067,
     Z  0.064, 0.060, 0.057, 0.054, 0.052, 0.049, 0.047, 0.044, 0.042,
     A  0.039, 0.038, 0.035, 0.034, 0.032, 0.030, 0.029, 0.027, 0.026,
     B  0.024, 0.023, 0.022, 0.021, 0.020, 0.018, 0.018, 0.017, 0.016,
     C  0.015, 0.014, 0.013, 0.013, 0.012, 0.011, 0.011, 0.010, 0.009,
     D  0.009, 3*0.008, 2*0.007, 3*0.006, 4*0.005, 4*0.004, 6*0.003,
     E  9*0.002, 18*0.001 /
C
C
      IF ( IPHAS.LT.1 .OR. IPHAS.GT.5 )
     $     CALL ERRMSG( 'GETMOM--BAD INPUT VARIABLE IPHAS', .TRUE. )
      IF ( IPHAS.EQ.3 .AND. (GG.LE.-1.0 .OR. GG.GE.1.0) )
     $     CALL ERRMSG( 'GETMOM--BAD INPUT VARIABLE GG', .TRUE. )
      IF ( NMOM.LT.2 )
     $     CALL ERRMSG( 'GETMOM--BAD INPUT VARIABLE NMOM', .TRUE. )
C
C
      PMOM(0) = 1.0
      DO  10  K = 1, NMOM
         PMOM(K) = 0.0
   10 CONTINUE
C
      IF ( IPHAS.EQ.2 )  THEN
C                                       ** RAYLEIGH PHASE FUNCTION
         PMOM(2) = 0.1
C
      ELSE IF ( IPHAS.EQ.3 ) THEN
C                                       ** HENYEY-GREENSTEIN PHASE FCN
         DO  20  K = 1, NMOM
            PMOM(K) = GG**K
   20    CONTINUE
C
      ELSE IF ( IPHAS.EQ.4 ) THEN
C                                        ** HAZE-L PHASE FUNCTION
         DO  30  K = 1, MIN0(82,NMOM)
            PMOM(K) = HAZELM(K) / ( 2*K+1 )
   30    CONTINUE
C
      ELSE IF ( IPHAS.EQ.5 ) THEN
C                                        ** CLOUD C.1 PHASE FUNCTION
         DO  40  K = 1, MIN0(298,NMOM)
            PMOM(K) = CLDMOM(K) / ( 2*K+1 )
40       CONTINUE
C
      END IF
C
      RETURN
      END
      SUBROUTINE  INTCOR( EPSIL, FBEAM, FLYR, IMTHD, LAYRU, LYRCUT, 
     $                    MAXCOE, MAXULV, MAXUMU, NCOEF, NCUT, NPHI,
     $                    NSTR, NTAU, NUMU, OPRIM, PHASA, PHASE, PHASM,
     $                    PHIRAD, PI, PMOM, SSALB, TAUC, TAUCPR,
     $                    UMU, UMU0, UTAU, UTAUPR, UU )
 
C             Correct intensity field by using Nakajima-Tanaka algorithm
C         (1988).  For more details, see Section 3.6 of STW NASA report.
 
C                I N P U T   V A R I A B L E S
 
C        EPSIL   10 times machine precision
C        FBEAM   incident beam radiation at top
C        FLYR    truncated fraction in delta-M method.
C        IMTHD   method flag:  1: MS, 2: TMS, 3, IMS
C        LAYRU   index of UTAU in multi-layered system
C        LYRCUT  logical flag for truncation of computational layer
C        NCOEF   number of phase function Legendre coefficients supplied
C        NCUT    total number of computational layers considered
C        NPHI    number of user azimuthal angles
C        NSTR    number of polar quadrature angles
C        NTAU    number of user-defined optical depths
C        NUMU    number of user polar angles
C        OPRIM   delta-M-scaled single-scatter albedo
C        PHIRAD  azimuthal angles
C        PMOM    phase function Legendre coefficients (K, LC)
C                    K = 0 to NCOEF, LC = 1 to NLYR with PMOM(0,LC)=1
C        SSALB   single scattering albedo at computational layers
C        TAUC    optical thickness at computational levels
C        TAUCPR  delta-M-scaled optical thickness
C        UMU     cosine of emergent angle
C        UMU0    cosine of incident zenith angle
C        UTAU    user defined optical depths
C        UTAUPR  delta-M-scaled version of UTAU
 
C                O U T P U T   V A R I A B L E S
 
C        UU      corrected intensity field; UU(IU,LU,J)
C                          IU=1,NUMU; LU=1,NTAU; J=1,NPHI
 
C                I N T E R N A L   V A R I A B L E S
 
C        CS      cosine of scattering angle
C        PHASA   actual phase function
C        PHASM   delta-M-scaled phase function
C        PHASE   scratch array, phase function multiplied by
C                various factors
 
C   Routines called:  SECSCA, SINSCA, XIFUNC.
 
	LOGICAL  LYRCUT
	INTEGER  LAYRU( * )
	REAL     FLYR( * ), OPRIM( * ), PHASA( * ), PHASE( * ),
     $         PHASM( * ), PHIRAD( * ), PMOM( 0:MAXCOE,*),
     $         SSALB( * ), TAUC( 0:* ), TAUCPR( 0:* ), UMU( * ),
     $         UTAU( * ), UTAUPR( * ), UU( MAXUMU,MAXULV,* )
 
C                                   Loop over polar and azimuthal angles

      DO 200  IU = 1, NUMU
         DO 100  JP = 1, NPHI
C                                                  COS(SCATTERING ANGLE)

            CS = -UMU0*UMU(IU) + SQRT( (1.-UMU0**2)*(1.-UMU(IU)**2) ) *
     $                           COS( PHIRAD(JP) )
 
C                                              Initialize phase function
            DO 10  LC = 1, NCUT
               PHASA(LC) = 1.
               PHASM(LC) = 1.
10          CONTINUE
C                                   Initialize Legendre poly. recurrence
            PL1 = 1.
            PL2 = 0.
            DO 40  K = 1, NCOEF
C                                              Legendre poly. recurrence
 
               PL  = ( (2*K-1) * CS * PL1 - (K-1) * PL2 ) / K
               PL2 = PL1
               PL1 = PL
C                                        Calculate actual phase function
               DO 20  LC = 1, NCUT
20             PHASA(LC) = PHASA(LC) + (2*K+1) * PL * PMOM(K,LC)
 
C                                       Delta-M truncated phase function
               IF( K.LT.NSTR )  THEN
                  DO 30  LC = 1, NCUT
30                PHASM(LC) = PHASM(LC) + (2*K+1) * PL *
     $                        (PMOM(K,LC)-FLYR(LC)) / (1.-FLYR(LC))
               END IF
40          CONTINUE
C                                                        ** MS method **
C                                               US & UST of EQ. STW (67)
            IF ( IMTHD.EQ.1 )  THEN
               DO 50  LC = 1, NCUT
50             PHASE(LC) = (1.-FLYR(LC)) * PHASM(LC)
               DO 60  LU = 1, NTAU
	            IF ( LYRCUT .AND. LAYRU(LU).GT.NCUT )  GO TO 60
                  US  = FBEAM/(4.*PI) * SINSCA( EPSIL, LAYRU(LU), NCUT,
     $                  PHASA, SSALB, TAUC, UMU(IU), UMU0, UTAU(LU) )
                  UTS = FBEAM/(4.*PI) * SINSCA( EPSIL, LAYRU(LU), NCUT,
     $                  PHASE, SSALB, TAUC, UMU(IU), UMU0, UTAU(LU) )

                  UU(IU,LU,JP) = UU(IU,LU,JP) + US - UTS
60             CONTINUE
 
               GO TO 100
            END IF
C                                                       ** TMS method **
C                                             UPTS & UPS of EQ. STW (68)
            DO 70  LC = 1, NCUT
70          PHASE(LC) = PHASA(LC) / (1.-FLYR(LC)*SSALB(LC))
            DO 80  LU = 1, NTAU
	         IF ( LYRCUT .AND. LAYRU(LU).GT.NCUT )  GO TO 80
               UPTS = FBEAM/(4.*PI) * SINSCA( EPSIL, LAYRU(LU), NCUT,
     $                PHASE, SSALB, TAUCPR, UMU(IU), UMU0, UTAUPR(LU) )
               UPS  = FBEAM/(4.*PI) * SINSCA( EPSIL, LAYRU(LU), NCUT,
     $                PHASM, OPRIM, TAUCPR, UMU(IU), UMU0, UTAUPR(LU) )
 
               UU(IU,LU,JP) = UU(IU,LU,JP) + UPTS - UPS
80          CONTINUE
C                                                       ** IMS method **
C                     Correction of secondary scattering below top level
 
            IF ( IMTHD.EQ.3 .AND. UMU(IU).LE.0.0 )  THEN
               LTAU = 1
               IF ( UTAU(1).LE.EPSIL )  LTAU = 2
               DO 90 LU = LTAU, NTAU
	            IF ( LYRCUT .AND. LAYRU(LU).GT.NCUT )  GO TO 90
                  UHAT = FBEAM/(4.*PI) * SECSCA( CS, FLYR, LAYRU(LU),
     $                   MAXCOE, NCOEF, NSTR, NTAU, PMOM, SSALB, TAUC, 
     $                   UMU(IU), UMU0, UTAU(LU) )
 
                  UU(IU,LU,JP) = UU(IU,LU,JP) - UHAT
90             CONTINUE
            END IF
C                                                   End of angular loops
100      CONTINUE
200   CONTINUE
 
      RETURN
      END
      SUBROUTINE  LEPOLY( NMU, M, MAXMU, TWONM1, MU, YLM )

C       COMPUTES THE NORMALIZED ASSOCIATED LEGENDRE POLYNOMIAL,
C       DEFINED IN TERMS OF THE ASSOCIATED LEGENDRE POLYNOMIAL
C       PLM = P-SUB-L-SUPER-M AS

C             YLM(MU) = SQRT( (L-M)!/(L+M)! ) * PLM(MU)

C       FOR FIXED ORDER -M- AND ALL DEGREES FROM L = M TO TWONM1.
C       WHEN M.GT.0, ASSUMES THAT Y-SUB(M-1)-SUPER(M-1) IS AVAILABLE
C       FROM A PRIOR CALL TO THE ROUTINE.

C       REFERENCE: Dave, J.V. and B.H. Armstrong, Computations of
C                  High-Order Associated Legendre Polynomials,
C                  J. Quant. Spectrosc. Radiat. Transfer 10,
C                  557-562, 1970.  (hereafter D/A)

C       METHOD: Varying degree recurrence relationship.

C       NOTE 1: The D/A formulas are transformed by
C               setting  M = n-1; L = k-1.
C       NOTE 2: Assumes that routine is called first with  M = 0,
C               then with  M = 1, etc. up to  M = TWONM1.
C       NOTE 3: Loops are written in such a way as to vectorize.

C  I N P U T     V A R I A B L E S:

C       NMU    :  NUMBER OF ARGUMENTS OF -YLM-
C       M      :  ORDER OF -YLM-
C       MAXMU  :  FIRST DIMENSION OF -YLM-
C       TWONM1 :  MAX DEGREE OF -YLM-
C       MU(I)  :  I = 1 TO NMU, ARGUMENTS OF -YLM-
C       IF M.GT.0, YLM(M-1,I) FOR I = 1 TO NMU IS REQUIRED

C  O U T P U T     V A R I A B L E:

C       YLM(L,I) :  L = M TO TWONM1, NORMALIZED ASSOCIATED LEGENDRE
C                   POLYNOMIALS EVALUATED AT ARGUMENT -MU(I)-
C+---------------------------------------------------------------------+
      REAL     MU(*), YLM( 0:MAXMU,* )
      INTEGER  M, NMU, TWONM1
      PARAMETER  ( MAXSQT = 1000 )
      REAL     SQT( MAXSQT )
      LOGICAL  PASS1
      SAVE  SQT, PASS1
      DATA  PASS1 / .TRUE. /

      IF ( PASS1 )  THEN
         PASS1 = .FALSE.
         DO 1  NS = 1, MAXSQT
            SQT( NS ) = SQRT( FLOAT(NS) )
    1    CONTINUE
      ENDIF

      IF ( 2*TWONM1 .GT. MAXSQT )
     $     CALL ERRMSG( 'LEPOLY--NEED TO INCREASE PARAM MAXSQT',.TRUE.)

      IF ( M .EQ. 0 )  THEN
C                             ** UPWARD RECURRENCE FOR ORDINARY
C                             ** LEGENDRE POLYNOMIALS
         DO  10  I = 1, NMU
            YLM( 0,I ) = 1.
            YLM( 1,I ) = MU( I )
 10      CONTINUE

         DO  20  L = 2, TWONM1
            DO  20  I = 1, NMU
               YLM( L,I ) = ( ( 2*L-1 ) * MU(I) * YLM( L-1,I )
     $              - ( L-1 ) * YLM( L-2,I ) ) / L
 20      CONTINUE

      ELSE

         DO  30  I = 1, NMU
C                               ** Y-SUB-M-SUPER-M; DERIVED FROM
C                               ** D/A EQS. (11,12)

            YLM( M,I) = - SQT( 2*M-1 ) / SQT( 2*M )
     $           * SQRT( 1. - MU(I)**2 ) * YLM( M-1,I )

C                              ** Y-SUB-(M+1)-SUPER-M; DERIVED FROM
C                              ** D/A EQS. (13,14) USING EQS. (11,12)

            YLM( M+1,I ) = SQT( 2*M+1 ) * MU(I) * YLM( M,I )
 30      CONTINUE
C                                   ** UPWARD RECURRENCE; D/A EQ. (10)
         DO  40  L = M+2, TWONM1
            TMP1 = SQT( L-M ) * SQT( L+M )
            TMP2 = SQT( L-M-1 ) * SQT( L+M-1 )
            DO  40  I = 1, NMU
               YLM( L,I ) = ( ( 2*L-1 ) * MU(I) * YLM( L-1,I )
     $              - TMP2 * YLM( L-2,I ) ) / TMP1
40       CONTINUE

      END IF
      
      RETURN
      END
      REAL FUNCTION  PLKAVG( WNUMLO, WNUMHI, T )

C        COMPUTES PLANCK FUNCTION INTEGRATED BETWEEN TWO WAVENUMBERS,
*        except if wnumlo .EQ. wnmuhi, then the Planck function at 
*        wnumlo is returned

C  NOTE ** CHANGE 'R1MACH' TO 'D1MACH' TO RUN IN DOUBLE PRECISION

C  I N P U T :  WNUMLO : LOWER WAVENUMBER ( INV CM ) OF SPECTRAL
C                           INTERVAL
C               WNUMHI : UPPER WAVENUMBER
C               T      : TEMPERATURE (K)

C  O U T P U T :  PLKAVG : INTEGRATED PLANCK FUNCTION ( WATTS/SQ M )
C                           = INTEGRAL (WNUMLO TO WNUMHI) OF
C                              2H C**2  NU**3 / ( EXP(HC NU/KT) - 1)
C                              (WHERE H=PLANCKS CONSTANT, C=SPEED OF
C                              LIGHT, NU=WAVENUMBER, T=TEMPERATURE,
C                              AND K = BOLTZMANN CONSTANT)

C  REFERENCE : SPECIFICATIONS OF THE PHYSICAL WORLD: NEW VALUE
C                 OF THE FUNDAMENTAL CONSTANTS, DIMENSIONS/N.B.S.,
C                 JAN. 1974

C  METHOD :  FOR  -WNUMLO-  CLOSE TO  -WNUMHI-, A SIMPSON-RULE
C            QUADRATURE IS DONE TO AVOID ILL-CONDITIONING; OTHERWISE

C            (1)  FOR 'WNUMLO' OR 'WNUMHI' SMALL,
C                 INTEGRAL(0 TO WNUMLO/HI) IS CALCULATED BY EXPANDING
C                 THE INTEGRAND IN A POWER SERIES AND INTEGRATING
C                 TERM BY TERM;

C            (2)  OTHERWISE, INTEGRAL(WNUMLO/HI TO INFINITY) IS
C                 CALCULATED BY EXPANDING THE DENOMINATOR OF THE
C                 INTEGRAND IN POWERS OF THE EXPONENTIAL AND
C                 INTEGRATING TERM BY TERM.

C  ACCURACY :  AT LEAST 6 SIGNIFICANT DIGITS, ASSUMING THE
C              PHYSICAL CONSTANTS ARE INFINITELY ACCURATE

C  ERRORS WHICH ARE NOT TRAPPED:

C      * POWER OR EXPONENTIAL SERIES MAY UNDERFLOW, GIVING NO
C        SIGNIFICANT DIGITS.  THIS MAY OR MAY NOT BE OF CONCERN,
C        DEPENDING ON THE APPLICATION.

C      * SIMPSON-RULE SPECIAL CASE IS SKIPPED WHEN DENOMINATOR OF
C        INTEGRAND WILL CAUSE OVERFLOW.  IN THAT CASE THE NORMAL
C        PROCEDURE IS USED, WHICH MAY BE INACCURATE IF THE
C        WAVENUMBER LIMITS (WNUMLO, WNUMHI) ARE CLOSE TOGETHER.
C ----------------------------------------------------------------------
C                                   *** ARGUMENTS
      REAL     T, WNUMLO, WNUMHI
C                                   *** LOCAL VARIABLES

C        A1,2,... :  POWER SERIES COEFFICIENTS
C        c1       :  First radiation constant ( 2 * h * c**2 )
C        C2       :  H * C / K, IN UNITS CM*K (H = PLANCKS CONSTANT,
C                      C = SPEED OF LIGHT, K = BOLTZMANN CONSTANT)
C        D(I)     :  EXPONENTIAL SERIES EXPANSION OF INTEGRAL OF
C                       PLANCK FUNCTION FROM WNUMLO (I=1) OR WNUMHI
C                       (I=2) TO INFINITY
C        EPSIL    :  SMALLEST NUMBER SUCH THAT 1+EPSIL .GT. 1 ON
C                       COMPUTER
C        EX       :  EXP( - V(I) )
C        EXM      :  EX**M
C        MMAX     :  NO. OF TERMS TO TAKE IN EXPONENTIAL SERIES
C        MV       :  MULTIPLES OF 'V(I)'
C        P(I)     :  POWER SERIES EXPANSION OF INTEGRAL OF
C                       PLANCK FUNCTION FROM ZERO TO WNUMLO (I=1) OR
C                       WNUMHI (I=2)
C        PI       :  3.14159...
C        SIGMA    :  STEFAN-BOLTZMANN CONSTANT (W/M**2/K**4)
C        SIGDPI   :  SIGMA / PI
C        SMALLV   :  NUMBER OF TIMES THE POWER SERIES IS USED (0,1,2)
C        V(I)     :  C2 * (WNUMLO(I=1) OR WNUMHI(I=2)) / TEMPERATURE
C        VCUT     :  POWER-SERIES CUTOFF POINT
C        VCP      :  EXPONENTIAL SERIES CUTOFF POINTS
C        VMAX     :  LARGEST ALLOWABLE ARGUMENT OF 'EXP' FUNCTION

      PARAMETER  ( A1 = 1./3., A2 = -1./8., A3 = 1./60., A4 = -1./5040.,
     $     A5 = 1./272160., A6 = -1./13305600. )
      INTEGER  SMALLV
      REAL     C2, CONC, D(2), EPSIL, EX, MV, P(2), SIGMA, SIGDPI,
     $     V(2), VCUT, VCP(7), VSQ
      DOUBLE PRECISION   D1MACH
      SAVE     PI, CONC, VMAX, EPSIL, SIGDPI
      DATA     c1 /1.1911E-8/
      DATA     C2 / 1.438786 /,  SIGMA / 5.67032E-8 /,
     $     VCUT / 1.5 /, VCP / 10.25, 5.7, 3.9, 2.9, 2.3, 1.9, 0.0 /
      DATA     PI / 0.0 /
      F(X) = X**3 / ( EXP(X) - 1 )

      IF ( PI.EQ.0.0 )  THEN
         PI = 2. * ASIN( 1.0 )
         VMAX = ALOG( R1MACH(2) )
         EPSIL = R1MACH(4)
         SIGDPI = SIGMA / PI
         CONC = 15. / PI**4
      END IF

      IF( T.LT.0.0 .OR. WNUMHI.LE.WNUMLO .OR. WNUMLO.LT.0. )
     $     CALL ERRMSG('PLKAVG--TEMPERATURE OR WAVENUMS. WRONG',.TRUE.)
      
      IF ( T.LT.1.E-4 )  THEN
         PLKAVG = 0.0
         RETURN
      ENDIF
*     
      IF ( wnumhi .eq. wnumlo ) THEN
         wvn  =  wnumhi
         arg  = EXP( - C2 * wvn / T )
         plkavg = c1 * (wvn**3.) * arg / ( 1. - arg )
         RETURN
      ENDIF
*     
      V(1) = C2 * WNUMLO / T
      V(2) = C2 * WNUMHI / T
      IF ( V(1).GT.EPSIL .AND. V(2).LT.VMAX .AND.
     $     (WNUMHI-WNUMLO)/WNUMHI .LT. 1.E-2 )  THEN

C                          ** WAVENUMBERS ARE VERY CLOSE.  GET INTEGRAL
C                          ** BY ITERATING SIMPSON RULE TO CONVERGENCE.
         HH = V(2) - V(1)
         OLDVAL = 0.0
         VAL0 = F( V(1) ) + F( V(2) )
         
         DO  2  N = 1, 10
            DEL = HH / (2*N)
            VAL = VAL0
            DO  1  K = 1, 2*N-1
               VAL = VAL + 2*(1+MOD(K,2)) * F( V(1) + K*DEL )
    1       CONTINUE
            VAL = DEL/3. * VAL
            IF ( ABS( (VAL-OLDVAL)/VAL ) .LE. 1.E-6 )  GO TO 3
            OLDVAL = VAL
    2    CONTINUE
         CALL ERRMSG( 'PLKAVG--SIMPSON RULE DIDNT CONVERGE', .FALSE. )

    3    PLKAVG = SIGDPI * T**4 * CONC * VAL
         RETURN
      END IF

      SMALLV = 0
      DO  50  I = 1, 2

         IF( V(I).LT.VCUT )  THEN
C                                   ** USE POWER SERIES
            SMALLV = SMALLV + 1
            VSQ = V(I)**2
            P(I) =  CONC * VSQ * V(I) * ( A1 + V(I) * ( A2 + V(I) *
     $           ( A3 + VSQ * ( A4 + VSQ * ( A5 + VSQ*A6 ) ) ) ) )
         ELSE
C                    ** USE EXPONENTIAL SERIES
            MMAX = 0
C                   ** FIND UPPER LIMIT OF SERIES
 20         MMAX = MMAX + 1
            IF ( V(I).LT.VCP( MMAX ) )  GO TO 20

            EX = EXP( - V(I) )
            EXM = 1.0
            D(I) = 0.0
            
            DO  30  M = 1, MMAX
               MV = M * V(I)
               EXM = EX * EXM
               D(I) = D(I) +
     $              EXM * ( 6. + MV*( 6. + MV*( 3. + MV ) ) ) / M**4
 30         CONTINUE

            D(I) = CONC * D(I)
         END IF

 50   CONTINUE

      IF ( SMALLV .EQ. 2 ) THEN
C                                    ** WNUMLO AND WNUMHI BOTH SMALL
         PLKAVG = P(2) - P(1)

      ELSE IF ( SMALLV .EQ. 1 ) THEN
C                                    ** WNUMLO SMALL, WNUMHI LARGE
         PLKAVG = 1. - P(1) - D(2)

      ELSE
C                                    ** WNUMLO AND WNUMHI BOTH LARGE
         PLKAVG = D(1) - D(2)

      END IF

      PLKAVG = SIGDPI * T**4 * PLKAVG
      IF( PLKAVG.EQ.0.0 )
     $     CALL ERRMSG( 'PLKAVG--RETURNS ZERO; POSSIBLE UNDERFLOW',
     $     .FALSE. )
      
      RETURN
      END
      SUBROUTINE  PRALTR( UMU, NUMU, ALBMED, TRNMED )

C        PRINT PLANAR ALBEDO AND TRANSMISSIVITY OF MEDIUM
C        AS A FUNCTION OF INCIDENT BEAM ANGLE

      REAL     UMU(*), ALBMED(*), TRNMED(*)

      WRITE( *,110 )
      DO 20  IU = 1, NUMU
         ANGL = 180.0/3.14159265 * ACOS( UMU(IU) )
         WRITE(*,111)  ANGL, UMU(IU), ALBMED(IU), TRNMED(IU)
 20   CONTINUE

      RETURN

 110  FORMAT( ///, ' *******  FLUX ALBEDO AND/OR TRANSMISSIVITY OF ',
     $ 'ENTIRE MEDIUM  ********', //,
     $ ' BEAM ZEN ANG   COS(BEAM ZEN ANG)      ALBEDO   TRANSMISSIVITY')
 111  FORMAT( 0P,F13.4, F20.6, F12.5, 1P,E17.4 )
      END
      SUBROUTINE  PRAVIN( UMU, NUMU, MAXUMU, UTAU, NTAU, U0U )

C        PRINT AZIMUTHALLY AVERAGED INTENSITIES AT USER ANGLES

      REAL     UMU(*), UTAU(*), U0U( MAXUMU,* )

      WRITE ( *, '(//,A)' )
     $     ' *********  AZIMUTHALLY AVERAGED INTENSITIES '
     $     // '(USER POLAR ANGLES)  *********'
      LENFMT = 8
      NPASS = 1 + NUMU / LENFMT
      IF ( MOD(NUMU,LENFMT) .EQ. 0 )  NPASS = NPASS - 1
      DO 10  NP = 1, NPASS
         IUMIN = 1 + LENFMT * (NP-1)
         IUMAX = MIN0( LENFMT*NP, NUMU )
         WRITE ( *,101 )  ( UMU(IU), IU = IUMIN, IUMAX )
         DO 10  LU = 1, NTAU
            WRITE( *,102 ) UTAU(LU), ( U0U(IU,LU), IU=IUMIN,IUMAX)
 10   CONTINUE

      RETURN

 101  FORMAT( /, 3X,'OPTICAL   POLAR ANGLE COSINES',
     $     /, 3X,'  DEPTH', 8F14.5 )
 102  FORMAT( 0P,F10.4, 1P,8E14.4 )
      END
      SUBROUTINE  PRTINP( HEADER, NLYR, DTAUC, SSALB, PMOM, TEMPER,
     $     WVNMLO, WVNMHI, NTAU, UTAU, NSTR, NUMU, UMU,
     $     NPHI, PHI, IBCND, FBEAM, UMU0, PHI0,
     $     FISOT, LAMBER, ALBEDO, HL, BTEMP, TTEMP,
     $     TEMIS, DELTAM, PLANCK, ONLYFL, ACCUR,
     $     FLYR, LYRCUT, OPRIM, TAUC, TAUCPR,
     $     MAXCOE, PRTMOM )

C                                        Print values of input variables

      CHARACTER  HEADER*127
      LOGICAL  DELTAM, LAMBER, LYRCUT, PLANCK, ONLYFL, PRTMOM
      REAL     UMU( * ), FLYR( * ), DTAUC( * ), OPRIM( * ), PHI( * ),
     $     PMOM( 0:MAXCOE,* ), SSALB( * ), UTAU( * ), TAUC( 0:* ),
     $     TAUCPR( 0:* ), TEMPER( 0:* ), HL( 0:* )

      WRITE( *,1000 )  HEADER
      WRITE( *,1010 )  NSTR, NLYR
      IF ( IBCND.NE.1 ) WRITE( *,1020 )  NTAU, (UTAU(LU), LU = 1, NTAU)
      IF ( .NOT.ONLYFL )
     $     WRITE( *,1030 )  NUMU, ( UMU(IU), IU = 1, NUMU )
      IF ( .NOT.ONLYFL .AND. IBCND.NE.1 )
     $     WRITE( *,1040 )  NPHI, ( PHI(J), J = 1, NPHI )
      IF ( .NOT.PLANCK .OR. IBCND.EQ.1 )  WRITE( *,1100 )
      WRITE( *,1050 )  IBCND
      IF ( IBCND.EQ.0 )  THEN
         WRITE( *,1060 ) FBEAM, UMU0, PHI0, FISOT
         IF ( LAMBER )   WRITE( *,1080 ) ALBEDO
         IF ( .NOT.LAMBER )  WRITE( *,1090 ) ( HL(K), K = 0, NSTR )
         IF ( PLANCK )  WRITE( *,1110 ) WVNMLO, WVNMHI, BTEMP, 
     $        TTEMP, TEMIS
      ELSE IF ( IBCND.EQ.1 )  THEN
         WRITE( *,1070 )
         WRITE( *,1080 ) ALBEDO
      ENDIF
      IF ( DELTAM )      WRITE( *,1120 )
      IF ( .NOT.DELTAM ) WRITE( *,1130 )
      IF ( IBCND.EQ.1 )  THEN
         WRITE( *,1140 )
      ELSE IF ( ONLYFL )  THEN
         WRITE( *,1150 )
      ELSE
         WRITE( *,1160 )
      ENDIF
      WRITE( *,1170 )  ACCUR
      IF ( LYRCUT )  WRITE( *,1180 )
      IF ( PLANCK  )  WRITE( *,1190 )
      IF ( .NOT.PLANCK )  WRITE( *,1200 )
      YESSCT = 0.0
      DO 10 LC = 1, NLYR
         YESSCT = YESSCT + SSALB(LC)
         IF( PLANCK )
     $        WRITE( *,1210 )  LC, DTAUC(LC), TAUC(LC), SSALB(LC),
     $        FLYR(LC), TAUCPR(LC)-TAUCPR(LC-1), TAUCPR(LC),
     $        OPRIM(LC), PMOM(1,LC), TEMPER(LC-1)
         IF( .NOT.PLANCK )
     $        WRITE( *,1210 )  LC, DTAUC(LC), TAUC(LC), SSALB(LC),
     $        FLYR(LC), TAUCPR(LC)-TAUCPR(LC-1), TAUCPR(LC),
     $        OPRIM(LC), PMOM(1,LC)
 10   CONTINUE
      IF( PLANCK )  WRITE( *,1220 ) TEMPER(NLYR)
      
      IF( PRTMOM .AND. YESSCT.GT.0.0 )  THEN
         WRITE( *, '(/,A)' )  ' LAYER   PHASE FUNCTION MOMENTS'
         DO 20 LC = 1, NLYR
            IF( SSALB(LC).GT.0.0 )
     $           WRITE( *,1230 )  LC, ( PMOM(K,LC), K = 0, NSTR )
 20      CONTINUE
      END IF
      
      RETURN

 1000 FORMAT( ////, 1X, 120('*'), /, 25X,
     $     'DISCRETE ORDINATES RADIATIVE TRANSFER PROGRAM, VERSION 2.0',
     $     /, 1X, A, /, 1X, 120('*') )
 1010 FORMAT( /, ' NO. STREAMS =', I4,
     $     '     NO. COMPUTATIONAL LAYERS =', I4 )
 1020 FORMAT( I4,' USER OPTICAL DEPTHS :',10F10.4, /, (26X,10F10.4) )
 1030 FORMAT( I4,' USER POLAR ANGLE COSINES :',10F9.5,/,(31X,10F9.5) )
1040  FORMAT( I4,' USER AZIMUTHAL ANGLES :', 10F9.2, /, (28X,10F9.2) )
1050  FORMAT( ' BOUNDARY CONDITION FLAG: IBCND =', I2 )
1060  FORMAT( '    INCIDENT BEAM WITH INTENSITY =', 1P,E11.3, ' AND',
     $ ' POLAR ANGLE COSINE = ', 0P,F8.5,'  AND AZIMUTH ANGLE =', F7.2,
     $ /,'    PLUS ISOTROPIC INCIDENT INTENSITY =', 1P,E11.3 )
1070  FORMAT( '    ISOTROPIC ILLUMINATION FROM TOP AND BOTTOM' )
1080  FORMAT( '    BOTTOM ALBEDO (LAMBERTIAN) =', 0P,F8.4 )
1090  FORMAT( '    LEGENDRE COEFFS OF BOTTOM BIDIRECTIONAL',
     $ ' REFLECTIVITY :', /, (10X,10F9.5) )
1100  FORMAT( ' NO THERMAL EMISSION' )
1110  FORMAT( '    THERMAL EMISSION IN WAVENUMBER INTERVAL :', 2F14.4,/,
     $   '    BOTTOM TEMPERATURE =', F10.2, '     TOP TEMPERATURE =',
     $   F10.2,'    TOP EMISSIVITY =', F8.4 )
1120  FORMAT( ' USES DELTA-M METHOD' )
1130  FORMAT( ' DOES NOT USE DELTA-M METHOD' )
1140  FORMAT( ' CALCULATE ALBEDO AND TRANSMISSIVITY OF MEDIUM',
     $   ' VS. INCIDENT BEAM ANGLE' )
1150  FORMAT( ' CALCULATE FLUXES AND AZIM-AVERAGED INTENSITIES ONLY' )
1160  FORMAT( ' CALCULATE FLUXES AND INTENSITIES' )
1170  FORMAT( ' RELATIVE CONVERGENCE CRITERION FOR AZIMUTH SERIES =',
     $   1P,E11.2 )
1180  FORMAT( ' SETS RADIATION = 0 BELOW ABSORPTION OPTICAL DEPTH 10' )
1190  FORMAT( /, 37X, '<------------- DELTA-M --------------->', /,
     $'                   TOTAL    SINGLE                           ',
     $               'TOTAL    SINGLE', /,
     $'       OPTICAL   OPTICAL   SCATTER   TRUNCATED   ',
     $   'OPTICAL   OPTICAL   SCATTER    ASYMM', /,
     $'         DEPTH     DEPTH    ALBEDO    FRACTION     ',
     $     'DEPTH     DEPTH    ALBEDO   FACTOR   TEMPERATURE' )
1200  FORMAT( /, 37X, '<------------- DELTA-M --------------->', /,
     $'                   TOTAL    SINGLE                           ',
     $               'TOTAL    SINGLE', /,
     $'       OPTICAL   OPTICAL   SCATTER   TRUNCATED   ',
     $   'OPTICAL   OPTICAL   SCATTER    ASYMM', /,
     $'         DEPTH     DEPTH    ALBEDO    FRACTION     ',
     $     'DEPTH     DEPTH    ALBEDO   FACTOR' )
1210  FORMAT( I4, 2F10.4, F10.5, F12.5, 2F10.4, F10.5, F9.4,F14.3 )
1220  FORMAT( 85X, F14.3 )
1230  FORMAT( I6, 10F11.6, /, (6X,10F11.6) )

      END




      SUBROUTINE  PRTINT( UU, UTAU, NTAU, UMU, NUMU, PHI, NPHI,
     $     MAXULV, MAXUMU )

C         PRINTS THE INTENSITY AT USER POLAR AND AZIMUTHAL ANGLES

C     ALL ARGUMENTS ARE DISORT INPUT OR OUTPUT VARIABLES

C+---------------------------------------------------------------------+
      REAL   PHI( * ), UMU( * ), UTAU( * ), UU( MAXUMU,MAXULV,* )


      WRITE ( *, '(//,A)' )
     $         ' *********  I N T E N S I T I E S  *********'
      LENFMT = 10
      NPASS = 1 + NPHI / LENFMT
      IF ( MOD(NPHI,LENFMT) .EQ. 0 )  NPASS = NPASS - 1
      DO 10  LU = 1, NTAU
         DO 10  NP = 1, NPASS
            JMIN = 1 + LENFMT * (NP-1)
            JMAX = MIN0( LENFMT*NP, NPHI )
            WRITE( *,100 )  ( PHI(J), J = JMIN, JMAX )
            DO 10  IU = 1, NUMU
               IF( IU.EQ.1 )  WRITE( *,200 )  UTAU(LU), UMU(IU),
     $              ( UU( IU,LU,J ), J = JMIN, JMAX )
               IF( IU.GT.1 )  WRITE( *,300 )  UMU(IU),
     $              ( UU( IU,LU,J ), J = JMIN, JMAX )
10    CONTINUE

      RETURN

100   FORMAT( /, 3X,'          POLAR   AZIMUTH ANGLES (DEGREES)',
     $        /, 3X,'OPTICAL   ANGLE',
     $        /, 3X,' DEPTH   COSINE', 10F11.2 )
200   FORMAT( F10.4, F8.4, 1P,10E11.3 )
300   FORMAT( 10X,   F8.4, 1P,10E11.3 )
      END
      SUBROUTINE  QGAUSN( M, GMU, GWT )

C         Compute weights and abscissae for ordinary gaussian quadrature
C             (no weight function inside integral) on the interval (0,1)

C   INPUT :    M                     order of quadrature rule

C   OUTPUT :  GMU(I)  I = 1 TO M,    array of abscissae
C             GWT(I)  I = 1 TO M,    array of weights

C   REFERENCE:  Davis, P.J. and P. Rabinowitz, Methods of Numerical
C                   Integration, Academic Press, New York, pp. 87, 1975.

C   METHOD:  Compute the abscissae as roots of the Legendre
C            polynomial P-sub-M using a cubically convergent
C            refinement of Newton's method.  Compute the
C            weights from EQ. 2.7.3.8 of Davis/Rabinowitz.  Note
C            that Newton's method can very easily diverge; only a
C            very good initial guess can guarantee convergence.
C            The initial guess used here has never led to divergence
C            even for M up to 1000.

C   ACCURACY:  at least 13 significant digits

C   INTERNAL VARIABLES:

C    ITER      : number of Newton Method iterations
C    MAXIT     : maximum allowed iterations of Newton Method
C    PM2,PM1,P : 3 successive Legendre polynomials
C    PPR       : derivative of Legendre polynomial
C    P2PRI     : 2nd derivative of Legendre polynomial
C    TOL       : convergence criterion for Legendre poly root iteration
C    X,XI      : successive iterates in cubically-convergent version
C                of Newtons Method (seeking roots of Legendre poly.)
C+---------------------------------------------------------------------+
      REAL     CONA, GMU(*), GWT(*), PI, T
      INTEGER  ITER, LIM, M, MAXIT, NP1
      REAL     EN, NNP1, ONE, P, PM1, PM2, PPR, P2PRI, PROD,
     $     TMP, TOL, TWO, X, XI
      SAVE     PI, TOL
      DATA     PI / 0.0 /,  MAXIT / 1000 /,  ONE / 1.0 /,  TWO / 2.0 /

      IF ( PI.EQ.0.0 )  THEN
         PI = 2. * ASIN(1.0)
         TOL = 10.*R1MACH(4)
      END IF

      IF ( M.LT.1 )  CALL ErrMsg( 'QGAUSN--Bad value of M', .TRUE. )
      IF ( M.EQ.1 )  THEN
         GMU( 1 ) = 0.5
         GWT( 1 ) = 1.0
         RETURN
      END IF

      EN   = M
      NP1  = M + 1
      NNP1 = M * NP1
      CONA = FLOAT( M-1 ) / ( 8 * M**3 )

      LIM  = M / 2
      DO 30  K = 1, LIM
C                                Initial guess for k-th root of Legendre
C                           polynomial, from Davis/Rabinowitz (2.7.3.3a)

         T = ( 4*K - 1 ) * PI / ( 4*M + 2 )
         X = COS ( T + CONA / TAN( T ) )
         ITER = 0
C                             Upward recurrence for Legendre polynomials
 10      ITER = ITER + 1
         PM2 = ONE
         PM1 = X
         DO 20 NN = 2, M
            P   = ( ( 2*NN - 1 ) * X * PM1 - ( NN-1 ) * PM2 ) / NN
            PM2 = PM1
            PM1 = P
 20      CONTINUE
C                                                          Newton Method
         TMP   = ONE / ( ONE - X**2 )
         PPR   = EN * ( PM2 - X * P ) * TMP
         P2PRI = ( TWO * X * PPR - NNP1 * P ) * TMP
         XI    = X - ( P / PPR ) * ( ONE +
     $        ( P / PPR ) * P2PRI / ( TWO * PPR ) )

C                                                  Check for convergence
         IF ( ABS(XI-X) .GT. TOL ) THEN
            IF( ITER.GT.MAXIT )
     $           CALL ERRMSG( 'QGAUSN--MAX ITERATION COUNT', .TRUE. )
            X = XI
            GO TO 10
         END IF

C            Iteration finished--calculate weights, abscissae for (-1,1)

         GMU( K ) = - X
         GWT( K ) = TWO / ( TMP * ( EN * PM2 )**2 )
         GMU( NP1 - K ) = - GMU( K )
         GWT( NP1 - K ) =   GWT( K )
30    CONTINUE
C                  Set middle abscissa and weight for rules of odd order

      IF ( MOD( M,2 ) .NE. 0 )  THEN
         GMU( LIM + 1 ) = 0.0
         PROD = ONE
         DO 40 K = 3, M, 2
            PROD = PROD * K / ( K-1 )
 40      CONTINUE
         GWT( LIM + 1 ) = TWO / PROD**2
      END IF
C                                           Convert from (-1,1) to (0,1)
      DO 50  K = 1, M
         GMU( K ) = 0.5 * GMU( K ) + 0.5
         GWT( K ) = 0.5 * GWT( K )
50    CONTINUE

      RETURN
      END
      REAL FUNCTION  RATIO( A, B )

C               Calculate ratio A/B with over- and under-flow protection

         IF ( ABS(A).LT.1.0E-8 .AND. ABS(B).LT.1.0E-8 )  THEN
            RATIO = 1.0
         ELSE IF ( B.EQ.0.0 )  THEN
            RATIO = 1.E+20
         ELSE
            RATIO = A / B
         END IF

      RETURN
      END
      REAL FUNCTION  SECSCA( CS, FLYR, LAYRU, MAXCOE, NCOEF, NSTR, NTAU,
     $                       PMOM, SSALB, TAUC, UMU, UMU0, UTAU )
 
C                          Secondary scattered intensity of EQ. STW (71)
 
C                I N P U T   V A R I A B L E S
 
C        CS      cosine of scattering angle
C        FLYR    truncated fraction in Delta-M method
C        LAYRU   index of UTAU in multi-layered system
C        MAXCOE  maximum number of phase function moment coefficients
C        NCOEF   number of phase function Legendre coefficients supplied
C        NSTR    number of polar quadrature angles
C        NTAU    number of user defined optical depths
C        PMOM    phase function Legendre coefficients (K, LC)
C                K = 0 to NCOEF, LC = 1 to NLYR, with PMOM(0,LC)=1
C        SSALB   single scattering albedo of computational layers
C        TAUC    cumulative optical depth at computational layers
C        UMU     cosine of emergent angle
C        UMU0    cosine of incident zenith angle
C        UTAU    user defined optical depth for output intensity
 
      REAL       FLYR( * ), PMOM( 0:MAXCOE,* ), SSALB( * ), TAUC( 0:* )
 
C              Weighting optical properties of single scattering albedo,
C          truncated fraction and incident zenith angle of EQ. STW (71b)

      TBAR = UTAU - TAUC(LAYRU-1)
      WBAR = SSALB(LAYRU) * TBAR
      FBAR = FLYR(LAYRU) * WBAR
      DO 10  LYR = 1, LAYRU-1
         DTAU = TAUC(LYR) - TAUC(LYR-1)
         TBAR = TBAR + DTAU
         WBAR = WBAR + SSALB(LYR) * DTAU
         FBAR = FBAR + SSALB(LYR) * DTAU * FLYR(LYR)
10    CONTINUE
 
      FBAR = FBAR / WBAR
      WBAR = WBAR / TBAR
      UMU0P= UMU0 / (1.-FBAR*WBAR)
C                                               Weighting phase function
      PSPIKE = 1.
      PHAT = 1.
      PL1 = 1.
      PL2 = 0.
C                                        Legendre poly. recurrence: L<2N
      DO 20  K = 1, NSTR-1
         PL  = ( (2*K-1) * CS * PL1 - (K-1) * PL2 ) / K
         PL2 = PL1
         PL1 = PL
         PSPIKE = PSPIKE + (2.*PHAT - PHAT**2) * (2*K+1) * PL
20    CONTINUE
C                                      Legendre poly. recurrence: L>2N-1
      DO 40  K = NSTR, NCOEF
         PL  = ( (2*K-1) * CS * PL1 - (K-1) * PL2 ) / K
         PL2 = PL1
         PL1 = PL
         DTAU = UTAU - TAUC(LAYRU-1)
         PHAT = PMOM(K,LAYRU) * SSALB(LAYRU) * DTAU
         DO 30  LYR = 1, LAYRU-1
            DTAU = TAUC(LYR) - TAUC(LYR-1)
            PHAT = PHAT + PMOM(K,LYR) * SSALB(LYR) * DTAU
30       CONTINUE
         PHAT = PHAT / (FBAR*WBAR*TBAR)
         PSPIKE = PSPIKE + (2.*PHAT - PHAT**2) * (2*K+1) * PL
40    CONTINUE
C                                                  UHAT of EQ. STW (71a)
 
      SECSCA = ( FBAR*WBAR )**2 / (1.-FBAR*WBAR) * PSPIKE *
     $           XIFUNC( -UMU, UMU0P, UMU0P, UTAU )
 
      RETURN
      END
      SUBROUTINE  SETCOE( KK, LC, NCUT, NN, Q0, Q1, Q2, TAUCPR,
     $     XR0, XR1, XRA ) 

C       Set coefficients for expansion of thermal source function, 
C       eq. KS(15) 

C       Routines called:  None

C       INPUT :  
C                Q0,Q1,Q2    Source at top, center and bottom of layer
C                TAUCPR      delta-M-scaled optical depth

C       OUTPUT:  
C                XR0         expansion of thermal source function
C                XR1         expansion of thermal source function
C                XRA         expansion of thermal source function eq. KS(15)

      INTEGER LC, NCUT
      LOGICAL FIRST
      SAVE big, biggest, first, small, smallest
      REAL big, biggest, small, smallest
      REAL   KK(*), TAUCPR( 0:* ), XR0, XR1, XRA 
C
      DATA FIRST /.TRUE./
C
C The calculation of the particular solutions require some care, small
C and big have been set so that no problems should occurr on 32-bits
C machine running single precision
C
      IF ( FIRST ) THEN
         SMALLEST = R1MACH(1)
         BIGGEST  = R1MACH(2)	 
	 BIG   = SQRT(BIGGEST)
	 SMALL = SMALLEST
	 FIRST = .FALSE.
      ENDIF
C
      XR1 = 0.0
      XRA = 0.0
C
C Set coefficients in KS(15) for thermal source
C
C Calculate alpha coefficient 
C
      DELTAT = TAUCPR(LC) - TAUCPR(LC-1)
C
C Case 1: source small at bottom layer
C
      IF ( (Q2.LT.(Q0*1.E-02) .OR. Q2.LE.SMALL )
     $     .AND. Q1.GT.SMALL .AND. Q0.GT.SMALL ) THEN
C
C alpha Eq. KS(50)
C
         XRA = (2./DELTAT) * LOG( Q0/Q1 )
	 IF ( XRA .GT. BIG )    XRA = BIG
	 IF ( XRA*TAUCPR(LC-1) .GE. ALOG(BIG) ) THEN
	    XR0 =  BIG
	 ELSE
	    XR0 = Q0
	 ENDIF
	 XR1 = 0.0
C
C Case 2: Source small at center and bottom of layer
C
      ELSE IF ( (Q2.LE.(Q1*1.E-02) .OR. Q2.LE.SMALL ) .AND.
     $     ((Q1.LE.(Q0*1.E-02)) .OR. (Q1.LE.SMALL))
     $     .AND. (Q0.GT.SMALL) ) THEN
C       
	 XRA  =   BIG / TAUCPR(NCUT)
	 XR0 = Q0
	 XR1 = 0.0
C
C Case 3:All sources zero
C
	ELSE IF ( Q2.LE.SMALL .AND. Q1.LE.SMALL
     $                           .AND. Q0.LE.SMALL) THEN
	   XRA = 0.0
	   XR0 = 0.0
	   XR1 = 0.0
C
C Case 4: Sources same at center, bottom and top of layer
C         or layer optically very thin
C
	ELSE IF ( (ABS((Q2-Q0)/Q2).LT.1.E-04) .AND.
     $          (ABS((Q2-Q1)/Q2).LT.1.E-04)
     $             .OR. DELTAT.LT. 1.E-04           ) THEN

	   XRA = 0.0
	   XR0 = Q0
	   XR1 = 0.0

C                         **  Case 5: Normal case
	ELSE

	 ARG = (Q1/Q2)**2. - Q0/Q2
	 IF ( ARG .LT. 0.0 ) ARG = 0.0

C alpha Eq. KS(44). For source that has its maximum the top of the
C layer, use negative solution

	 SGN = 1.0
	 IF ( Q0 .GT. Q2 ) SGN = -1.
	 FACT3 = LOG(Q1/Q2 + SGN*SQRT(ARG) )
	 IF ( ABS(FACT3) .LE. 0.005 ) THEN ! BE CAREFUL WITH LOG OF
	    Q1 = 0.99 * Q1	! NUMBERS CLOSE TO ONE
	    FACT3 = LOG(Q1/Q2 + SGN*SQRT(ARG) )
	 ENDIF 
	 XRA = (2./DELTAT) * FACT3
	 IF(ABS(XRA*TAUCPR(LC)) .GT. 
     $             (LOG(BIGGEST)-LOG(Q0*100.) ) )    XRA = 0.0 

C Dither alpha if it is close to an eigenvalue
	 
	 DO IQ = 1, NN
	    IF ( ABS((XRA-KK(IQ))/XRA) .LT. 1.E-4 ) THEN
	       XRA = 1.01*XRA
	    ENDIF
	 ENDDO

C
C Set constants in Eqs. KS(45-46)
C
	 XR1 = (1./DELTAT)*(Q2*EXP(XRA*TAUCPR(LC)) 
     $                                 -Q0*EXP(XRA*TAUCPR(LC-1)))
	 XR0 = Q0 * EXP(XRA*TAUCPR(LC-1)) -
     $                                   XR1*TAUCPR(LC-1)
      ENDIF

      RETURN
      END
      SUBROUTINE  SETDIS( CMU, CWT, DELTAM, DTAUC, EXPBEA, FBEAM, FLYR, 
     $     GL, HL, HLPR, IBCND, ierror, LAMBER, LAYRU, LYRCUT,
     $     MAXCOE, MAXUMU, MXCMU, NCUT, NLYR, NTAU, NN,
     $     NSTR, PLANCK, NUMU, ONLYFL, OPRIM, PMOM, quiet, SSALB, 
     $     TAUC, TAUCPR, UTAU, UTAUPR, UMU, UMU0, USRTAU,
     $     USRANG )
      
C                            Perform miscellaneous setting-up operations

C       Routines called:  ERRMSG, QGAUSN

C       INPUT :  all are DISORT input variables (see DOC file)

C       OUTPUT:  NTAU,UTAU   if USRTAU = FALSE
C                NUMU,UMU    if USRANG = FALSE
C                CMU,CWT     computational polar angles and
C                            corresponding quadrature weights
C                EXPBEA      transmission of direct beam
C                FLYR        truncated fraction in delta-M method
C                GL          phase function Legendre coefficients multi-
C                            plied by (2L+1) and single-scatter albedo
C                HLPR        Legendre moments of surface bidirectional
C                            reflectivity, times 2K+1
C                LAYRU       Computational layer in which UTAU falls
C                LYRCUT      flag as to whether radiation will be zeroed
c                            below layer NCUT
C                NCUT        computational layer where absorption
c                            optical depth first exceeds  ABSCUT
C                NN          NSTR / 2
C                OPRIM       delta-M-scaled single-scatter albedo
C                TAUCPR      delta-M-scaled optical depth
C                UTAUPR      delta-M-scaled version of  UTAU

	LOGICAL  DELTAM, LAMBER, LYRCUT, PLANCK, quiet, ONLYFL, 
     $     USRTAU, USRANG
	INTEGER  ierror(*), LAYRU( * )
	REAL     CMU( * ), CWT( * ), DTAUC( * ), EXPBEA( 0:* ), 
     $         FLYR( * ), GL( 0:MXCMU,* ), HL( 0:* ), HLPR( 0:* ), 
     $         OPRIM( * ), PMOM( 0:MAXCOE,* ), SSALB( * ), TAUC( 0:* ), 
     $         TAUCPR( 0:* ), UTAU( * ), UTAUPR( * ), UMU( * )
	DATA     ABSCUT / 10. /

C                    Set output levels at computational layer boundaries

	IF ( .NOT.USRTAU )  THEN
	   NTAU = NLYR + 1
	   DO 10 LC = 0, NTAU-1
10	   UTAU(LC+1) = TAUC(LC)
	END IF
C                             Apply delta-M scaling and move description
C                             of computational layers to local variables
	EXPBEA( 0 ) = 1.0
	TAUCPR( 0 ) = 0.0
	ABSTAU = 0.0
	DO 40 LC = 1, NLYR
	   PMOM(0,LC) = 1.0
	   IF ( ABSTAU.LT.ABSCUT )  NCUT = LC
	   ABSTAU = ABSTAU + ( 1. - SSALB(LC) ) * DTAUC(LC)

	   IF ( .NOT.DELTAM )  THEN
	      OPRIM(LC) = SSALB(LC)
	      TAUCPR(LC) = TAUC(LC)
	      DO 20 K = 0, NSTR-1
20	      GL(K,LC) = (2*K+1) * OPRIM(LC) * PMOM(K,LC)
	      F = 0.0
	   ELSE
C                                                 delta-M transformation
	      F = PMOM( NSTR,LC )
	      OPRIM(LC) = SSALB(LC) * ( 1. - F ) / ( 1. - F*SSALB(LC))
	      TAUCPR(LC) = TAUCPR(LC-1) + ( 1. - F*SSALB(LC))*DTAUC(LC)
	      DO 30 K = 0, NSTR-1
30	      GL(K,LC) = (2*K+1) * OPRIM(LC) * (PMOM(K,LC)-F) / (1.-F)
	   END IF

	   FLYR(LC) = F
	   EXPBEA(LC) = 0.0
	   IF ( FBEAM.GT.0.0 )  EXPBEA(LC) = EXP(- TAUCPR(LC) / UMU0)
40    CONTINUE
C                If no thermal emission, cut off medium below absorption
C               optical depth = ABSCUT (note that delta-M transformation
C                 leaves absorption optical depth invariant).  Not wortH
C                            the trouble for one-layer problems, though.
	LYRCUT = .FALSE.
	IF ( ABSTAU.GE.ABSCUT .AND. .NOT.PLANCK .AND. IBCND.NE.1
     $     .AND. NLYR.GT.1 )  LYRCUT =.TRUE.
	IF ( .NOT.LYRCUT )  NCUT = NLYR

C                     Set arrays defining location of user output levels
C                               within delta-M-scaled computational mesh
	DO 70 LU = 1, NTAU
	   DO 50 LC = 1, NLYR
	      IF ( UTAU(LU).GE.TAUC(LC-1) .AND. UTAU(LU).LE.TAUC(LC) )
     $             GO TO 60
 50        CONTINUE
	   LC = NLYR

 60        UTAUPR(LU) = UTAU(LU)
           IF(DELTAM) UTAUPR(LU) = TAUCPR(LC-1) + 
     $          (1.-SSALB(LC)*FLYR(LC)) * (UTAU(LU) - TAUC(LC-1))
	   LAYRU(LU) = LC
 70     CONTINUE
C                        Calculate computational polar angle cosines and
C                             associated quadrature weights for Gaussian
C                              quadrature on the interval (0,1) (upward)
	NN = NSTR / 2
	CALL  QGAUSN( NN, CMU, CWT )
C                                      Downward (neg) angles and weights
	DO 80 IQ = 1, NN
	   CMU(IQ+NN) = - CMU(IQ)
	   CWT(IQ+NN) =   CWT(IQ)
80    CONTINUE
C                             Compare beam angle to computational angles

	IF ( FBEAM.GT.0.0 )  THEN
           ierror(47) = 0
	   DO 90 IQ = 1, NN
	      IF ( ABS(UMU0-CMU(IQ))/UMU0 .LT. 1.E-4 )  THEN
                 ierror(47) = ierror(47) + 1
                 IF ( .NOT. quiet ) THEN
                    CALL ERRMSG
     $        ( 'SETDIS--beam angle=computational angle; change NSTR',
     $            .TRUE. )
                 ENDIF
              ENDIF
90       CONTINUE
	END IF
C                  Set output polar angles to computational polar angles

	IF ( .NOT.USRANG .OR. (ONLYFL .AND. MAXUMU.GE.NSTR) )  THEN
	   NUMU = NSTR
	   DO 100 IU = 1, NN
100	   UMU(IU) = - CMU(NN+1-IU)
	   DO 110 IU = NN+1, NSTR
110	   UMU(IU) = CMU(IU-NN)
	END IF
C                             Shift positive user angle cosines to upper 
C                         locations and put negatives in lower locations

	IF ( USRANG .AND. IBCND.EQ.1 )  THEN
	   DO 120 IU = 1, NUMU
120	   UMU(IU+NUMU) = UMU(IU)
	   DO 130 IU = 1, NUMU
130	   UMU(IU) = - UMU( 2*NUMU+1-IU)
	   NUMU = 2*NUMU
	END IF

	IF ( .NOT.LYRCUT .AND. .NOT.LAMBER )  THEN
	   DO 140 K = 0, NSTR
140	   HLPR(K) = (2*K+1) * HL(K)
	END IF

	RETURN
	END
      SUBROUTINE  SETMTX( BDR, CBAND, CMU, CWT, DELM0, GC, KK, LAMBER,
     $     LYRCUT, MI, MI9M2, MXCMU, NCOL, NCUT, NNLYRI,
     $     NN, NSTR, TAUCPR, WK )

C        CALCULATE COEFFICIENT MATRIX FOR THE SET OF EQUATIONS
C        OBTAINED FROM THE BOUNDARY CONDITIONS AND THE CONTINUITY-
C        OF-INTENSITY-AT-LAYER-INTERFACE EQUATIONS;  STORE IN THE
C        SPECIAL BANDED-MATRIX FORMAT REQUIRED BY LINPACK ROUTINES

C     ROUTINES CALLED:  ZEROIT

C     I N P U T      V A R I A B L E S:

C       BDR      :  SURFACE BIDIRECTIONAL REFLECTIVITY
C       CMU      :  ABSCISSAE FOR GAUSS QUADRATURE OVER ANGLE COSINE
C       CWT      :  WEIGHTS FOR GAUSS QUADRATURE OVER ANGLE COSINE
C       DELM0    :  KRONECKER DELTA, DELTA-SUB-M0
C       GC       :  EIGENVECTORS AT POLAR QUADRATURE ANGLES, SC(1)
C       KK       :  EIGENVALUES OF COEFF. MATRIX IN EQ. SS(7)
C       LYRCUT   :  LOGICAL FLAG FOR TRUNCATION OF COMPUT. LAYER
C       NN       :  NUMBER OF STREAMS IN A HEMISPHERE (NSTR/2)
C       NCUT     :  TOTAL NUMBER OF COMPUTATIONAL LAYERS CONSIDERED
C       TAUCPR   :  CUMULATIVE OPTICAL DEPTH (DELTA-M-SCALED)
C       (REMAINDER ARE 'DISORT' INPUT VARIABLES)

C   O U T P U T     V A R I A B L E S:

C       CBAND    :  LEFT-HAND SIDE MATRIX OF LINEAR SYSTEM EQ. SC(5),
C                   SCALED BY EQ. SC(12); IN BANDED FORM REQUIRED
C                   BY LINPACK SOLUTION ROUTINES
C       NCOL     :  COUNTS OF COLUMNS IN -CBAND-

C   I N T E R N A L    V A R I A B L E S:

C       IROW     :  POINTS TO ROW IN  -CBAND-
C       JCOL     :  POINTS TO POSITION IN LAYER BLOCK
C       LDA      :  ROW DIMENSION OF -CBAND-
C       NCD      :  NUMBER OF DIAGONALS BELOW OR ABOVE MAIN DIAGONAL
C       NCOL     :  COUNTS OF COLUMNS IN -CBAND-
C       NSHIFT   :  FOR POSITIONING NUMBER OF ROWS IN BAND STORAGE
C       WK       :  TEMPORARY STORAGE FOR 'EXP' EVALUATIONS
C ---------------------------------------------------------------------+
      LOGICAL LAMBER, LYRCUT
      REAL    BDR( MI,0:* ), CBAND( MI9M2,NNLYRI ), CMU(*), CWT(*),
     $     GC( MXCMU,MXCMU,* ), KK( MXCMU,* ), TAUCPR( 0:* ), WK(*)

      CALL  ZEROIT( CBAND, MI9M2*NNLYRI )
      NCD    = 3*NN - 1
      LDA    = 3*NCD + 1
      NSHIFT = LDA - 2*NSTR + 1
      NCOL   = 0
C                         ** USE CONTINUITY CONDITIONS OF EQ. STWJ(17)
C                         ** TO FORM COEFFICIENT MATRIX IN STWJ(20);
C                         ** EMPLOY SCALING TRANSFORMATION STWJ(22)
      DO 30  LC = 1, NCUT

         DO 4  IQ = 1, NN
            WK(IQ) = EXP( KK(IQ,LC) * (TAUCPR(LC) - TAUCPR(LC-1)) )
 4       CONTINUE

         JCOL = 0
         DO 10  IQ = 1, NN
            NCOL = NCOL + 1
            IROW = NSHIFT - JCOL
            DO 5  JQ = 1, NSTR
               CBAND(IROW+NSTR,NCOL) =   GC(JQ,IQ,LC)
               CBAND(IROW,     NCOL) = - GC(JQ,IQ,LC) * WK(IQ)
               IROW = IROW + 1
 5          CONTINUE
            JCOL = JCOL + 1
 10      CONTINUE

         DO 20  IQ = NN+1, NSTR
            NCOL = NCOL + 1
            IROW = NSHIFT - JCOL
            DO 15  JQ = 1, NSTR
               CBAND(IROW+NSTR,NCOL) =   GC(JQ,IQ,LC) * WK(NSTR+1-IQ)
               CBAND(IROW,     NCOL) = - GC(JQ,IQ,LC)
               IROW = IROW + 1
 15         CONTINUE
            JCOL = JCOL + 1
 20      CONTINUE

 30   CONTINUE
C                  ** USE TOP BOUNDARY CONDITION OF STWJ(20A) FOR
C                  ** FIRST LAYER
      JCOL = 0
      DO 40  IQ = 1, NN
         EXPA = EXP( KK(IQ,1) * TAUCPR(1) )
         IROW = NSHIFT - JCOL + NN
         DO 35  JQ = NN, 1, -1
            CBAND(IROW,JCOL+1) = GC(JQ,IQ,1) * EXPA
            IROW = IROW+1
 35      CONTINUE
         JCOL = JCOL+1
40    CONTINUE

      DO 50  IQ = NN+1, NSTR
         IROW = NSHIFT - JCOL + NN
         DO 45  JQ = NN, 1, -1
            CBAND(IROW,JCOL+1) = GC(JQ,IQ,1)
            IROW = IROW+1
 45      CONTINUE
         JCOL = JCOL+1
 50   CONTINUE
C                           ** USE BOTTOM BOUNDARY CONDITION OF
C                           ** STWJ(20C) FOR LAST LAYER
      NNCOL = NCOL - NSTR
      JCOL  = 0
      DO 70  IQ = 1, NN
         NNCOL = NNCOL + 1
         IROW  = NSHIFT - JCOL + NSTR
         
         DO 60  JQ = NN+1, NSTR
            IF ( LYRCUT .OR. (LAMBER .AND. DELM0.EQ.0) ) THEN

C                          ** NO AZIMUTHAL-DEPENDENT INTENSITY IF LAM-
C                          ** BERT SURFACE; NO INTENSITY COMPONENT IF
C                          ** TRUNCATED BOTTOM LAYER

               CBAND(IROW,NNCOL) = GC(JQ,IQ,NCUT)
            ELSE
               SUM = 0.0
               DO 55  K = 1, NN
                  SUM = SUM + CWT(K) * CMU(K) * BDR(JQ-NN,K)
     $                 * GC(NN+1-K,IQ,NCUT)
 55            CONTINUE
               CBAND(IROW,NNCOL) = GC(JQ,IQ,NCUT) - (1.+DELM0) * SUM
            END IF
            
            IROW = IROW + 1
 60      CONTINUE
         JCOL = JCOL + 1
 70   CONTINUE

      DO 90  IQ = NN+1, NSTR
         NNCOL = NNCOL + 1
         IROW  = NSHIFT - JCOL + NSTR
         EXPA = WK(NSTR+1-IQ)
         
         DO 80  JQ = NN+1, NSTR
            
            IF ( LYRCUT .OR. (LAMBER .AND. DELM0.EQ.0) ) THEN
               CBAND(IROW,NNCOL) = GC(JQ,IQ,NCUT) * EXPA
            ELSE
               SUM = 0.0
               DO 75  K = 1, NN
                  SUM = SUM + CWT(K) * CMU(K) * BDR(JQ-NN,K)
     $                 * GC(NN+1-K,IQ,NCUT)
 75            CONTINUE
               CBAND(IROW,NNCOL) = ( GC(JQ,IQ,NCUT)
     $              - (1.+DELM0) * SUM ) * EXPA
            END IF
            
            IROW = IROW + 1
 80      CONTINUE
         JCOL = JCOL + 1
 90   CONTINUE
      
      RETURN
      END
      REAL FUNCTION  SINSCA( EPSIL, LAYRU, NLYR, PHASE, OMEGA, TAU, UMU,
     $                       UMU0, UTAU )
 
C                       Singly scattered intensity of EQS. STW (65b,d,e)
 
C                I N P U T   V A R I A B L E S
 
C        EPSIL    10 times machine precision
C        LAYRU    index of UTAU in multi-layered system
C        NLYR     number of sublayers
C        PHASE    phase functions of sublayers
C        OMEGA    single scattering albedos of sublayers
C        TAU      optical thicknesses of sublayers
C        UMU      cosine of emergent angle
C        UMU0     cosine of incident zenith angle
C        UTAU     user defined optical depth for output intensity
 
      REAL        PHASE( * ), OMEGA( * ), TAU( 0:* )
C                                                         Initialization
      SINSCA = 0.
      EXP0   = EXP( -UTAU/UMU0 )
C                                      EQ. STW (65e): direct transmitted

      IF ( ABS(UMU+UMU0).LE.EPSIL )  THEN
         DO 10  LYR = 1, LAYRU-1
10       SINSCA = SINSCA + OMEGA(LYR)*PHASE(LYR)*(TAU(LYR)-TAU(LYR-1))
         SINSCA = EXP0 / UMU0 * ( SINSCA + OMEGA(LAYRU)*PHASE(LAYRU)
     $                                   * ( UTAU-TAU(LAYRU-1) ) )
         RETURN
      END IF
C                                               EQ. STW (65b): reflected
      IF ( UMU.GT.0. )  THEN
         DO 20  LYR = LAYRU, NLYR
            EXP1 = EXP( - ( (TAU(LYR)-UTAU)/UMU + TAU(LYR)/UMU0 ) )
            SINSCA = SINSCA + OMEGA(LYR) * PHASE(LYR) * (EXP0-EXP1)
            EXP0 = EXP1
20       CONTINUE
      ELSE
C                                     EQ. STW (65d): diffuse transmitted
         DO 30  LYR = LAYRU, 1, -1
            EXP1 = EXP( - ( (TAU(LYR-1)-UTAU)/UMU + TAU(LYR-1)/UMU0 ) )
            SINSCA = SINSCA + OMEGA(LYR) * PHASE(LYR) * (EXP0-EXP1)
            EXP0 = EXP1
30       CONTINUE
      END IF
 
      SINSCA = SINSCA / ( 1. + UMU/UMU0 )
 
      RETURN
      END
      SUBROUTINE  SLFTST( ACCUR, ALBEDO, BTEMP, DELTAM, DTAUC, FBEAM,
     $     FISOT, IBCND, IMTHD, LAMBER, NLYR, PLANCK, 
     $     NPHI, NUMU, NSTR, NTAU, ONLYFL, PHI, PHI0, 
     $     PMOM, PRNT, SSALB, TEMIS, TEMPER, TTEMP, 
     $     UMU, USRANG, USRTAU, UTAU, UMU0, WVNMHI, 
     $     WVNMLO, COMPAR, FLUP, RFLDIR, RFLDN, UU )

C       If  COMPAR = FALSE, save user input values that would otherwise
C       be destroyed and replace them with input values for self-test.
C       If  COMPAR = TRUE, compare self-test case results with correct
C       answers and restore user input values if test is passed.

C       (See file 'DisORT.Doc' for variable definitions.)

C                 I N T E R N A L    V A R I A B L E S:

C         ACC     Relative accuracy required for passing self-test
C         ERRORn  Relative errors in 'DisORT' output variables
C         OK      Logical variable for determining failure of self-test
C         All variables ending in 'S':  Temporary 'S'torage for input
C+---------------------------------------------------------------------+
      REAL     PMOM( 0:* ), TEMPER( 0:* )
      LOGICAL  COMPAR, DELTAM, LAMBER, PLANCK, OK, ONLYFL, PRNT( * ),
     $     USRANG, USRTAU
      REAL     PMOMS( 0:4 ), TEMPES ( 0:1 )
      LOGICAL  DELTAS, LAMBES, NOPLNS, ONLYFS, PRNTS ( 7 ), USRANS,
     $         USRTAS, TSTBAD
      SAVE
      DATA     ACC / 1.E-4 /

      IF  ( .NOT.COMPAR )  THEN
C                                                 Save user input values
         NLYRS  = NLYR
         DTAUCS = DTAUC
         SSALBS = SSALB
         DO  N = 0, 4
            PMOMS(N) = PMOM(N)
         ENDDO
         NSTRS  = NSTR
         USRANS = USRANG
         NUMUS  = NUMU
         UMUS   = UMU
         USRTAS = USRTAU
         NTAUS  = NTAU
         UTAUS  = UTAU
         NPHIS  = NPHI
         PHIS   = PHI
         IBCNDS = IBCND
         IMTHDS = IMTHD
         FBEAMS = FBEAM
         UMU0S  = UMU0
         PHI0S  = PHI0
         FISOTS = FISOT
         LAMBES = LAMBER
         ALBEDS = ALBEDO
         DELTAS = DELTAM
         ONLYFS = ONLYFL
         ACCURS = ACCUR
         NOPLNS = PLANCK
         WVNMLS = WVNMLO
         WVNMHS = WVNMHI
         BTEMPS = BTEMP
         TTEMPS = TTEMP
         TEMISS = TEMIS
         TEMPES( 0 ) = TEMPER( 0 )
         TEMPES( 1 ) = TEMPER( 1 )
         DO  I = 1, 7
            PRNTS( I ) = PRNT( I )
         ENDDO
C                                         Set input values for self-test
         NLYR  = 1
         DTAUC = 1.0
         SSALB = 0.9
C                                                         Haze-L moments
         PMOM(0) = 1.0
         PMOM(1) = 0.8042
         PMOM(2) = 0.646094
         PMOM(3) = 0.481851
         PMOM(4) = 0.359056
         NSTR   = 4
         USRANG = .TRUE.
         NUMU   = 1
         UMU    = 0.5
         USRTAU = .TRUE.
         NTAU   = 1
         UTAU   = 0.5
         NPHI   = 1
         PHI    = 90.0
         IBCND  = 0
         IMTHD  = 0
         FBEAM  = 3.14159265
         UMU0   = 0.866
         PHI0   = 0.0
         FISOT  = 1.0
         LAMBER = .TRUE.
         ALBEDO = 0.7
         DELTAM = .TRUE.
         ONLYFL = .FALSE.
         ACCUR  = 1.E-4
         PLANCK  = .TRUE.
         WVNMLO = 0.0
         WVNMHI = 50000.
         BTEMP  = 300.0
         TTEMP  = 100.0
         TEMIS  = 0.8
         TEMPER( 0 ) = 210.0
         TEMPER( 1 ) = 200.0
         DO  I = 1, 7
            PRNT( I ) = .FALSE.
         ENDDO
         
      ELSE
           
C        Compare test case results with correct answers and abort if bad

         OK = .TRUE.
         ERROR1 = ( UU  - 47.84218 ) / 47.84218
         ERROR2 = ( RFLDIR - 1.527286 ) / 1.527286
         ERROR3 = ( RFLDN - 28.34313 ) / 28.34313
         ERROR4 = ( FLUP   - 152.5322 ) / 152.5322
         IF( ABS(ERROR1).GT.ACC ) OK = TSTBAD( 'UU',     ERROR1 )
         IF( ABS(ERROR2).GT.ACC ) OK = TSTBAD( 'RFLDIR', ERROR2 )
         IF( ABS(ERROR3).GT.ACC ) OK = TSTBAD( 'RFLDN',  ERROR3 )
         IF( ABS(ERROR4).GT.ACC ) OK = TSTBAD( 'FLUP',   ERROR4 )
         
         IF( .NOT. OK )
     $        CALL ERRMSG( 'DISORT--SELF-TEST FAILED', .TRUE. )

C                                              Restore user input values
         NLYR  = NLYRS
         DTAUC = DTAUCS
         SSALB = SSALBS
         DO  N = 0, 4
            PMOM(N) = PMOMS(N)
         ENDDO
         NSTR   = NSTRS
         USRANG = USRANS
         NUMU   = NUMUS
         UMU    = UMUS
         USRTAU = USRTAS
         NTAU   = NTAUS
         UTAU   = UTAUS
         NPHI   = NPHIS
         PHI    = PHIS
         IBCND  = IBCNDS
         IMTHD  = IMTHDS
         FBEAM  = FBEAMS
         UMU0   = UMU0S
         PHI0   = PHI0S
         FISOT  = FISOTS
         LAMBER = LAMBES
         ALBEDO = ALBEDS
         DELTAM = DELTAS
         ONLYFL = ONLYFS
         ACCUR  = ACCURS
         PLANCK  = NOPLNS
         WVNMLO = WVNMLS
         WVNMHI = WVNMHS
         BTEMP  = BTEMPS
         TTEMP  = TTEMPS
         TEMIS  = TEMISS
         TEMPER( 0 ) = TEMPES( 0 )
         TEMPER( 1 ) = TEMPES( 1 )
         DO  I = 1, 7
            PRNT( I ) = PRNTS( I )
         ENDDO
      END IF
      
      RETURN
      END
      SUBROUTINE  SOLEIG( AMB, APB, ARRAY, CMU, CWT, GL, MI, MAZIM,
     $     MXCMU, NN, NSTR, WK, YLMC, CC, EVECC, EVAL,
     $     KK, GC )

C         SOLVES EIGENVALUE/VECTOR PROBLEM NECESSARY TO CONSTRUCT
C         HOMOGENEOUS PART OF DISCRETE ORDINATE SOLUTION; STWJ(8B)
C         ** NOTE ** EIGENVALUE PROBLEM IS DEGENERATE WHEN SINGLE
C                    SCATTERING ALBEDO = 1;  PRESENT WAY OF DOING IT
C                    SEEMS NUMERICALLY MORE STABLE THAN ALTERNATIVE
C                    METHODS THAT WE TRIED

C     ROUTINES CALLED:  ASYMTX

C   I N P U T     V A R I A B L E S:

C       GL     :  DELTA-M SCALED LEGENDRE COEFFICIENTS OF PHASE FUNCTION
C                    (INCLUDING FACTORS 2L+1 AND SINGLE-SCATTER ALBEDO)
C       CMU    :  COMPUTATIONAL POLAR ANGLE COSINES
C       CWT    :  WEIGHTS FOR QUADRATURE OVER POLAR ANGLE COSINE
C       MAZIM  :  ORDER OF AZIMUTHAL COMPONENT
C       NN     :  HALF THE TOTAL NUMBER OF STREAMS
C       YLMC   :  NORMALIZED ASSOCIATED LEGENDRE POLYNOMIAL
C                 AT THE QUADRATURE ANGLES -CMU-
C       (REMAINDER ARE 'DISORT' INPUT VARIABLES)

C   O U T P U T    V A R I A B L E S:

C       CC     :  CAPITAL-C-SUB-IJ IN EQ. SS(5); NEEDED IN SS(15&18)
C       EVAL   :  -NN- EIGENVALUES OF EQ. SS(12) ON RETURN FROM 'ASYMTX'
C                    BUT THEN SQUARE ROOTS TAKEN
C       EVECC  :  -NN- EIGENVECTORS  (G+) - (G-)  ON RETURN
C                    FROM 'ASYMTX' ( COLUMN J CORRESPONDS TO -EVAL(J)- )
C                    BUT THEN  (G+) + (G-)  IS CALCULATED FROM SS(10),
C                    G+  AND  G-  ARE SEPARATED, AND  G+  IS STACKED ON
C                    TOP OF  G-  TO FORM -NSTR- EIGENVECTORS OF SS(7)
C       GC     :  PERMANENT STORAGE FOR ALL -NSTR- EIGENVECTORS, BUT
C                    IN AN ORDER CORRESPONDING TO -KK-
C       KK     :  PERMANENT STORAGE FOR ALL -NSTR- EIGENVALUES OF SS(7),
C                    BUT RE-ORDERED WITH NEGATIVE VALUES FIRST ( SQUARE
C                    ROOTS OF -EVAL- TAKEN AND NEGATIVES ADDED )

C   I N T E R N A L   V A R I A B L E S:

C       AMB,APB :  MATRICES (ALPHA-BETA), (ALPHA+BETA) IN REDUCED
C                    EIGENVALUE PROBLEM
C       ARRAY   :  COMPLETE COEFFICIENT MATRIX OF REDUCED EIGENVALUE
C                    PROBLEM: (ALFA+BETA)*(ALFA-BETA)
C       GPPLGM  :  (G+) + (G-) (CF. EQS. SS(10-11))
C       GPMIGM  :  (G+) - (G-) (CF. EQS. SS(10-11))
C       WK      :  SCRATCH ARRAY REQUIRED BY 'ASYMTX'
C+---------------------------------------------------------------------+
      REAL    AMB( MI,* ), APB( MI,* ), ARRAY( MI,* ), CC( MXCMU,* ),
     $     CMU(*), CWT(*), EVAL(*), EVECC( MXCMU,* ), GC( MXCMU,* ),
     $     GL(0:*), KK(*), WK(*), YLMC( 0:MXCMU,* )

C                             ** CALCULATE QUANTITIES IN EQS. SS(5-6)
      DO 40 IQ  = 1, NN
         
         DO 20  JQ = 1, NSTR
            SUM = 0.0
            DO 10  L = MAZIM, NSTR-1
               SUM = SUM + GL(L) * YLMC(L,IQ) * YLMC(L,JQ)
 10         CONTINUE
            CC(IQ,JQ) = 0.5 * SUM * CWT(JQ)
 20      CONTINUE

         DO 30  JQ = 1, NN
C                             ** FILL REMAINDER OF ARRAY USING SYMMETRY
C                             ** RELATIONS  C(-MUI,MUJ) = C(MUI,-MUJ)
C                             ** AND        C(-MUI,-MUJ) = C(MUI,MUJ)

            CC(IQ+NN,JQ) = CC(IQ,JQ+NN)
            CC(IQ+NN,JQ+NN) = CC(IQ,JQ)
C                                      ** GET FACTORS OF COEFF. MATRIX
C                                      ** OF REDUCED EIGENVALUE PROBLEM
            ALPHA =   CC(IQ,JQ) / CMU(IQ)
            BETA = CC(IQ,JQ+NN) / CMU(IQ)
            AMB(IQ,JQ) = ALPHA - BETA
            APB(IQ,JQ) = ALPHA + BETA
 30      CONTINUE
         AMB(IQ,IQ) = AMB(IQ,IQ) - 1.0 / CMU(IQ)
         APB(IQ,IQ) = APB(IQ,IQ) - 1.0 / CMU(IQ)

 40   CONTINUE
C                      ** FINISH CALCULATION OF COEFFICIENT MATRIX OF
C                      ** REDUCED EIGENVALUE PROBLEM:  GET MATRIX
C                      ** PRODUCT (ALFA+BETA)*(ALFA-BETA); SS(12)
      DO 70  IQ = 1, NN
         DO 70  JQ = 1, NN
            SUM = 0.
            DO 60  KQ = 1, NN
               SUM = SUM + APB(IQ,KQ) * AMB(KQ,JQ)
 60         CONTINUE
            ARRAY(IQ,JQ) = SUM
70    CONTINUE
C                      ** FIND (REAL) EIGENVALUES AND EIGENVECTORS


      CALL  ASYMTX( ARRAY, EVECC, EVAL, NN, MI, MXCMU, IER, WK )
      
      IF ( IER.GT.0 )  THEN
         WRITE( *, '(//,A,I4,A)' )  ' ASYMTX--EIGENVALUE NO. ', IER,
     $        '  DIDNT CONVERGE.  LOWER-NUMBERED EIGENVALUES WRONG.'
         CALL  ERRMSG( 'ASYMTX--CONVERGENCE PROBLEMS', .TRUE. )
      END IF

CDIR$ IVDEP
      DO 75  IQ = 1, NN
         EVAL(IQ) = SQRT( ABS( EVAL(IQ) ) )
         KK( IQ+NN ) = EVAL(IQ)
C                                             ** ADD NEGATIVE EIGENVALUE
         KK( NN+1-IQ ) = - EVAL(IQ)
75    CONTINUE
C                          ** FIND EIGENVECTORS (G+) + (G-) FROM SS(10)
C                          ** AND STORE TEMPORARILY IN -APB- ARRAY
      DO 90  JQ = 1, NN
         DO 90  IQ = 1, NN
            SUM = 0.
            DO 80  KQ = 1,NN
               SUM = SUM + AMB(IQ,KQ) * EVECC(KQ,JQ)
 80         CONTINUE
            APB(IQ,JQ) = SUM / EVAL(JQ)
 90   CONTINUE

      DO 100  JQ = 1, NN
CDIR$ IVDEP
         DO 100  IQ = 1, NN
            GPPLGM = APB(IQ,JQ)
            GPMIGM = EVECC(IQ,JQ)
C                                ** RECOVER EIGENVECTORS G+,G- FROM
C                                   THEIR SUM AND DIFFERENCE; STACK THEM
C                                   TO GET EIGENVECTORS OF FULL SYSTEM
C                                   SS(7) (JQ = EIGENVECTOR NUMBER)

            EVECC(IQ,      JQ) = 0.5 * ( GPPLGM + GPMIGM )
            EVECC(IQ+NN,   JQ) = 0.5 * ( GPPLGM - GPMIGM )

C                                ** EIGENVECTORS CORRESPONDING TO
C                                ** NEGATIVE EIGENVALUES (CORRESP. TO
C                                ** REVERSING SIGN OF 'K' IN SS(10) )
            GPPLGM = - GPPLGM
            EVECC(IQ,   JQ+NN) = 0.5 * ( GPPLGM + GPMIGM )
            EVECC(IQ+NN,JQ+NN) = 0.5 * ( GPPLGM - GPMIGM )
            GC( IQ+NN,   JQ+NN )   = EVECC( IQ,    JQ )
            GC( NN+1-IQ, JQ+NN )   = EVECC( IQ+NN, JQ )
            GC( IQ+NN,   NN+1-JQ ) = EVECC( IQ,    JQ+NN )
            GC( NN+1-IQ, NN+1-JQ ) = EVECC( IQ+NN, JQ+NN )
100   CONTINUE

      RETURN
      END
      SUBROUTINE  SOLVE0( B, BDR, BEM, BPLANCK, CBAND, CMU, CWT, EXPBEA,
     $     FBEAM, FISOT, IPVT, LAMBER, LL, LYRCUT,
     $     MAZIM, MI, MI9M2, MXCMU, NCOL, NCUT, NN, NSTR,
     $     NNLYRI, PI, TPLANCK, TAUCPR, UMU0, XRA, Z, ZZ,
     $     ZPLK0, ZPLK1 )

C        CONSTRUCT RIGHT-HAND SIDE VECTOR -B- FOR GENERAL BOUNDARY
C        CONDITIONS STWJ(17) AND SOLVE SYSTEM OF EQUATIONS OBTAINED
C        FROM THE BOUNDARY CONDITIONS AND THE
C        CONTINUITY-OF-INTENSITY-AT-LAYER-INTERFACE EQUATIONS.
C        THERMAL EMISSION CONTRIBUTES ONLY IN AZIMUTHAL INDEPENDENCE.

C     ROUTINES CALLED:  SGBCO, SGBSL, ZEROIT

C     I N P U T      V A R I A B L E S:

C       BDR      :  SURFACE BIDIRECTIONAL REFLECTIVITY
C       BEM      :  SURFACE BIDIRECTIONAL EMISSIVITY
C       BPLANCK   :  BOTTOM BOUNDARY THERMAL EMISSION
C       CBAND    :  LEFT-HAND SIDE MATRIX OF LINEAR SYSTEM EQ. SC(5),
C                   SCALED BY EQ. SC(12); IN BANDED FORM REQUIRED
C                   BY LINPACK SOLUTION ROUTINES
C       CMU      :  ABSCISSAE FOR GAUSS QUADRATURE OVER ANGLE COSINE
C       CWT      :  WEIGHTS FOR GAUSS QUADRATURE OVER ANGLE COSINE
C       EXPBEA   :  TRANSMISSION OF INCIDENT BEAM, EXP(-TAUCPR/UMU0)
C       LYRCUT   :  LOGICAL FLAG FOR TRUNCATION OF COMPUT. LAYER
C       MAZIM    :  ORDER OF AZIMUTHAL COMPONENT
C       NCOL     :  COUNTS OF COLUMNS IN -CBAND-
C       NN       :  ORDER OF DOUBLE-GAUSS QUADRATURE (NSTR/2)
C       NCUT     :  TOTAL NUMBER OF COMPUTATIONAL LAYERS CONSIDERED
C       TPLANCK   :  TOP BOUNDARY THERMAL EMISSION
C       TAUCPR   :  CUMULATIVE OPTICAL DEPTH (DELTA-M-SCALED)
C       XRA     :  thermal source expansion, alpha in Eq. KS(15)
C       ZZ       :  BEAM SOURCE VECTORS IN EQ. SS(19)
C       ZPLK0    :  THERMAL SOURCE VECTORS -Z0-, BY SOLVING EQ. SS(16)
C       ZPLK1    :  THERMAL SOURCE VECTORS -Z1-, BY SOLVING EQ. SS(16)
C       (REMAINDER ARE 'DISORT' INPUT VARIABLES)

C   O U T P U T     V A R I A B L E S:

C       B        :  RIGHT-HAND SIDE VECTOR OF EQ. SC(5) GOING INTO
C                   *SGBSL*; RETURNS AS SOLUTION VECTOR OF EQ.
C                   SC(12), CONSTANTS OF INTEGRATION WITHOUT
C                   EXPONENTIAL TERM
C      LL        :  PERMANENT STORAGE FOR -B-, BUT RE-ORDERED

C   I N T E R N A L    V A R I A B L E S:

C       IPVT     :  INTEGER VECTOR OF PIVOT INDICES
C       IT       :  POINTER FOR POSITION IN  -B-
C       NCD      :  NUMBER OF DIAGONALS BELOW OR ABOVE MAIN DIAGONAL
C       RCOND    :  INDICATOR OF SINGULARITY FOR -CBAND-
C       Z        :  SCRATCH ARRAY REQUIRED BY *SGBCO*
C+---------------------------------------------------------------------+

      LOGICAL  LAMBER, LYRCUT
      INTEGER  IPVT(*)
      REAL     B(*), BDR( MI,0:* ), BEM(*), CBAND( MI9M2,NNLYRI ),
     $     CMU(*), CWT(*), EXPBEA(0:*), LL( MXCMU,* ),
     $     TAUCPR( 0:* ), XRA(*), Z(*), ZZ( MXCMU,* ), 
     $     ZPLK0( MXCMU,* ), ZPLK1( MXCMU,* )


      CALL  ZEROIT( B, NNLYRI )
C                             ** CONSTRUCT -B-,  STWJ(20A,C) FOR
C                             ** PARALLEL BEAM + BOTTOM REFLECTION +
C                             ** THERMAL EMISSION AT TOP AND/OR BOTTOM

      IF ( MAZIM.GT.0 .AND. FBEAM.GT.0.0 )  THEN

C                                         ** AZIMUTH-DEPENDENT CASE
C                                         ** (NEVER CALLED IF FBEAM = 0)
         IF ( LYRCUT .OR. LAMBER ) THEN

C               ** NO AZIMUTHAL-DEPENDENT INTENSITY FOR LAMBERT SURFACE;
C               ** NO INTENSITY COMPONENT FOR TRUNCATED BOTTOM LAYER

            DO 10  IQ = 1, NN
C                                                     ** TOP BOUNDARY
               B(IQ) = - ZZ(NN+1-IQ,1)
C                                                  ** BOTTOM BOUNDARY
               B(NCOL-NN+IQ) = - ZZ(IQ+NN,NCUT) * EXPBEA(NCUT)
 10         CONTINUE

         ELSE

            DO 20  IQ = 1, NN
               B(IQ) = - ZZ(NN+1-IQ,1)
               
               SUM = 0.
               DO 15  JQ = 1, NN
                  SUM = SUM + CWT(JQ) * CMU(JQ) * BDR(IQ,JQ)
     $                        * ZZ(NN+1-JQ,NCUT) * EXPBEA(NCUT)
 15            CONTINUE
	         B(NCOL-NN+IQ) = SUM
	         IF ( FBEAM.GT.0.0 )
     $                B(NCOL-NN+IQ) = SUM + ( BDR(IQ,0) * UMU0*FBEAM/PI
     $                - ZZ(IQ+NN,NCUT) ) * EXPBEA(NCUT)
 20           CONTINUE
	   END IF
C                             ** CONTINUITY CONDITION FOR LAYER
C                             ** INTERFACES OF EQ. STWJ(20B)
	   IT = NN
	   DO 40  LC = 1, NCUT-1
	      DO 30  IQ = 1, NSTR
	         IT    = IT + 1
	         B(IT) = ( ZZ(IQ,LC+1) - ZZ(IQ,LC) ) * EXPBEA(LC)
 30           CONTINUE
 40        CONTINUE

	ELSE
C                                   ** AZIMUTH-INDEPENDENT CASE
	   IF ( FBEAM.EQ.0.0 )  THEN

	      DO 50 IQ = 1, NN
C                                      ** TOP BOUNDARY

	         B(IQ) = - ZPLK0(NN+1-IQ,1) + FISOT + TPLANCK
 50           CONTINUE

	      IF ( LYRCUT ) THEN
C                               ** NO INTENSITY COMPONENT FOR TRUNCATED
C                               ** BOTTOM LAYER
	         DO 60 IQ = 1, NN
C                                      ** BOTTOM BOUNDARY

	            B(NCOL-NN+IQ) =  
     $                   -  EXP( -XRA(NCUT)*TAUCPR(NCUT) ) * 
     $                   ( ZPLK0(IQ+NN,NCUT) + 
     $                   ZPLK1(IQ+NN,NCUT) * TAUCPR(NCUT) )
 60              CONTINUE

	      ELSE

	         DO 80 IQ = 1, NN

	            SUM = 0.
	            DO 70 JQ = 1, NN
	               SUM = SUM + CWT(JQ) * CMU(JQ) * BDR(IQ,JQ)
     $                      * (  EXP(-XRA(NCUT)*TAUCPR(NCUT) ) *  
     $                      (ZPLK0(NN+1-JQ,NCUT) + 
     $                      ZPLK1(NN+1-JQ,NCUT)*TAUCPR(NCUT) ) ) 
 70                 CONTINUE
	            B(NCOL-NN+IQ) = 2.*SUM + BEM(IQ) * BPLANCK
     $                   - EXP( -XRA(NCUT)*TAUCPR(NCUT) ) * 
     $                   (ZPLK0(IQ+NN,NCUT) + 
     $                   ZPLK1(IQ+NN,NCUT)*TAUCPR(NCUT) ) 
 80              CONTINUE
	      END IF
C                             ** CONTINUITY CONDITION FOR LAYER
C                             ** INTERFACES, STWJ(20B)
	      IT = NN
	      DO 100  LC = 1, NCUT-1
	         DO 90  IQ = 1, NSTR
	            IT    = IT + 1
	            B(IT) = 
     $                   +  EXP( -XRA(LC+1)*TAUCPR(LC) ) * 
     $                   ( ZPLK0(IQ,LC+1) + 
     $                   ZPLK1(IQ,LC+1)*TAUCPR(LC) ) 
     $                   -  EXP( -XRA(LC)*TAUCPR(LC) ) * 
     $                   ( ZPLK0(IQ,LC) + 
     $                   ZPLK1(IQ,LC)*TAUCPR(LC) ) 
 90              CONTINUE
 100          CONTINUE

	   ELSE

	      DO 150 IQ = 1, NN
	         B(IQ) = - ZZ(NN+1-IQ,1) - 
     $                ZPLK0(NN+1-IQ,1) +FISOT +TPLANCK
 150          CONTINUE
              
	      IF ( LYRCUT ) THEN

	         DO 160 IQ = 1, NN
	            B(NCOL-NN+IQ) = - ZZ(IQ+NN,NCUT) * EXPBEA(NCUT)
     $                   - EXP( -XRA(NCUT)*TAUCPR(NCUT) ) *
     $                   ( ZPLK0(IQ+NN,NCUT) + 
     $                   ZPLK1(IQ+NN,NCUT)*TAUCPR(NCUT) )  
 160             CONTINUE

	      ELSE

	         DO 180 IQ = 1, NN

	            SUM = 0.
	            DO 170 JQ = 1, NN
	               SUM = SUM + CWT(JQ) * CMU(JQ) * BDR(IQ,JQ)
     $                      * ( ZZ(NN+1-JQ,NCUT) * EXPBEA(NCUT)
     $                      + EXP(-XRA(NCUT)*TAUCPR(NCUT) ) * 
     $                      (ZPLK0(NN+1-JQ,NCUT) + 
     $                      ZPLK1(NN+1-JQ,NCUT)*TAUCPR(NCUT)))  
 170                CONTINUE
	            B(NCOL-NN+IQ) = 2.*SUM + (BDR(IQ,0) * UMU0*FBEAM/PI
     $                   - ZZ(IQ+NN,NCUT) ) * EXPBEA(NCUT)
     $                   + BEM(IQ) * BPLANCK
     $                   - EXP( -XRA(NCUT)*TAUCPR(NCUT) ) * 
     $                   (ZPLK0(IQ+NN,NCUT) + 
     $                   ZPLK1(IQ+NN,NCUT)*TAUCPR(NCUT)) 
 180             CONTINUE
	      END IF

	      IT = NN
	      DO 200  LC = 1, NCUT-1
	         DO 190  IQ = 1, NSTR
	            IT    = IT + 1
	            B(IT) = ( ZZ(IQ,LC+1) - ZZ(IQ,LC) ) * EXPBEA(LC)
     $                   + EXP( -XRA(LC+1)*TAUCPR(LC) ) * 
     $                   ( ZPLK0(IQ,LC+1) + 
     $                   ZPLK1(IQ,LC+1)*TAUCPR(LC) ) 
     $                   - EXP( -XRA(LC)*TAUCPR(LC) ) * 
     $                   ( ZPLK0(IQ,LC) + 
     $                   ZPLK1(IQ,LC)*TAUCPR(LC)) 
 190             CONTINUE
 200          CONTINUE

	   END IF

	END IF
C                     ** FIND L-U (LOWER/UPPER TRIANGULAR) DECOMPOSITION
C                     ** OF BAND MATRIX -CBAND- AND TEST IF IT IS NEARLY
C                     ** SINGULAR (NOTE: -CBAND- IS DESTROYED)
C                     ** (-CBAND- IS IN LINPACK PACKED FORMAT)
	RCOND = 0.0
	NCD = 3*NN - 1
	CALL  SGBCO( CBAND, MI9M2, NCOL, NCD, NCD, IPVT, RCOND, Z )
	IF ( 1.0+RCOND .EQ. 1.0 )  CALL  ERRMSG
     $       ( 'SOLVE0--SGBCO SAYS MATRIX NEAR SINGULAR',.FALSE.)

C                   ** SOLVE LINEAR SYSTEM WITH COEFF MATRIX -CBAND-
C                   ** AND R.H. SIDE(S) -B- AFTER -CBAND- HAS BEEN L-U
C                   ** DECOMPOSED.  SOLUTION IS RETURNED IN -B-.

	CALL  SGBSL( CBAND, MI9M2, NCOL, NCD, NCD, IPVT, B, 0 )

C                   ** ZERO -CBAND- (IT MAY CONTAIN 'FOREIGN'
C                   ** ELEMENTS UPON RETURNING FROM LINPACK);
C                   ** NECESSARY TO PREVENT ERRORS

	CALL  ZEROIT( CBAND, MI9M2*NNLYRI )

	DO 220  LC = 1, NCUT
	   IPNT = LC*NSTR - NN
	   DO 220  IQ = 1, NN
	      LL(NN+1-IQ,LC) = B(IPNT+1-IQ)
	      LL(IQ+NN,  LC) = B(IQ+IPNT)
220   CONTINUE

      RETURN
      END
      SUBROUTINE  SOLVE1( B, CBAND, FISOT, IHOM, IPVT, LL, MI9M2, MXCMU,
     $     NCOL, NCUT, NN, NNLYRI, NSTR, Z )

C        CONSTRUCT RIGHT-HAND SIDE VECTOR -B- FOR ISOTROPIC INCIDENCE
C        (ONLY) ON EITHER TOP OR BOTTOM BOUNDARY AND SOLVE SYSTEM
C        OF EQUATIONS OBTAINED FROM THE BOUNDARY CONDITIONS AND THE
C        CONTINUITY-OF-INTENSITY-AT-LAYER-INTERFACE EQUATIONS

C     ROUTINES CALLED:  SGBCO, SGBSL, ZEROIT

C     I N P U T      V A R I A B L E S:

C       CBAND    :  LEFT-HAND SIDE MATRIX OF LINEAR SYSTEM EQ. SC(5),
C                   SCALED BY EQ. SC(12); IN BANDED FORM REQUIRED
C                   BY LINPACK SOLUTION ROUTINES
C       IHOM     :  DIRECTION OF ILLUMINATION FLAG
C       NCOL     :  COUNTS OF COLUMNS IN -CBAND-
C       NN       :  ORDER OF DOUBLE-GAUSS QUADRATURE (NSTR/2)
C       (REMAINDER ARE 'DISORT' INPUT VARIABLES)

C    O U T P U T     V A R I A B L E S:

C       B        :  RIGHT-HAND SIDE VECTOR OF EQ. SC(5) GOING INTO
C                   *SGBSL*; RETURNS AS SOLUTION VECTOR OF EQ.
C                   SC(12), CONSTANTS OF INTEGRATION WITHOUT
C                   EXPONENTIAL TERM
C       LL      :   PERMANENT STORAGE FOR -B-, BUT RE-ORDERED

C   I N T E R N A L    V A R I A B L E S:

C       IPVT     :  INTEGER VECTOR OF PIVOT INDICES
C       NCD      :  NUMBER OF DIAGONALS BELOW OR ABOVE MAIN DIAGONAL
C       RCOND    :  INDICATOR OF SINGULARITY FOR -CBAND-
C       Z        :  SCRATCH ARRAY REQUIRED BY *SGBCO*
C----------------------------------------------------------------------+
      INTEGER  IPVT(*)
      REAL     B( NNLYRI ), CBAND( MI9M2,NNLYRI ), LL( MXCMU,* ), Z(*)

      CALL  ZEROIT( B, NNLYRI )
      NCD = 3*NN - 1

      IF ( IHOM.EQ.1 )  THEN
C                             ** BECAUSE THERE ARE NO BEAM OR EMISSION
C                             ** SOURCES, REMAINDER OF -B- ARRAY IS ZERO
         DO 10  I = 1, NN
            B(I) = FISOT
            B( NCOL-NN+I ) = 0.0
 10      CONTINUE

         RCOND = 0.0
         CALL  SGBCO( CBAND, MI9M2, NCOL, NCD, NCD, IPVT, RCOND, Z )
         IF ( 1.0+RCOND .EQ. 1.0 )  CALL  ERRMSG
     $        ( 'SOLVE1--SGBCO SAYS MATRIX NEAR SINGULAR', .FALSE. )
         
      ELSE IF ( IHOM.EQ.2 )  THEN
         
         DO 20 I = 1, NN
            B(I) = 0.0
            B( NCOL-NN+I ) = FISOT
 20      CONTINUE

      END IF

      CALL  SGBSL( CBAND, MI9M2, NCOL, NCD, NCD, IPVT, B, 0 )

C                          ** ZERO -CBAND- TO GET RID OF 'FOREIGN'
C                          ** ELEMENTS PUT IN BY LINPACK
      DO 30  LC = 1, NCUT
         IPNT = LC*NSTR - NN
         DO 30  IQ = 1, NN
            LL( NN+1-IQ, LC) = B( IPNT+1-IQ )
            LL( IQ+NN,   LC) = B( IQ+IPNT )
30    CONTINUE

      RETURN
      END
      SUBROUTINE  SPALTR( CMU, CWT, GC, KK, LL, MXCMU, NLYR,
     $     NN, NSTR, TAUCPR, SFLUP, SFLDN )

C       CALCULATES SPHERICAL ALBEDO AND TRANSMISSIVITY FOR THE ENTIRE
C       MEDIUM FROM THE M=0 INTENSITY COMPONENTS
C       (THIS IS A VERY SPECIALIZED VERSION OF 'FLUXES')

C    I N P U T    V A R I A B L E S:

C       CMU     :  ABSCISSAE FOR GAUSS QUADRATURE OVER ANGLE COSINE
C       CWT     :  WEIGHTS FOR GAUSS QUADRATURE OVER ANGLE COSINE
C       KK      :  EIGENVALUES OF COEFF. MATRIX IN EQ. SS(7)
C       GC      :  EIGENVECTORS AT POLAR QUADRATURE ANGLES, SC(1)
C       LL      :  CONSTANTS OF INTEGRATION IN EQ. SC(1), OBTAINED
C                  BY SOLVING SCALED VERSION OF EQ. SC(5);
C                  EXPONENTIAL TERM OF EQ. SC(12) NOT INCLUDED
C       NN      :  ORDER OF DOUBLE-GAUSS QUADRATURE (NSTR/2)
C       (REMAINDER ARE 'DISORT' INPUT VARIABLES)

C    O U T P U T   V A R I A B L E S:

C       SFLUP   :  UP-FLUX AT TOP (EQUIVALENT TO SPHERICAL ALBEDO DUE TO
C                  RECIPROCITY).  FOR ILLUMINATION FROM BELOW IT GIVES
C                  SPHERICAL TRANSMISSIVITY
C       SFLDN   :  DOWN-FLUX AT BOTTOM (FOR SINGLE LAYER
C                  EQUIVALENT TO SPHERICAL TRANSMISSIVITY
C                  DUE TO RECIPROCITY)

C    I N T E R N A L   V A R I A B L E S:

C       ZINT    :  INTENSITY OF M=0 CASE, IN EQ. SC(1)
C+----------------------------------------------------------------------

      REAL  CMU(*), CWT(*), GC( MXCMU,MXCMU,* ), KK( MXCMU,* ),
     $     LL( MXCMU,* ), TAUCPR( 0:* )


      SFLUP = 0.0
      DO 20  IQ = NN+1, NSTR
         ZINT  = 0.0
         DO 10   JQ = 1, NN
            ZINT = ZINT + GC(IQ,JQ,1) * LL(JQ,1) *
     $           EXP( KK(JQ,1) * TAUCPR(1) )
 10      CONTINUE
         DO 11  JQ = NN+1, NSTR
            ZINT = ZINT + GC(IQ,JQ,1) * LL(JQ,1)
 11      CONTINUE

         SFLUP = SFLUP + CWT(IQ-NN) * CMU(IQ-NN) * ZINT
 20   CONTINUE

      SFLDN  = 0.0
      DO 40  IQ = 1, NN
         ZINT   = 0.0
         DO 30  JQ = 1, NN
            ZINT = ZINT + GC(IQ,JQ,NLYR) * LL(JQ,NLYR)
 30      CONTINUE
         DO 31  JQ = NN+1, NSTR
            ZINT = ZINT + GC(IQ,JQ,NLYR) * LL(JQ,NLYR) *
     $           EXP( - KK(JQ,NLYR)*(TAUCPR(NLYR) - TAUCPR(NLYR-1)) )
 31      CONTINUE
         
         SFLDN = SFLDN + CWT(NN+1-IQ) * CMU(NN+1-IQ) * ZINT
 40   CONTINUE

      SFLUP = 2.0 * SFLUP
      SFLDN = 2.0 * SFLDN
      
      RETURN
      END
      SUBROUTINE  SURFAC( ALBEDO, DELM0, FBEAM, HLPR, LAMBER,
     $     MI, MAZIM, MXCMU, MXUMU, NN, NUMU, NSTR, 
     $     ONLYFL, UMU, USRANG, YLM0, YLMC, YLMU, BDR, 
     $     EMU, BEM, RMU )

C       SPECIFIES USER'S SURFACE BIDIRECTIONAL PROPERTIES, STWJ(21)

C   I N P U T     V A R I A B L E S:

C       DELM0  :  KRONECKER DELTA, DELTA-SUB-M0
C       HLPR   :  LEGENDRE MOMENTS OF SURFACE BIDIRECTIONAL REFLECTIVITY
C                    (WITH 2K+1 FACTOR INCLUDED)
C       MAZIM  :  ORDER OF AZIMUTHAL COMPONENT
C       NN     :  ORDER OF DOUBLE-GAUSS QUADRATURE (NSTR/2)
C       YLM0   :  NORMALIZED ASSOCIATED LEGENDRE POLYNOMIAL
C                 AT THE BEAM ANGLE
C       YLMC   :  NORMALIZED ASSOCIATED LEGENDRE POLYNOMIALS
C                 AT THE QUADRATURE ANGLES
C       YLMU   :  NORMALIZED ASSOCIATED LEGENDRE POLYNOMIALS
C                 AT THE USER ANGLES
C       (REMAINDER ARE 'DISORT' INPUT VARIABLES)

C    O U T P U T     V A R I A B L E S:

C       BDR :  SURFACE BIDIRECTIONAL REFLECTIVITY (COMPUTATIONAL ANGLES)
C       RMU :  SURFACE BIDIRECTIONAL REFLECTIVITY (USER ANGLES)
C       BEM :  SURFACE DIRECTIONAL EMISSIVITY (COMPUTATIONAL ANGLES)
C       EMU :  SURFACE DIRECTIONAL EMISSIVITY (USER ANGLES)

C    I N T E R N A L     V A R I A B L E S:

C       DREF      DIRECTIONAL REFLECTIVITY
C       NMUG   :  NUMBER OF ANGLE COSINE QUADRATURE POINTS
C                 ON (0,1) FOR INTEGRATING BIDIRECTIONAL REFLECTIVITY
C                 TO GET DIRECTIONAL EMISSIVITY (IT IS NECESSARY TO USE
C                 A QUADRATURE SET DISTINCT FROM THE COMPUTATIONAL
C                 ANGLES, BECAUSE THE COMPUTATIONAL ANGLES MAY NOT BE
C                 DENSE ENOUGH -- I.E. 'NSTR' MAY BE TOO SMALL-- TO GIVE
C                 AN ACCURATE APPROXIMATION FOR THE INTEGRATION).
C       GMU    :  THE 'NMUG' ANGLE COSINE QUADRATURE POINTS ON (0,1)
C       GWT    :  THE 'NMUG' ANGLE COSINE QUADRATURE WEIGHTS ON (0,1)
C       YLMG   :  NORMALIZED ASSOCIATED LEGENDRE POLYNOMIALS
C                 AT THE 'NMUG' QUADRATURE ANGLES
C+---------------------------------------------------------------------+
      LOGICAL  LAMBER, ONLYFL, USRANG
      REAL     BDR( MI,0:* ), BEM(*), EMU(*),
     $     HLPR(0:*), RMU( MXUMU,0:* ), UMU(*),
     $     YLM0(0:*), YLMC( 0:MXCMU,* ), YLMU( 0:MXCMU,* )
      PARAMETER  ( NMUG = 10, MAXSTR = 100 )
      LOGICAL  PASS1
      REAL     GMU( NMUG ), GWT( NMUG ), YLMG( 0:MAXSTR, NMUG )
      SAVE  PASS1, GMU, GWT, YLMG
      DATA  PASS1 / .TRUE. /

      IF ( PASS1 )  THEN
         PASS1 = .FALSE.
         CALL QGAUSN( NMUG, GMU, GWT )
         
         CALL LEPOLY( NMUG, 0, MAXSTR, MAXSTR, GMU, YLMG )
C                    ** CONVERT LEGENDRE POLYS. TO NEGATIVE -GMU-
         SGN  = - 1.0
         DO 1  K = 0, MAXSTR
            SGN = - SGN
            DO 1  JG = 1, NMUG
               YLMG( K,JG ) = SGN * YLMG( K,JG )
 1       CONTINUE

      END IF

      CALL  ZEROIT( BDR, MI*(MI+1) )
      CALL  ZEROIT( BEM, MI )
      
      IF ( LAMBER .AND. MAZIM.EQ.0 ) THEN
         
         DO 20 IQ = 1, NN
            BEM(IQ) = 1.0 - ALBEDO
            DO 20 JQ = 0, NN
               BDR(IQ,JQ) = ALBEDO
20       CONTINUE

      ELSE IF ( .NOT.LAMBER ) THEN
C                                  ** COMPUTE SURFACE BIDIRECTIONAL
C                                  ** PROPERTIES AT COMPUTATIONAL ANGLES
         DO 60 IQ = 1, NN
            
            DO 40 JQ = 1, NN
               SUM = 0.0
               DO 30 K = MAZIM, NSTR-1
                  SUM = SUM + HLPR(K) * YLMC(K,IQ) * YLMC(K,JQ+NN)
 30            CONTINUE
               BDR(IQ,JQ) = (2.-DELM0) * SUM
 40         CONTINUE

            IF ( FBEAM.GT.0.0 )  THEN
               SUM = 0.0
               DO 50 K = MAZIM, NSTR-1
                  SUM = SUM + HLPR(K) * YLMC(K,IQ) * YLM0(K)
 50            CONTINUE
               BDR(IQ,0) = (2.-DELM0) * SUM
            ENDIF

 60      CONTINUE

         IF ( MAZIM.EQ.0 ) THEN

            IF ( NSTR.GT.MAXSTR )  CALL
     $           ERRMSG( 'SURFAC--PARAMETER MAXSTR TOO SMALL', .TRUE. )

C                              ** INTEGRATE BIDIRECTIONAL REFLECTIVITY
C                              ** AT REFLECTION POLAR ANGLES -CMU- AND
C                              ** INCIDENT ANGLES -GMU- TO GET
C                              ** DIRECTIONAL EMISSIVITY AT
C                              ** COMPUTATIONAL ANGLES -CMU-.
            DO 100  IQ = 1, NN
               DREF = 0.0
               DO 90  JG = 1, NMUG
                  SUM = 0.0
                  DO 80  K = 0, NSTR-1
                     SUM = SUM + HLPR(K) * YLMC(K,IQ) * YLMG(K,JG)
 80               CONTINUE
                  DREF = DREF + 2.* GWT(JG) * GMU(JG) * SUM
 90            CONTINUE
               BEM(IQ) = 1.0 - DREF
 100        CONTINUE

         END IF

      END IF
C                                       ** COMPUTE SURFACE BIDIRECTIONAL
C                                       ** PROPERTIES AT USER ANGLES

      IF ( .NOT.ONLYFL .AND. USRANG )  THEN
         
         CALL  ZEROIT( EMU, MXUMU )
         CALL  ZEROIT( RMU, MXUMU*(MI+1) )
         
         DO 170 IU = 1, NUMU
            IF ( UMU(IU).GT.0.0 )  THEN
               
               IF ( LAMBER .AND. MAZIM.EQ.0 )  THEN
                  DO 110 IQ = 0, NN
                     RMU(IU,IQ) = ALBEDO
 110              CONTINUE
                  EMU(IU) = 1.0 - ALBEDO
                  
               ELSE IF ( .NOT.LAMBER ) THEN
                  DO 130 IQ = 1, NN
                     SUM = 0.0
                     DO 120 K = MAZIM, NSTR-1
                        SUM = SUM + HLPR(K) * YLMU(K,IU) * YLMC(K,IQ+NN)
 120                 CONTINUE
                     RMU(IU,IQ) = (2.-DELM0) * SUM
 130              CONTINUE
                  
                  IF ( FBEAM.GT.0.0 )  THEN
                     SUM = 0.0
                     DO 140 K = MAZIM, NSTR-1
                        SUM = SUM + HLPR(K) * YLMU(K,IU) * YLM0(K)
 140                 CONTINUE
                     RMU(IU,0) = (2.-DELM0) * SUM
                  END IF
                  
                  IF ( MAZIM.EQ.0 ) THEN
                     
C                               ** INTEGRATE BIDIRECTIONAL REFLECTIVITY
C                               ** AT REFLECTION ANGLES -UMU- AND
C                               ** INCIDENT ANGLES -GMU- TO GET
C                               ** DIRECTIONAL EMISSIVITY AT
C                               ** USER ANGLES -UMU-.
                     DREF = 0.0
                     DO 160 JG = 1, NMUG
                        SUM = 0.0
                        DO 150 K = 0, NSTR-1
                           SUM = SUM+ HLPR(K) * YLMU(K,IU) * YLMG(K,JG)
 150                    CONTINUE
                        DREF = DREF + 2.* GWT(JG) * GMU(JG) * SUM
 160                 CONTINUE

                     EMU(IU) = 1.0 - DREF
                  END IF
                  
               END IF
            END IF
 170     CONTINUE

      END IF

      RETURN
      END
      SUBROUTINE  TERPEV( CWT, EVECC, GL, GU, MAZIM, MXCMU, MXUMU,
     $     NN, NSTR, NUMU, WK, YLMC, YLMU )

C         INTERPOLATE EIGENVECTORS TO USER ANGLES; EQ SD(8)

      REAL  CWT(*), EVECC( MXCMU,* ), GL(0:*), GU(  MXUMU,* ), WK(*),
     $     YLMC(  0:MXCMU,* ), YLMU(  0:MXCMU,* )


      DO 50  IQ = 1, NSTR

         DO 20  L = MAZIM, NSTR-1
C                                       ** INNER SUM IN SD(8) TIMES ALL
C                                   ** FACTORS IN OUTER SUM BUT PLM(MU)
            SUM = 0.0
            DO 10  JQ = 1, NSTR
               SUM = SUM + CWT(JQ) * YLMC(L,JQ) * EVECC(JQ,IQ)
 10         CONTINUE
            WK(L+1) = 0.5 * GL(L) * SUM
 20      CONTINUE
C                                    ** FINISH OUTER SUM IN SD(8)
C                                    ** AND STORE EIGENVECTORS
         DO 40  IU = 1, NUMU
            SUM = 0.
            DO 30  L = MAZIM, NSTR-1
               SUM = SUM + WK(L+1) * YLMU(L,IU)
 30         CONTINUE
            IF ( IQ.LE.NN )  GU( IU, IQ+NN     ) = SUM
            IF ( IQ.GT.NN )  GU( IU, NSTR+1-IQ ) = SUM
 40      CONTINUE

 50   CONTINUE

      RETURN
      END
      SUBROUTINE  TERPSO( CWT, DELM0, FBEAM, GL, MAZIM, MXCMU,
     $     PLANCK, NUMU, NSTR, OPRIM, PI, YLM0, YLMC,
     $     YLMU, PSI0, PSI1, XR0, XR1, Z0, Z1, ZJ, 
     $     ZBEAM, Z0U, Z1U )
      
C         INTERPOLATES SOURCE FUNCTIONS TO USER ANGLES

C    I N P U T      V A R I A B L E S:

C       CWT    :  WEIGHTS FOR GAUSS QUADRATURE OVER ANGLE COSINE
C       DELM0  :  KRONECKER DELTA, DELTA-SUB-M0
C       GL     :  DELTA-M SCALED LEGENDRE COEFFICIENTS OF PHASE FUNCTION
C                    (INCLUDING FACTORS 2L+1 AND SINGLE-SCATTER ALBEDO)
C       MAZIM  :  ORDER OF AZIMUTHAL COMPONENT
C       OPRIM  :  SINGLE SCATTERING ALBEDO
C       XR0    :  EXPANSION OF THERMAL SOURCE FUNCTION
C       XR1    :  EXPANSION OF THERMAL SOURCE FUNCTION EQS. KS(15)
C       YLM0   :  NORMALIZED ASSOCIATED LEGENDRE POLYNOMIAL
C                 AT THE BEAM ANGLE
C       YLMC   :  NORMALIZED ASSOCIATED LEGENDRE POLYNOMIAL
C                 AT THE QUADRATURE ANGLES
C       YLMU   :  NORMALIZED ASSOCIATED LEGENDRE POLYNOMIAL
C                 AT THE USER ANGLES
C       Z0     :  SOLUTION VECTORS Y-SUB-ZERO OF EQ. KS(16)
C       Z1     :  SOLUTION VECTORS Y-SUB-ONE OF EQ. KS(16)
C       ZJ     :  SOLUTION VECTOR CAPITAL -Z-SUB-ZERO AFTER SOLVING
C                 EQ. SS(19)
C       (REMAINDER ARE 'DISORT' INPUT VARIABLES)

C    O U T P U T     V A R I A B L E S:

C       ZBEAM  :  INCIDENT-BEAM SOURCE FUNCTION AT USER ANGLES
C       Z0U,Z1U:  COMPONENTS OF A LINEAR-IN-OPTICAL-DEPTH-DEPENDENT
C                    SOURCE (APPROXIMATING THE PLANCK EMISSION SOURCE)

C   I N T E R N A L       V A R I A B L E S:

C       PSI    :  SUM JUST AFTER SQUARE BRACKET IN  EQ. SD(9)
C+---------------------------------------------------------------------+
      LOGICAL  PLANCK
      REAL   CWT(*), GL(0:*), PSI0(*), PSI1(*), YLM0(0:*), 
     $     YLMC( 0:MXCMU,* ), YLMU( 0:MXCMU,*), Z0(*), Z1(*),
     $     ZJ(*), ZBEAM(*), Z0U(*), Z1U(*)


      IF ( FBEAM.GT.0.0 )  THEN
C                                  ** BEAM SOURCE TERMS; EQ. SD(9)
         DO 20  IQ = MAZIM, NSTR-1
            PSUM = 0.
            DO 10  JQ = 1, NSTR
               PSUM = PSUM + CWT(JQ) * YLMC(IQ,JQ) * ZJ(JQ)
 10         CONTINUE
            PSI0(IQ+1) = 0.5 * GL(IQ) * PSUM
 20      CONTINUE

         FACT = ( 2. - DELM0 ) * FBEAM / (4.0*PI)
         DO 40  IU = 1, NUMU
            SUM = 0.
            DO 30 IQ = MAZIM, NSTR-1
               SUM = SUM + YLMU(IQ,IU) *
     $              ( PSI0(IQ+1) + FACT * GL(IQ) * YLM0(IQ) )
 30         CONTINUE
            ZBEAM(IU) = SUM
 40      CONTINUE
      END IF

      IF ( PLANCK .AND. MAZIM.EQ.0 )  THEN

C                                ** THERMAL SOURCE TERMS, STWJ(27C)
         DO 80  IQ = MAZIM, NSTR-1
            PSUM0 = 0.0
            PSUM1 = 0.0
            DO 70  JQ = 1, NSTR
               PSUM0 = PSUM0 + CWT(JQ) * YLMC(IQ,JQ) * Z0(JQ)
               PSUM1 = PSUM1 + CWT(JQ) * YLMC(IQ,JQ) * Z1(JQ)
 70         CONTINUE
            PSI0(IQ+1) = 0.5 * GL(IQ) * PSUM0
            PSI1(IQ+1) = 0.5 * GL(IQ) * PSUM1
 80      CONTINUE

         DO 100  IU = 1, NUMU
            SUM0 = 0.0
            SUM1 = 0.0
            DO 90   IQ = MAZIM, NSTR-1
               SUM0 = SUM0 + YLMU(IQ,IU) * PSI0(IQ+1)
               SUM1 = SUM1 + YLMU(IQ,IU) * PSI1(IQ+1)
 90         CONTINUE
            Z0U(IU) = SUM0 + (1.-OPRIM) * XR0
            Z1U(IU) = SUM1 + (1.-OPRIM) * XR1
 100     CONTINUE
         
      END IF
      
      RETURN
      END
      LOGICAL FUNCTION  TSTBAD( VARNAM, RELERR )

C            Write name (-VARNAM-) of variable failing self-test and its
C                percent error from the correct value;  return  'FALSE'.

      CHARACTER*(*)  VARNAM
      REAL           RELERR

      TSTBAD = .FALSE.
      WRITE( *, '(/,3A,1P,E11.2,A)' )
     $       ' OUTPUT VARIABLE  ', VARNAM,'  DIFFERED BY', 100.*RELERR,
     $       '  PER CENT FROM CORRECT VALUE.  SELF-TEST FAILED.'

      RETURN
      END
      SUBROUTINE  UPBEAM( ARRAY, CC, CMU, DELM0, FBEAM, GL, IPVT, MAZIM,
     $     MXCMU, NN, NSTR, PI, UMU0, WK, YLM0, YLMC, ZJ,
     $     ZZ )

C         FINDS THE INCIDENT-BEAM PARTICULAR SOLUTION  OF SS(18)

C     ROUTINES CALLED:  SGECO, SGESL

C   I N P U T    V A R I A B L E S:

C       CC     :  CAPITAL-C-SUB-IJ IN EQ. SS(5)
C       CMU    :  ABSCISSAE FOR GAUSS QUADRATURE OVER ANGLE COSINE
C       DELM0  :  KRONECKER DELTA, DELTA-SUB-M0
C       GL     :  DELTA-M SCALED LEGENDRE COEFFICIENTS OF PHASE FUNCTION
C                    (INCLUDING FACTORS 2L+1 AND SINGLE-SCATTER ALBEDO)
C       MAZIM  :  ORDER OF AZIMUTHAL COMPONENT
C       YLM0   :  NORMALIZED ASSOCIATED LEGENDRE POLYNOMIAL
C                 AT THE BEAM ANGLE
C       YLMC   :  NORMALIZED ASSOCIATED LEGENDRE POLYNOMIAL
C                 AT THE QUADRATURE ANGLES
C       (REMAINDER ARE 'DISORT' INPUT VARIABLES)

C   O U T P U T    V A R I A B L E S:

C       ZJ     :  RIGHT-HAND SIDE VECTOR CAPITAL-X-SUB-ZERO IN SS(19);
C                 ALSO THE SOLUTION VECTOR CAPITAL-Z-SUB-ZERO
C                 AFTER SOLVING THAT SYSTEM
C       ZZ     :  PERMANENT STORAGE FOR -ZJ-, BUT RE-ORDERED

C   I N T E R N A L    V A R I A B L E S:

C       ARRAY  :  COEFFICIENT MATRIX IN LEFT-HAND SIDE OF EQ. SS(19)
C       IPVT   :  INTEGER VECTOR OF PIVOT INDICES REQUIRED BY *LINPACK*
C       WK     :  SCRATCH ARRAY REQUIRED BY *LINPACK*
C+---------------------------------------------------------------------+

      INTEGER  IPVT(*)
      REAL     ARRAY( MXCMU,* ), CC( MXCMU,* ), CMU(*), GL(0:*),
     $     WK(*), YLM0(0:*), YLMC( 0:MXCMU,* ), ZJ(*), ZZ(*)
      

      DO 40  IQ = 1, NSTR
         
         DO 10  JQ = 1, NSTR
            ARRAY(IQ,JQ) = - CC(IQ,JQ)
 10      CONTINUE
         ARRAY(IQ,IQ) = 1. + CMU(IQ) / UMU0 + ARRAY(IQ,IQ)

         SUM = 0.
         DO 20  K = MAZIM, NSTR-1
            SUM = SUM + GL(K) * YLMC(K,IQ) * YLM0(K)
 20      CONTINUE
         ZJ(IQ) = ( 2. - DELM0 ) * FBEAM * SUM / (4.0*PI)
 40   CONTINUE
C                  ** FIND L-U (LOWER/UPPER TRIANGULAR) DECOMPOSITION
C                  ** OF -ARRAY- AND SEE IF IT IS NEARLY SINGULAR
C                  ** (NOTE:  -ARRAY- IS DESTROYED)
      RCOND = 0.0
      CALL  SGECO( ARRAY, MXCMU, NSTR, IPVT, RCOND, WK )
      IF ( 1.0+RCOND .EQ. 1.0 )  CALL  ERRMSG
     $     ( 'UPBEAM--SGECO SAYS MATRIX NEAR SINGULAR',.FALSE.)

C                ** SOLVE LINEAR SYSTEM WITH COEFF MATRIX -ARRAY-
C                ** (ASSUMED ALREADY L-U DECOMPOSED) AND R.H. SIDE(S)
C                ** -ZJ-;  RETURN SOLUTION(S) IN -ZJ-
      JOB = 0
      CALL  SGESL( ARRAY, MXCMU, NSTR, IPVT, ZJ, JOB )
      
      DO 50  IQ = 1, NN
         ZZ( IQ+NN )   = ZJ( IQ )
         ZZ( NN+1-IQ ) = ZJ( IQ+NN )
 50   CONTINUE

      RETURN
      END
      SUBROUTINE  UPISOT( ARRAY, CC, CMU, IPVT, MXCMU, NN, NSTR, OPRIM,
     $     WK, XR0, XR1, XRA, Z0, Z1, ZPLK0, ZPLK1 )

C       FINDS THE PARTICULAR SOLUTION OF THERMAL RADIATION OF SS(15)

C     ROUTINES CALLED:  SGECO, SGESL

C   I N P U T     V A R I A B L E S:

C       CC     :  CAPITAL-C-SUB-IJ IN EQ. SS(5)
C       CMU    :  ABSCISSAE FOR GAUSS QUADRATURE OVER ANGLE COSINE
C       OPRIM  :  DELTA-M SCALED SINGLE SCATTERING ALBEDO
C       XR0    :  EXPANSION OF THERMAL SOURCE FUNCTION
C       XR1    :  EXPANSION OF THERMAL SOURCE FUNCTION
C       XRA    :  EXPANSION OF THERMAL SOURCE FUNCTION EQ. KS(15)
C       (REMAINDER ARE 'DISORT' INPUT VARIABLES)

C    O U T P U T    V A R I A B L E S:

C       Z0     :  SOLUTION VECTORS Z-SUB-ZERO OF EQ. SS(16)
C       Z1     :  SOLUTION VECTORS Z-SUB-ONE  OF EQ. SS(16)
C       ZPLK0, :  PERMANENT STORAGE FOR -Z0,Z1-, BUT RE-ORDERED
C        ZPLK1

C   I N T E R N A L    V A R I A B L E S:

C       ARRAY  :  COEFFICIENT MATRIX IN LEFT-HAND SIDE OF EQ. SS(16)
C       IPVT   :  INTEGER VECTOR OF PIVOT INDICES REQUIRED BY *LINPACK*
C       WK     :  SCRATCH ARRAY REQUIRED BY *LINPACK*
C+---------------------------------------------------------------------+

      INTEGER IPVT(*)
      REAL    ARRAY( MXCMU,* ), CC( MXCMU,* ), CMU(*), WK(*),
     $     Z0(*), Z1(*), ZPLK0(*), ZPLK1(*)

      DO 20 IQ = 1, NSTR

         DO 10 JQ = 1, NSTR
            ARRAY(IQ,JQ) = - CC(IQ,JQ)
 10      CONTINUE
	 ARRAY(IQ,IQ) = 1.0 +  XRA*CMU(IQ) + ARRAY(IQ,IQ)

         Z1(IQ) = (1.-OPRIM) * XR1
 20   CONTINUE
C                       ** SOLVE LINEAR EQUATIONS: SAME AS IN *UPBEAM*,
C                       ** EXCEPT -ZJ- REPLACED BY -Z0-
      RCOND = 0.0
      CALL  SGECO( ARRAY, MXCMU, NSTR, IPVT, RCOND, WK )
      IF ( 1.0+RCOND .EQ. 1.0 )  CALL  ERRMSG
     $     ( 'UPISOT--SGECO SAYS MATRIX NEAR SINGULAR',.FALSE.)

      CALL  SGESL( ARRAY, MXCMU, NSTR, IPVT, Z1, 0 )

      DO 30 IQ = 1, NSTR
         Z0(IQ) = (1.-OPRIM) * XR0 + CMU(IQ) * Z1(IQ)
 30   CONTINUE
      CALL  SGESL( ARRAY, MXCMU, NSTR, IPVT, Z0, 0 )

      DO 40  IQ = 1, NN
         ZPLK0( IQ+NN )   = Z0( IQ )
         ZPLK1( IQ+NN )   = Z1( IQ )
         ZPLK0( NN+1-IQ ) = Z0( IQ+NN )
         ZPLK1( NN+1-IQ ) = Z1( IQ+NN )
 40   CONTINUE

      RETURN
      END
      SUBROUTINE  USRINT( BPLANCK, CMU, CWT, DELM0, EMU, EXPBEA, FBEAM,
     $     FISOT, GC, GU, KK, LAMBER, LAYRU, LL, LYRCUT,
     $     MAZIM, MXCMU, MXUMU, NCUT, NLYR, NN, NSTR,
     $     PLANCK, NUMU, NTAU, PI, RMU, TAUCPR, TPLANCK,
     $     UMU, UMU0, UTAUPR, WK, XRA, ZBEAM, Z0U, Z1U, 
     $     ZZ, ZPLK0, ZPLK1, UUM )

C                     Compute intensity components at user output angles
C                             for azimuthal expansion terms in EQ. SD(2)

C                 I N P U T    V A R I A B L E S:

C       BPLANCK :  integrated Planck function for emission from
C                 bottom boundary
C       CMU    :  abscissae for Gauss quadrature over angle cosine
C       CWT    :  weights for Gauss quadrature over angle cosine
C       DELM0  :  Kronecker delta, delta-sub-M0
C       EMU    :  surface directional emissivity (user angles)
C       EXPBEA :  transmission of incident beam, EXP(-TAUCPR/UMU0)
C       GC     :  eigenvectors at polar quadrature angles, SC(1)
C       GU     :  eigenvectors interpolated to user polar angles
C                 (i.e., g in EQ. SC(1) )
C       KK     :  eigenvalues of coeff. matrix in EQ. SS(7)
C       LAYRU  :  layer number of user level -UTAU-
C       LL     :  constants of integration in EQ. SC(1), obtained
C                 by solving scaled version of EQ. SC(5);
C                 exponential term of EQ. SC(12) not included
C       LYRCUT :  logical flag for truncation of computational layer
C       MAZIM  :  order of azimuthal component
C       NCUT   :  total number of computational layers considered
C       NN     :  order of double-Gauss quadrature (NSTR/2)
C       RMU    :  surface bidirectional reflectivity (user angles)
C       TAUCPR :  cumulative optical depth (delta-M-Scaled)
C       TPLANCK :  integrated Planck function for emission from
C                 top boundary
C       UTAUPR :  optical depths of user output levels in delta-M
C                 coordinates;  equal to -UTAU- if no delta-M
C       Z0U    :  Y-sub-zero in EQ. KS(16) interpolated to user
C                 angles from an equation derived from KS(16)
C       Z1U    :  Y-sub-one in EQ. KS(16) interpolated to user
c                 angles from an equation derived from KS(16)
C       ZZ     :  beam source vectors in EQ. SS(19)
C       ZPLK0  :  thermal source vectors -Y0-, by solving EQ. KS(16)
C       ZPLK1  :  thermal source vectors -Y1-, by solving EQ. KS(16)
C       ZBEAM  :  incident-beam source vectors
C       (Remainder are 'disort' input variables)

C                 O U T P U T    V A R I A B L E S:

C       UUM    :  azimuthal components of the intensity in EQ. STWJ(5)

C                 I N T E R N A L    V A R I A B L E S:

C       BNDDIR :  direct intensity down at the bottom boundary
C       BNDDFU :  diffuse intensity down at the bottom boundary
C       BNDINT :  intensity attenuated at both boundaries, STWJ(25-6)
C       DTAU   :  optical depth of a computational layer
C       LYREND :  end layer of integration
C       LYRSTR :  start layer of integration
C       PALINT :  intensity component from parallel beam
C       PLKINT :  intensity component from planck source
C       WK     :  scratch vector for saving 'EXP' evaluations

C       All the exponential factors ( EXP1, EXPN,... etc.)
C       come from the substitution of constants of integration in
C       EQ. SC(12) into EQS. S1(8-9).  They all have negative
C       arguments so there should never be overflow problems.

C+---------------------------------------------------------------------+

      LOGICAL  LAMBER, LYRCUT, PLANCK, NEGUMU
      INTEGER  LAYRU( * )
      REAL     CMU( * ), CWT( * ), EMU( * ), EXPBEA( 0:* ),
     $     GC( MXCMU,MXCMU,* ), GU( MXUMU,MXCMU,* ), KK( MXCMU,* ),
     $     LL( MXCMU,* ), RMU( MXUMU,0:* ), TAUCPR( 0:* ),
     $     UUM( MXUMU,* ), UMU( * ), UTAUPR( * ), WK( * ), XRA(*),
     $     Z0U( MXUMU,* ), Z1U( MXUMU,* ), ZBEAM( MXUMU,* ),
     $     ZZ( MXCMU,* ), ZPLK0( MXCMU,* ), ZPLK1( MXCMU,* )

C    Incorporate constants of integration into interpolated eigenvectors

      DO 10 LC = 1, NCUT
         DO 10 IQ = 1, NSTR
            DO 10 IU = 1, NUMU
               GU(IU,IQ,LC) = GU(IU,IQ,LC) * LL(IQ,LC)
10    CONTINUE
C                                  Loop over levels at which intensities
C                                     are desired ('user output levels')
      DO 200 LU = 1, NTAU

         EXP0 = EXP( - UTAUPR(LU) / UMU0 )
         LYU = LAYRU(LU)

C                Loop over polar angles at which intensities are desired

         DO 100 IU = 1, NUMU

            IF ( LYRCUT .AND. LYU.GT.NCUT )  GO TO 100
            NEGUMU = UMU(IU).LT.0.0
            IF( NEGUMU )  THEN
               LYRSTR = 1
               LYREND = LYU - 1
               SGN = - 1.0
            ELSE
               LYRSTR = LYU + 1
               LYREND = NCUT
               SGN = 1.0
            END IF
            
C               For downward intensity, integrate from top to 'LYU-1' in
C       EQ. S1(8); for upward, integrate from bottom to 'LYU+1' in S1(9)

            PALINT = 0.0
            PLKINT = 0.0
            DO 30 LC = LYRSTR, LYREND
               
               DTAU = TAUCPR(LC) - TAUCPR(LC-1)
               EXP1 =  EXP( (UTAUPR(LU) - TAUCPR(LC-1)) / UMU(IU) )
               EXP2 =  EXP( (UTAUPR(LU) - TAUCPR( LC )) / UMU(IU) )
               
               IF ( PLANCK .AND. MAZIM.EQ.0 ) THEN
C              
C                Eqs. KS(56-57)
C
                  DENOM  =  SGN*1./(XRA(LC)*UMU(IU)+1.)
                  PLKINT = PLKINT +
     $                 (Z0U(iu,lc)*DENOM*
     $                 (EXP(-XRA(LC)*TAUCPR(LC-1)) *EXP1
     $                 -EXP(-XRA(LC)*TAUCPR(LC)) *EXP2 )
     $                 +Z1U(IU,LC)*DENOM*
     $                 ( (TAUCPR(LC-1)+SGN*DENOM*UMU(IU) )
     $                 *EXP(-XRA(LC)*TAUCPR(LC-1)) *EXP1
     $                 -(TAUCPR(LC)+SGN*DENOM*UMU(IU) )
     $                 *EXP(-XRA(LC)*TAUCPR(LC))* EXP2) )
               ENDIF

               IF ( FBEAM.GT.0.0 )  THEN
                  DENOM = 1.0 + UMU(IU) / UMU0
                  IF ( ABS(DENOM).LT.0.0001 )  THEN
C                                                       L'Hospital limit
                     EXPN = ( DTAU / UMU0 ) * EXP0
                  ELSE
                     EXPN = ( EXP1 * EXPBEA(LC-1) - EXP2 * EXPBEA(LC) )
     $                    * SGN / DENOM
                  END IF
                  PALINT = PALINT + ZBEAM(IU,LC) * EXPN
               END IF
C                                                          -KK- negative
               DO 20 IQ = 1, NN
                  WK(IQ) = EXP( KK(IQ,LC) * DTAU )
                  DENOM = 1.0 + UMU(IU) * KK(IQ,LC)
                  IF ( ABS(DENOM).LT.0.0001 )  THEN
C                                                       L'Hospital limit
                     EXPN = DTAU / UMU(IU) * EXP2
                  ELSE
                     EXPN = SGN * ( EXP1 * WK(IQ) - EXP2 ) / DENOM
                  END IF
                  PALINT = PALINT + GU(IU,IQ,LC) * EXPN
 20            CONTINUE
C                                                          -KK- positive
               DO 21 IQ = NN+1, NSTR
                  DENOM = 1.0 + UMU(IU) * KK(IQ,LC)
                  IF ( ABS(DENOM).LT.0.0001 )  THEN
C                                                       L'Hospital limit
                     EXPN = - DTAU / UMU(IU) * EXP1
                  ELSE
                     EXPN = SGN *( EXP1 - EXP2 * WK(NSTR+1-IQ) ) / DENOM
                  END IF
                  PALINT = PALINT + GU(IU,IQ,LC) * EXPN
 21            CONTINUE

 30         CONTINUE
C                                Calculate contribution from user output
C                                      level to next computational level

            DTAU1 = UTAUPR(LU) - TAUCPR(LYU-1)
            DTAU2 = UTAUPR(LU) - TAUCPR(LYU)
            IF( ABS(DTAU1).LT.1.E-6 .AND. NEGUMU )  GO TO 50
            IF( ABS(DTAU2).LT.1.E-6 .AND. (.NOT.NEGUMU) )  GO TO 50
            IF( NEGUMU ) EXP1 = EXP( DTAU1 / UMU(IU) )
            IF( .NOT.NEGUMU ) EXP2 = EXP( DTAU2 / UMU(IU) )

            IF ( FBEAM.GT.0.0 )  THEN
               DENOM = 1.0 + UMU(IU) / UMU0
               IF ( ABS(DENOM).LT.0.0001 )  THEN
                  EXPN =  ( DTAU1 / UMU0 ) * EXP0
               ELSE IF ( NEGUMU ) THEN
                  EXPN = ( EXP0 - EXPBEA(LYU-1) * EXP1 ) / DENOM
               ELSE
                  EXPN = ( EXP0 - EXPBEA(LYU) * EXP2 ) / DENOM
               END IF
               PALINT = PALINT + ZBEAM(IU,LYU) * EXPN
            ENDIF
C                                                          -KK- negative
            DTAU = TAUCPR(LYU) - TAUCPR(LYU-1)
            DO 40 IQ = 1, NN
               DENOM = 1.0 + UMU(IU) * KK(IQ,LYU)
               IF ( ABS(DENOM).LT.0.0001 )  THEN
                  EXPN = - DTAU2 / UMU(IU) * EXP2
               ELSE IF ( NEGUMU ) THEN
                  EXPN = ( EXP( - KK(IQ,LYU) * DTAU2 ) -
     $                 EXP( KK(IQ,LYU) * DTAU ) * EXP1 ) / DENOM
               ELSE
                  EXPN = ( EXP( - KK(IQ,LYU) * DTAU2 ) - EXP2 ) / DENOM
               END IF
               PALINT = PALINT + GU(IU,IQ,LYU) * EXPN
 40         CONTINUE
C                                                          -KK- positive
            DO 41 IQ = NN+1, NSTR
               
               DENOM = 1.0 + UMU(IU) * KK(IQ,LYU)
               IF ( ABS(DENOM).LT.0.0001 )  THEN
                  EXPN = - DTAU1 / UMU(IU) * EXP1
               ELSE IF ( NEGUMU ) THEN
                  EXPN = ( EXP(- KK(IQ,LYU) * DTAU1 ) - EXP1 ) / DENOM
               ELSE
                  EXPN = ( EXP( - KK(IQ,LYU) * DTAU1 ) -
     $                 EXP( - KK(IQ,LYU) * DTAU ) * EXP2 ) / DENOM
               END IF
               PALINT = PALINT + GU(IU,IQ,LYU) * EXPN
 41         CONTINUE

            IF ( PLANCK .AND. MAZIM.EQ.0 )  THEN
               IF ( NEGUMU )  THEN
                  EXPN = EXP1
                  ALFA = XRA(LYU)
                  DENOM = (-1./(ALFA*UMU(IU)+1.))
                  PLKINT = PLKINT + 
     $                 Z0U(IU,LYU)*DENOM*
     $                 (-EXP(-ALFA*UTAUPR(LU))
     $                 + EXPN*EXP(-ALFA*TAUCPR(LYU-1)) )
     $                 +  Z1U(IU,LYU)*DENOM*
     $                 ( -(UTAUPR(LU)-UMU(IU)*DENOM)*
     $                 EXP(-ALFA*UTAUPR(LU))
     $                 +(TAUCPR(LYU-1)-UMU(IU)*DENOM) 
     $                 *EXPN*EXP(-ALFA*TAUCPR(LYU-1)) )
               ELSE
                  EXPN = EXP2
                  ALFA = XRA(LYU)
                  denom = (1./(ALFA*UMU(IU)+1.))
                  PLKINT = PLKINT + 
     $                 Z0U(IU,LYU)*DENOM*
     $                 ( EXP(-ALFA*UTAUPR(LU))
     $                 -EXP(-ALFA*TAUCPR(LYU))*EXPN)
     $                 +  Z1U(IU,LYU)*DENOM*
     $                 ( (UTAUPR(LU) +UMU(IU)*DENOM)
     $                 *EXP(-ALFA*UTAUPR(LU))
     $                 -(TAUCPR(LYU)+UMU(IU)*DENOM)
     $                 *EXP(-ALFA*TAUCPR(LYU)) *EXPN )
               END IF
            END IF

C           Calculate intensity components attenuated at both boundaries
C           Note: no azimuthal intensity component for isotropic surface

 50         BNDINT = 0.0
            IF ( NEGUMU .AND. MAZIM.EQ.0 )  THEN
               BNDINT = ( FISOT + TPLANCK ) * EXP(UTAUPR(LU) / UMU(IU))
            ELSE IF ( .NOT.NEGUMU )  THEN
               IF ( LYRCUT .OR. (LAMBER .AND. MAZIM.GT.0) )  GO TO 90
               DO  JQ = NN+1, NSTR
                  WK(JQ) = EXP(-KK(JQ,NLYR)*
     $                 (TAUCPR(NLYR)-TAUCPR(NLYR-1)))
               ENDDO
               BNDDFU = 0.0
               DO 80 IQ = NN, 1, -1
                  DFUINT = 0.0
                  DO  JQ = 1, NN
                     DFUINT = DFUINT + GC(IQ,JQ,NLYR) * LL(JQ,NLYR)
                  ENDDO
                  DO  JQ = NN+1, NSTR
                     DFUINT = DFUINT + 
     $                    GC(IQ,JQ,NLYR) * LL(JQ,NLYR) * WK(JQ)
                  ENDDO
                  IF ( FBEAM.GT.0.0 )
     $                 DFUINT = DFUINT + ZZ(IQ,NLYR) * EXPBEA(NLYR)
                  DFUINT = DFUINT + DELM0 * ( 
     $                 EXP(-XRA(NLYR)*TAUCPR(NLYR))*
     $                 (ZPLK0(IQ,NLYR)+
     $                 ZPLK1(IQ,NLYR)*TAUCPR(NLYR)))
                  BNDDFU = BNDDFU + ( 1. + DELM0 ) * RMU(IU,NN+1-IQ)
     $                 * CMU(NN+1-IQ) * CWT(NN+1-IQ) * DFUINT
 80            CONTINUE

               BNDDIR = 0.0
               IF (FBEAM.GT.0.0) BNDDIR = UMU0*FBEAM/PI * RMU(IU,0)
     $              * EXPBEA(NLYR)
               BNDINT = ( BNDDFU + BNDDIR + DELM0 * EMU(IU) * BPLANCK )
     $              * EXP( (UTAUPR(LU)-TAUCPR(NLYR)) / UMU(IU) )
            END IF

90          UUM(IU,LU) = PALINT + PLKINT + BNDINT

 100     CONTINUE
 200  CONTINUE

      RETURN
      END
      REAL FUNCTION  XIFUNC( UMU1, UMU2, UMU3, TAU )
 
C                                            Xi function of EQ. STW (72)
 
C                    I N P U T   V A R I A B L E S
 
C        TAU         optical thickness of the layer
C        UMU1,2,3    cosine of zenith angle_1, _2, _3
 
      X1 = 1./UMU1 - 1./UMU2
      X2 = 1./UMU1 - 1./UMU3
 
      IF ( (UMU2.EQ.UMU3) .AND. (UMU1.EQ.UMU2) )  THEN
         XIFUNC = TAU**2*EXP(-TAU/UMU1) / ( 2.*UMU1*UMU2 )
 
      ELSE IF ( (UMU2.EQ.UMU3) .AND. (UMU1.NE.UMU2) )  THEN
         XIFUNC = ( (TAU-1./X1)*EXP(-TAU/UMU2) + EXP(-TAU/UMU1)/X1 ) /
     $            ( X1*UMU1*UMU2 )
 
      ELSE IF ( (UMU2.NE.UMU3) .AND. (UMU1.EQ.UMU2) )  THEN
         XIFUNC = ( ( EXP(-TAU/UMU3) - EXP(-TAU/UMU1) )/X2
     $              - TAU*EXP(-TAU/UMU1) ) / ( X2*UMU1*UMU2 )
 
      ELSE IF ( (UMU2.NE.UMU3) .AND. (UMU1.EQ.UMU3) )  THEN
         XIFUNC = ( ( EXP(-TAU/UMU2) - EXP(-TAU/UMU1) )/X1
     $              - TAU*EXP(-TAU/UMU1) ) / ( X1*UMU1*UMU2 )
 
      ELSE
         XIFUNC = ( ( EXP(-TAU/UMU3) - EXP(-TAU/UMU1) )/X2 -
     $              ( EXP(-TAU/UMU2) - EXP(-TAU/UMU1) )/X1 ) /
     $            ( X2*UMU1*UMU2 )
      END IF
 
      RETURN
      END
      SUBROUTINE  ZEROAL( ND1, EXPBEA, FLYR, OPRIM, PHASA, PHASE, PHASM,
     $     TAUCPR, XR0, XR1, XRA,
     $     ND2, CMU, CWT, PSI0, PSI1, WK, Z0, Z1, ZJ,
     $     ND3, HLPR, YLM0,
     $     ND4, ARRAY, CC, EVECC,
     $     ND5, GL,
     $     ND6, YLMC,
     $     ND7, YLMU,
     $     ND8, KK, LL, ZZ, ZPLK0, ZPLK1,
     $     ND9, GC,
     $     ND10, LAYRU, UTAUPR,
     $     ND11, GU,
     $     ND12, Z0U, Z1U, ZBEAM,
     $     ND13, EVAL,
     $     ND14, AMB, APB,
     $     ND15, IPVT, Z )

C                                                            Zero arrays

      INTEGER  IPVT( * ), LAYRU( * )
      REAL  AMB( * ), APB( * ), ARRAY( * ), CC( * ), CMU( * ), CWT( * ),
     $     EVAL( * ), EVECC( * ), EXPBEA( * ), FLYR( * ), GC( * ),
     $     GL( * ), GU( * ), HLPR( * ), KK( * ), LL( * ), OPRIM( * ),
     $     PHASA( * ), PHASE( * ), PHASM( * ), PSI0( * ), PSI1( * ),
     $     TAUCPR( * ), UTAUPR( * ), WK( * ), XR0( * ), XR1( * ), 
     $     XRA( * ), YLM0( * ), YLMC( * ), YLMU( * ), Z( * ), Z0( * ), 
     $     Z0U( * ), Z1( * ), Z1U( * ), ZBEAM( * ), ZJ( * ),
     $     ZZ( * ), ZPLK0( * ), ZPLK1( * )
      
      DO 10 N = 1, ND1
         EXPBEA(N)= 0.0
         FLYR(N)  = 0.0
         OPRIM(N) = 0.0
         PHASA(N) = 0.0
         PHASE(N) = 0.0
         PHASM(N) = 0.0
         TAUCPR(N)= 0.0
         XR0(N)   = 0.0
         XR1(N)   = 0.0
         XRA(N)   = 0.0
 10   CONTINUE
      
      DO 20 N = 1, ND2
         CMU(N) = 0.0
         CWT(N) = 0.0
         PSI0(N) = 0.0
         PSI1(N) = 0.0
         WK(N)  = 0.0
         Z0(N)  = 0.0
         Z1(N)  = 0.0
         ZJ(N)  = 0.0
 20   CONTINUE
      
      DO 30 N = 1, ND3
         HLPR(N) = 0.0
         YLM0(N) = 0.0
 30   CONTINUE

      DO 40 N = 1, ND4
         ARRAY(N) = 0.0
         CC(N)    = 0.0
         EVECC(N) = 0.0
 40   CONTINUE

      DO 50 N = 1, ND5
         GL(N) = 0.0
 50   CONTINUE   
     
      DO 60 N = 1, ND6
         YLMC(N) = 0.0
 60   CONTINUE   

      DO 70 N = 1, ND7
         YLMU(N) = 0.0
 70   CONTINUE   

      DO 80 N = 1, ND8
         KK(N) = 0.0
         LL(N) = 0.0
         ZZ(N) = 0.0
         ZPLK0(N) = 0.0
         ZPLK1(N) = 0.0
80    CONTINUE

      DO 90 N = 1, ND9
         GC(N) = 0.0
 90   CONTINUE

      DO 100 N = 1, ND10
         LAYRU(N)  = 0
         UTAUPR(N) = 0.0
 100  CONTINUE

      DO 110 N = 1, ND11
         GU(N) = 0.0
 110  CONTINUE

      DO 120 N = 1, ND12
         Z0U(N) = 0.0
         Z1U(N) = 0.0
         ZBEAM(N) = 0.0
 120  CONTINUE
      
      DO 130 N = 1, ND13
         EVAL(N) = 0.0
 130  CONTINUE

      DO 140 N = 1, ND14
         AMB(N) = 0.0
         APB(N) = 0.0
 140  CONTINUE

      DO 150 N = 1, ND15
         IPVT(N) = 0
         Z(N) = 0.0
 150  CONTINUE

      RETURN
      END
